import { __spread, __decorate, __param, __assign } from 'tslib';
import { InjectionToken, ɵresetCompiledComponents, NgModule, Component, NgZone, Inject, Input, ElementRef, Directive, Injectable, ChangeDetectorRef, ViewChildren, Pipe, EventEmitter, Output } from '@angular/core';
import { BrowserModule, EventManager } from '@angular/platform-browser';
import { Location, CommonModule, LocationStrategy, PathLocationStrategy } from '@angular/common';
import { FormControl, ReactiveFormsModule } from '@angular/forms';
import { platformBrowserDynamic } from '@angular/platform-browser-dynamic';
import { takeUntil, debounceTime, distinctUntilChanged } from 'rxjs/operators';
import { Subject, Observable, BehaviorSubject } from 'rxjs';

// This file is filled in via the Playground CLI.
// Empty functions are the expected initial value.
function getSandbox(path) {
    /*GET_SANDBOX*/
    /*END_GET_SANDBOX*/
}
function getSandboxMenuItems() {
    /*GET_SANDBOX_MENU_ITEMS*/
    return [];
    /*END_GET_SANDBOX_MENU_ITEMS*/
}

var SandboxLoader = /** @class */ (function () {
    function SandboxLoader() {
    }
    SandboxLoader.loadSandbox = function (path) {
        return getSandbox(path);
    };
    SandboxLoader.getSandboxMenuItems = function () {
        return getSandboxMenuItems();
    };
    return SandboxLoader;
}());

var MIDDLEWARE = new InjectionToken('middleware');

var ScenarioComponent = /** @class */ (function () {
    function ScenarioComponent(zone, middleware) {
        this.zone = zone;
        this.middleware = middleware;
        /**
         * Collection of bootstrapped apps
         */
        this.activeApps = [];
        /**
         * Unsubscribe all subscriptions on component destroy
         */
        this.onDestroy = new Subject();
    }
    ScenarioComponent.prototype.ngOnInit = function () {
        var _this = this;
        this.middleware
            .pipe(takeUntil(this.onDestroy))
            .subscribe(function (middlewares) { return _this.activeMiddleware = middlewares; });
    };
    ScenarioComponent.prototype.ngOnChanges = function (changes) {
        if (changes.selectedSandboxAndScenarioKeys) {
            this.bootstrapSandbox(changes.selectedSandboxAndScenarioKeys.currentValue);
        }
    };
    ScenarioComponent.prototype.ngOnDestroy = function () {
        this.onDestroy.next();
        // this cleans up the DOM when a sandboxed component is loaded and the search bar is then cleared
        if (this.activeApps.length > 0) {
            // destroy sandboxed app (doesn't remove it from the DOM though)
            var app = this.activeApps.pop();
            app.destroy();
            // remove the sandboxed component's element from the dom
            var hostElement = document.querySelector('playground-host');
            if (hostElement && hostElement.children) {
                hostElement.children[0].remove();
            }
        }
    };
    /**
     * Bootstrap a new Angular application with the sandbox's required dependencies
     */
    ScenarioComponent.prototype.bootstrapSandbox = function (selectedSandboxAndScenarioKeys) {
        var _this = this;
        SandboxLoader.loadSandbox(selectedSandboxAndScenarioKeys.sandboxKey).then(function (sandbox) {
            if (sandbox) {
                var scenario_1 = sandbox.scenarios
                    .find(function (s) { return s.key === selectedSandboxAndScenarioKeys.scenarioKey; });
                if (scenario_1) {
                    if (_this.activeApps.length > 0) {
                        var app = _this.activeApps.pop();
                        app.destroy();
                        ɵresetCompiledComponents();
                    }
                    // Don't bootstrap a new Angular application within an existing zone
                    _this.zone.runOutsideAngular(function () {
                        var module = _this.createModule(sandbox, scenario_1);
                        platformBrowserDynamic().bootstrapModule(module)
                            .then(function (app) {
                            _this.activeApps.push(app);
                            window.isPlaygroundComponentLoaded = function () { return true; };
                        })
                            .catch(function (err) {
                            window.isPlaygroundComponentLoadedWithErrors = function () { return true; };
                            console.error(err);
                        });
                    });
                }
            }
        });
    };
    /**
     * Create a module containing the dependencies of a sandbox
     */
    ScenarioComponent.prototype.createModule = function (sandboxMeta, scenario) {
        var hostComp = this.createComponent(scenario);
        var DynamicModule = /** @class */ (function () {
            function DynamicModule() {
            }
            DynamicModule.prototype.ngDoBootstrap = function (app) {
                var hostEl = document.querySelector('playground-host');
                if (!hostEl) {
                    var compEl = document.createElement('playground-host');
                    document.body.appendChild(compEl);
                }
                app.bootstrap(hostComp);
            };
            return DynamicModule;
        }());
        return NgModule({
            imports: __spread([
                BrowserModule
            ], sandboxMeta.imports, this.activeMiddleware.modules),
            declarations: __spread([
                hostComp,
                sandboxMeta.declareComponent ? sandboxMeta.type : []
            ], sandboxMeta.declarations),
            providers: __spread(sandboxMeta.providers),
            entryComponents: __spread([hostComp], sandboxMeta.entryComponents),
            schemas: __spread(sandboxMeta.schemas),
        })(DynamicModule);
    };
    /**
     * Construct a component to serve as the host for the provided scenario
     */
    ScenarioComponent.prototype.createComponent = function (scenario) {
        var DynamicComponent = /** @class */ (function () {
            function DynamicComponent() {
                Object.assign(this, scenario.context);
            }
            return DynamicComponent;
        }());
        return Component({
            selector: 'playground-host',
            template: scenario.template,
            styles: scenario.styles,
            providers: scenario.providers,
        })(DynamicComponent);
    };
    ScenarioComponent.ctorParameters = function () { return [
        { type: NgZone },
        { type: undefined, decorators: [{ type: Inject, args: [MIDDLEWARE,] }] }
    ]; };
    __decorate([
        Input()
    ], ScenarioComponent.prototype, "selectedSandboxAndScenarioKeys", void 0);
    ScenarioComponent = __decorate([
        Component({
            selector: 'ap-scenario',
            template: "\n    <ng-template></ng-template>"
        }),
        __param(1, Inject(MIDDLEWARE))
    ], ScenarioComponent);
    return ScenarioComponent;
}());

var StateService = /** @class */ (function () {
    function StateService() {
        var _this = this;
        this.filterKey = 'angularPlayground.filter';
        this.filter = sessionStorage.getItem(this.filterKey);
        sessionStorage.removeItem(this.filterKey);
        var beforeUnload = function () {
            sessionStorage.setItem(_this.filterKey, emptyStringIfNull(_this.filter));
            return 'unload';
        };
        window.addEventListener('beforeunload', beforeUnload);
    }
    StateService.prototype.getFilter = function () {
        return this.filter;
    };
    StateService.prototype.setFilter = function (value) {
        this.filter = value;
    };
    return StateService;
}());
function emptyStringIfNull(value) {
    return value ? value : '';
}

var FocusDirective = /** @class */ (function () {
    function FocusDirective(elementRef) {
        this.elementRef = elementRef;
    }
    Object.defineProperty(FocusDirective.prototype, "apFocus", {
        set: function (value) {
            if (value) {
                this.elementRef.nativeElement.focus();
            }
        },
        enumerable: true,
        configurable: true
    });
    FocusDirective.ctorParameters = function () { return [
        { type: ElementRef }
    ]; };
    __decorate([
        Input()
    ], FocusDirective.prototype, "apFocus", null);
    FocusDirective = __decorate([
        Directive({
            selector: '[apFocus]',
        })
    ], FocusDirective);
    return FocusDirective;
}());

var UrlService = /** @class */ (function () {
    function UrlService(location) {
        this.location = location;
        this._embed = null;
        this._select = null;
        this.sandboxMenuItems = SandboxLoader.getSandboxMenuItems();
        var urlPath = location.path();
        this._embed = /[?|&]embed=1/.exec(urlPath) !== null;
        this._select = this.parse('scenario', this.sandboxMenuItems, urlPath);
        if (this._select) {
            if (this._select.sandboxKey === null && this._select.scenarioKey === null) {
                this.location.replaceState('');
                return;
            }
        }
    }
    Object.defineProperty(UrlService.prototype, "embed", {
        get: function () {
            return this._embed;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(UrlService.prototype, "select", {
        get: function () {
            return this._select;
        },
        enumerable: true,
        configurable: true
    });
    UrlService.prototype.setSelected = function (sandboxKey, scenarioKey) {
        if (sandboxKey === null && scenarioKey === null) {
            this.location.replaceState('');
            return;
        }
        var sandBoxMenuItem = this.sandboxMenuItems
            .find(function (sandboxMenuItem) { return sandboxMenuItem.key.toLowerCase() === sandboxKey.toLowerCase(); });
        var scenarioMenuItem = sandBoxMenuItem.scenarioMenuItems.find(function (mi) { return mi.key === scenarioKey; });
        var key = sandBoxMenuItem.uniqueId ? sandBoxMenuItem.uniqueId : sandboxKey;
        this.location.replaceState("?scenario=" + encodeURIComponent(key) + "/" + encodeURIComponent(scenarioMenuItem.description));
    };
    UrlService.prototype.parse = function (key, sandboxMenuItems, urlPath) {
        var match = new RegExp('[?|&]' + key + '=([^&#]*)').exec(urlPath);
        if (match !== null) {
            var value = match[1];
            var firstSlash = value.indexOf('/');
            var sbKey = value.substr(0, firstSlash);
            var sandboxKey_1 = decodeURIComponent(sbKey);
            var sandboxMenuItem = sandboxMenuItems
                .find(function (smi) { return smi.key.toLowerCase() === sandboxKey_1.toLowerCase()
                || (smi.uniqueId && smi.uniqueId === sandboxKey_1); });
            sandboxKey_1 = sandboxMenuItem.key;
            if (!sandboxMenuItem) {
                return { sandboxKey: null, scenarioKey: null };
            }
            var scenarioDesc_1 = decodeURIComponent(value.substr(firstSlash + 1, value.length).toLowerCase());
            var scenarioKey = sandboxMenuItem.scenarioMenuItems
                .findIndex(function (scenarioMenuItem) { return scenarioMenuItem.description.toLowerCase() === scenarioDesc_1; }) + 1;
            if (scenarioKey <= 0) {
                return { sandboxKey: null, scenarioKey: null };
            }
            return { sandboxKey: sandboxKey_1, scenarioKey: scenarioKey };
        }
    };
    UrlService.ctorParameters = function () { return [
        { type: Location }
    ]; };
    UrlService = __decorate([
        Injectable()
    ], UrlService);
    return UrlService;
}());

// https://github.com/bevacqua/fuzzysearch
function fuzzySearch(needle, haystack) {
    var indexes = [];
    var hlen = haystack.length;
    var nlen = needle.length;
    if (nlen > hlen) {
        return null;
    }
    if (nlen === hlen) {
        return (needle === haystack)
            ? Array.apply(null, { length: nlen }).map(Number.call, Number)
            : null;
    }
    outer: for (var i = 0, j = 0; i < nlen; i++) {
        var nch = needle.charCodeAt(i);
        while (j < hlen) {
            if (haystack.charCodeAt(j++) === nch) {
                indexes.push(j - 1);
                continue outer;
            }
        }
        return null;
    }
    return indexes;
}

// https://github.com/gustf/js-levenshtein
var LevenshteinDistance = /** @class */ (function () {
    function LevenshteinDistance() {
    }
    LevenshteinDistance.prototype.getDistance = function (a, b) {
        if (a === b) {
            return 0;
        }
        if (a.length > b.length) {
            var tmp = a;
            a = b;
            b = tmp;
        }
        var la = a.length;
        var lb = b.length;
        while (la > 0 && (a.charCodeAt(la - 1) === b.charCodeAt(lb - 1))) {
            la--;
            lb--;
        }
        var offset = 0;
        while (offset < la && (a.charCodeAt(offset) === b.charCodeAt(offset))) {
            offset++;
        }
        la -= offset;
        lb -= offset;
        if (la === 0 || lb === 1) {
            return lb;
        }
        var x;
        var y;
        var d0;
        var d1;
        var d2;
        var d3;
        var dd;
        var dy;
        var ay;
        var bx0;
        var bx1;
        var bx2;
        var bx3;
        // tslint:disable-next-line:no-bitwise
        var vector = new Array(la << 1);
        for (y = 0; y < la;) {
            vector[la + y] = a.charCodeAt(offset + y);
            vector[y] = ++y;
        }
        for (x = 0; (x + 3) < lb;) {
            bx0 = b.charCodeAt(offset + (d0 = x));
            bx1 = b.charCodeAt(offset + (d1 = x + 1));
            bx2 = b.charCodeAt(offset + (d2 = x + 2));
            bx3 = b.charCodeAt(offset + (d3 = x + 3));
            dd = (x += 4);
            for (y = 0; y < la;) {
                ay = vector[la + y];
                dy = vector[y];
                d0 = this.minimum(dy, d0, d1, bx0, ay);
                d1 = this.minimum(d0, d1, d2, bx1, ay);
                d2 = this.minimum(d1, d2, d3, bx2, ay);
                dd = this.minimum(d2, d3, dd, bx3, ay);
                vector[y++] = dd;
                d3 = d2;
                d2 = d1;
                d1 = d0;
                d0 = dy;
            }
        }
        for (; x < lb;) {
            bx0 = b.charCodeAt(offset + (d0 = x));
            dd = ++x;
            for (y = 0; y < la; y++) {
                dy = vector[y];
                vector[y] = dd = dy < d0 || dd < d0
                    ? dy > dd ? dd + 1 : dy + 1
                    : bx0 === vector[la + y]
                        ? d0
                        : d0 + 1;
                d0 = dy;
            }
        }
        return dd;
    };
    LevenshteinDistance.prototype.minimum = function (d0, d1, d2, bx, ay) {
        return d0 < d1 || d2 < d1
            ? d0 > d2
                ? d2 + 1
                : d0 + 1
            : bx === ay
                ? d1
                : d1 + 1;
    };
    return LevenshteinDistance;
}());

var AppComponent = /** @class */ (function () {
    function AppComponent(stateService, urlService, eventManager, levenshteinDistance, changeDetectorRef, middleware) {
        this.stateService = stateService;
        this.urlService = urlService;
        this.eventManager = eventManager;
        this.levenshteinDistance = levenshteinDistance;
        this.changeDetectorRef = changeDetectorRef;
        this.middleware = middleware;
        this.commandBarActive = false;
        this.commandBarPreview = false;
        this.selectedSandboxAndScenarioKeys = { sandboxKey: null, scenarioKey: null };
        this.filter = new FormControl();
        this.shortcuts = this.getShortcuts();
        this.embed = false;
    }
    AppComponent.prototype.ngOnInit = function () {
        var _this = this;
        var sandboxMenuItems = SandboxLoader.getSandboxMenuItems();
        this.middleware
            .subscribe(function (middleware) { return _this.activeMiddleware = middleware; });
        this.embed = this.urlService.embed;
        if (this.embed) {
            this.selectedSandboxAndScenarioKeys = {
                sandboxKey: this.urlService.select.sandboxKey,
                scenarioKey: this.urlService.select.scenarioKey,
            };
        }
        else {
            if (this.urlService.select) {
                this.selectedSandboxAndScenarioKeys = {
                    sandboxKey: this.urlService.select.sandboxKey,
                    scenarioKey: this.urlService.select.scenarioKey,
                };
            }
            this.eventManager.addGlobalEventListener('window', 'keydown.control.p', this.blockEvent);
            this.eventManager.addGlobalEventListener('window', 'keydown.F2', this.blockEvent);
            this.eventManager.addGlobalEventListener('window', 'keyup.control.p', function (event) {
                _this.blockEvent(event);
                _this.toggleCommandBar();
            });
            this.eventManager.addGlobalEventListener('window', 'keyup.F2', function (event) {
                _this.blockEvent(event);
                _this.toggleCommandBar();
            });
            var filterValue = this.stateService.getFilter();
            this.totalSandboxes = sandboxMenuItems.length;
            this.filteredSandboxMenuItems = this.filterSandboxes(sandboxMenuItems, filterValue);
            this.filter.setValue(filterValue);
            this.filter.valueChanges.pipe(debounceTime(300), distinctUntilChanged())
                .subscribe(function (value) {
                _this.stateService.setFilter(value);
                _this.filteredSandboxMenuItems = _this.filterSandboxes(sandboxMenuItems, value);
                if (!value) {
                    _this.selectScenario(null, null);
                }
            });
        }
        // expose select scenario functionality for visual regression test
        window.loadScenario = function (sandboxKey, scenarioKey) {
            _this.selectScenario(sandboxKey, scenarioKey);
            _this.changeDetectorRef.detectChanges();
        };
        // set flag to check when component is loaded
        window.isPlaygroundComponentLoaded = function () { return false; };
        window.isPlaygroundComponentLoadedWithErrors = function () { return false; };
    };
    AppComponent.prototype.onFilterBoxArrowDown = function (event, switchToScenario) {
        if (switchToScenario === void 0) { switchToScenario = false; }
        event.preventDefault();
        var elementRef;
        var currentIndex = this.findCurrentScenarioIndex();
        if (currentIndex) {
            elementRef = this.focusScenarioLinkElement(currentIndex + 1);
        }
        else {
            elementRef = this.focusScenarioLinkElement(0);
        }
        if (switchToScenario) {
            this.showScenario(elementRef);
        }
    };
    AppComponent.prototype.onFilterBoxArrowUp = function (event, switchToScenario) {
        if (switchToScenario === void 0) { switchToScenario = false; }
        event.preventDefault();
        var elementRef;
        var currentIndex = this.findCurrentScenarioIndex();
        if (currentIndex) {
            elementRef = this.focusScenarioLinkElement(currentIndex - 1);
        }
        else if (this.scenarioLinkElements.length > 0) {
            elementRef = this.focusScenarioLinkElement(this.scenarioLinkElements.length - 1);
        }
        if (switchToScenario) {
            this.showScenario(elementRef);
        }
    };
    AppComponent.prototype.onScenarioLinkKeyDown = function (scenarioElement, filterElement, event) {
        event.preventDefault();
        switch (event.key) {
            case 'Up':
            case 'ArrowUp':
                this.goUp(scenarioElement);
                break;
            case 'Down':
            case 'ArrowDown':
                this.goDown(scenarioElement);
                break;
            default:
                if (event.key !== 'Escape' && event.key !== 'Enter') {
                    if (event.key.length === 1) {
                        this.filter.setValue("" + this.filter.value + event.key);
                    }
                    filterElement.focus();
                }
                break;
        }
    };
    AppComponent.prototype.onScenarioLinkKeyUp = function (scenarioElement, event) {
        event.preventDefault();
        switch (event.key) {
            case 'Escape':
                this.commandBarActive = false;
                break;
            case 'Enter':
                scenarioElement.click();
                break;
        }
    };
    AppComponent.prototype.onScenarioLinkControlDown = function (scenarioElement, event) {
        if (!this.commandBarActive)
            return;
        event.preventDefault();
        var elementRef = this.goDown(scenarioElement);
        this.showScenario(elementRef);
    };
    AppComponent.prototype.onScenarioLinkControlUp = function (scenarioElement, event) {
        if (!this.commandBarActive)
            return;
        event.preventDefault();
        var elementRef = this.goUp(scenarioElement);
        this.showScenario(elementRef);
    };
    AppComponent.prototype.onCommandBarStartPreview = function (event) {
        event.preventDefault();
        this.commandBarPreview = true;
    };
    AppComponent.prototype.onCommandBarStopPreview = function () {
        this.commandBarPreview = false;
    };
    AppComponent.prototype.onScenarioClick = function (sandboxKey, scenarioKey, event) {
        event.preventDefault();
        this.selectScenario(sandboxKey, scenarioKey);
    };
    AppComponent.prototype.isSelected = function (sandbox, scenario) {
        return this.selectedSandboxAndScenarioKeys.scenarioKey === scenario.key
            && this.selectedSandboxAndScenarioKeys.sandboxKey.toLowerCase() === sandbox.key.toLowerCase();
    };
    AppComponent.prototype.toggleCommandBar = function () {
        this.commandBarActive = !this.commandBarActive;
    };
    AppComponent.prototype.blockEvent = function (e) {
        e.preventDefault();
        e.stopPropagation();
    };
    AppComponent.prototype.showScenario = function (selectedScenarioElementRef) {
        if (selectedScenarioElementRef) {
            this.selectScenario(selectedScenarioElementRef.nativeElement.getAttribute('sandboxMenuItemKey'), parseInt(selectedScenarioElementRef.nativeElement.getAttribute('scenarioMenuItemkey'), 10));
        }
    };
    AppComponent.prototype.findCurrentScenarioIndex = function () {
        var currentIndex;
        if (this.scenarioLinkElements.length > 0) {
            this.scenarioLinkElements.map(function (element, i) {
                if (element.nativeElement.className.includes('selected')) {
                    currentIndex = i;
                }
            });
        }
        return currentIndex;
    };
    AppComponent.prototype.goUp = function (scenarioElement) {
        var currentIndex = -1;
        this.scenarioLinkElements.forEach(function (scenarioElementRef, index) {
            if (scenarioElementRef.nativeElement === scenarioElement) {
                currentIndex = index;
            }
        });
        if (currentIndex === 0) {
            return this.focusScenarioLinkElement(this.scenarioLinkElements.length - 1);
        }
        else {
            return this.focusScenarioLinkElement(currentIndex - 1);
        }
    };
    AppComponent.prototype.goDown = function (scenarioElement) {
        var currentIndex = -1;
        this.scenarioLinkElements.forEach(function (scenarioElementRef, index) {
            if (scenarioElementRef.nativeElement === scenarioElement) {
                currentIndex = index;
            }
        });
        if (currentIndex === this.scenarioLinkElements.length - 1) {
            return this.focusScenarioLinkElement(0);
        }
        else {
            return this.focusScenarioLinkElement(currentIndex + 1);
        }
    };
    AppComponent.prototype.focusScenarioLinkElement = function (index) {
        if (this.scenarioLinkElements.toArray()[index]) {
            var elementRef = this.scenarioLinkElements.toArray()[index];
            elementRef.nativeElement.focus();
            return elementRef;
        }
    };
    AppComponent.prototype.filterSandboxes = function (sandboxMenuItems, filter) {
        var _this = this;
        if (!filter) {
            return sandboxMenuItems.map(function (item, i) { return Object.assign({}, item, { tabIndex: i }); });
        }
        var tabIndex = 0;
        var filterNormalized = filter.toLowerCase();
        return sandboxMenuItems
            .reduce(function (accum, curr) {
            var searchKeyNormalized = curr.searchKey.toLowerCase();
            var indexMatches = fuzzySearch(filterNormalized, searchKeyNormalized);
            if (indexMatches) {
                var weight = _this.levenshteinDistance.getDistance(filterNormalized, searchKeyNormalized);
                return __spread(accum, [Object.assign({}, curr, { weight: weight, indexMatches: indexMatches })]);
            }
            else {
                return accum;
            }
        }, [])
            .sort(function (a, b) {
            return a.weight - b.weight;
        })
            .map(function (sandboxMenuItem) { return Object.assign({}, sandboxMenuItem, { tabIndex: tabIndex++ }); });
    };
    AppComponent.prototype.selectScenario = function (sandboxKey, scenarioKey) {
        // set flag to check when component is loaded
        window.isPlaygroundComponentLoaded = function () { return false; };
        window.isPlaygroundComponentLoadedWithErrors = function () { return false; };
        this.selectedSandboxAndScenarioKeys = { sandboxKey: sandboxKey, scenarioKey: scenarioKey };
        this.urlService.setSelected(sandboxKey, scenarioKey);
    };
    AppComponent.prototype.getShortcuts = function () {
        return [
            {
                keys: ['ctrl + p', 'f2'],
                description: 'Toggle command bar open/closed',
            },
            {
                keys: ['esc'],
                description: 'Close command bar',
            },
            {
                keys: ['\u2191', '\u2193'],
                description: 'Navigate up or down in command bar list',
            },
            {
                keys: ['ctrl + \u2191', 'ctrl + \u2193'],
                description: 'Switch scenarios while navigating up or down in command bar list',
            },
        ];
    };
    AppComponent.ctorParameters = function () { return [
        { type: StateService },
        { type: UrlService },
        { type: EventManager },
        { type: LevenshteinDistance },
        { type: ChangeDetectorRef },
        { type: Observable, decorators: [{ type: Inject, args: [MIDDLEWARE,] }] }
    ]; };
    __decorate([
        ViewChildren('scenarioElement')
    ], AppComponent.prototype, "scenarioLinkElements", void 0);
    AppComponent = __decorate([
        Component({
            selector: 'playground-root',
            template: "<div class=\"shield\" *ngIf=\"commandBarActive\" (click)=\"toggleCommandBar()\"></div>\n<div class=\"command-bar\" *ngIf=\"filteredSandboxMenuItems\"\n     (keydown.control)=\"onCommandBarStartPreview($event)\"\n     (keyup.control)=\"onCommandBarStopPreview()\"\n     [class.command-bar--open]=\"commandBarActive\"\n     [class.command-bar--preview]=\"commandBarPreview\">\n    <input\n        class=\"command-bar__filter\"\n        type=\"text\"\n        name=\"filter\"\n        placeholder=\"search...\"\n        [formControl]=\"filter\"\n        #filterElement\n        [apFocus]=\"commandBarActive\"\n        (keyup.Esc)=\"commandBarActive = false\"\n        (keydown.arrowup)=\"onFilterBoxArrowUp($event)\"\n        (keydown.arrowdown)=\"onFilterBoxArrowDown($event)\"\n        (keydown.control.arrowup)=\"onFilterBoxArrowUp($event, true)\"\n        (keydown.control.arrowdown)=\"onFilterBoxArrowDown($event, true)\">\n    <div *ngIf=\"filteredSandboxMenuItems && filteredSandboxMenuItems.length > 0\" class=\"command-bar__sandboxes\">\n        <div class=\"command-bar__sandbox\"\n             *ngFor=\"let sandboxMenuItem of filteredSandboxMenuItems; index as menuItemIndex\">\n            <h2 class=\"command-bar__title\" title=\"{{sandboxMenuItem.label}} {{sandboxMenuItem.name}}\"\n                [class.command-bar__sandbox-title--selected]=\"selectedSandboxAndScenarioKeys.sandboxKey === sandboxMenuItem.key\">\n                <span class=\"command-bar__name\"\n                      [innerHtml]=\"sandboxMenuItem.name | apHighlightSearchMatch : sandboxMenuItem.indexMatches\"></span>\n                <span class=\"command-bar__label\" *ngIf=\"sandboxMenuItem.label\"\n                      [innerHtml]=\"sandboxMenuItem.label | apHighlightSearchMatch : sandboxMenuItem.indexMatches : sandboxMenuItem.name.length\"></span>\n            </h2>\n            <div class=\"command-bar__scenarios\"\n                 *ngFor=\"let scenarioMenuItem of sandboxMenuItem.scenarioMenuItems; index as scenarioMenuItemIndex\">\n                <a\n                    class=\"command-bar__scenario-link\"\n                    #scenarioElement\n                    [tabindex]=\"scenarioMenuItem.tabIndex\"\n                    [attr.sandboxMenuItemKey]=\"sandboxMenuItem.key\"\n                    [attr.scenarioMenuItemkey]=\"scenarioMenuItem.key\"\n                    (keydown.control.arrowup)=\"onScenarioLinkControlUp(scenarioElement, $event)\"\n                    (keydown.control.arrowdown)=\"onScenarioLinkControlDown(scenarioElement, $event)\"\n                    (keydown.arrowup)=\"onScenarioLinkKeyDown(scenarioElement, filterElement, $event)\"\n                    (keydown.arrowdown)=\"onScenarioLinkKeyDown(scenarioElement, filterElement, $event)\"\n                    (keydown.esc)=\"onScenarioLinkKeyDown(scenarioElement, filterElement, $event)\"\n                    (keydown.enter)=\"onScenarioLinkKeyDown(scenarioElement, filterElement, $event)\"\n                    (keyup.esc)=\"onScenarioLinkKeyUp(scenarioElement, $event)\"\n                    (keyup.enter)=\"onScenarioLinkKeyUp(scenarioElement, $event)\"\n                    (click)=\"onScenarioClick(sandboxMenuItem.key, scenarioMenuItem.key, $event); toggleCommandBar()\"\n                    [class.command-bar__scenario-link--selected]=\"isSelected(sandboxMenuItem, scenarioMenuItem)\">\n                    <ap-pin [selected]=\"isSelected(sandboxMenuItem, scenarioMenuItem)\"></ap-pin>\n                    <span class=\"command-bar__scenario-label\">\n                  {{scenarioMenuItem.description}}\n                </span>\n                </a>\n            </div>\n        </div>\n    </div>\n    <a *ngIf=\"filteredSandboxMenuItems && filteredSandboxMenuItems.length > 0\" class=\"command-bar__brand\"\n       href=\"http://www.angularplayground.it/\" target=\"_blank\">\n        <ap-logo></ap-logo>\n    </a>\n</div>\n<section class=\"content\">\n    <ap-drawer *ngIf=\"!embed && activeMiddleware.overlay\" (openCommandBarClick)=\"toggleCommandBar()\" [class.content__menu--hide]=\"commandBarActive\" class=\"content__menu\"></ap-drawer>\n    <div class=\"content__none\" *ngIf=\"!selectedSandboxAndScenarioKeys.sandboxKey\">\n        <div class=\"content__none-message\">\n            <p *ngIf=\"totalSandboxes > 0\">\n                The playground has {{totalSandboxes}} sandboxed component{{totalSandboxes > 1 ? 's' : ''}}\n            </p>\n            <p *ngIf=\"totalSandboxes === 0\">\n                Get started - create your first sandbox! <a href=\"http://www.angularplayground.it/docs/how-to/sandboxing-components\" target=\"_blank\">Click here</a>.\n            </p>\n            <div class=\"content__shortcuts\">\n                <div class=\"content__shortcut\" *ngFor=\"let shortcut of shortcuts\">\n                    <div class=\"content__shortcut-label\">\n                        <ng-container *ngFor=\"let key of shortcut.keys; let i = index\">\n                            <code>\n                                {{key}}\n                            </code>\n                            <ng-container *ngIf=\"shortcut.keys.length > 1 && i < shortcut.keys.length - 1\">&nbsp;&nbsp;/&nbsp;&nbsp;</ng-container>\n                        </ng-container>\n                    </div>\n                    <div class=\"content__shortcut-value\">\n                        {{shortcut.description}}\n                    </div>\n                </div>\n            </div>\n        </div>\n    </div>\n    <ng-container *ngIf=\"selectedSandboxAndScenarioKeys.sandboxKey\">\n        <ap-scenario [selectedSandboxAndScenarioKeys]=\"selectedSandboxAndScenarioKeys\"></ap-scenario>\n    </ng-container>\n</section>\n",
            styles: ["*{box-sizing:border-box}:host{display:-webkit-box;display:flex;-webkit-box-orient:vertical;-webkit-box-direction:normal;flex-direction:column}.shield{height:100vh;opacity:0;position:absolute;z-index:2;width:100%}.command-bar{background-color:#252526;box-shadow:0 3px 8px 5px #000;color:#fff;display:-webkit-box;display:flex;-webkit-box-orient:vertical;-webkit-box-direction:normal;flex-direction:column;font-family:Consolas,monospace;left:50%;margin-top:-6px;max-height:100vh;padding-top:10px;position:absolute;-webkit-transform:translate(-50%,-120%);transform:translate(-50%,-120%);-webkit-transition:opacity .1s,-webkit-transform .1s;transition:transform .1s,opacity .1s,-webkit-transform .1s;width:376px;z-index:9999999999999}.command-bar::before{border-bottom:1px solid #000;content:\"\";display:block;position:absolute;top:61px;width:100%}.command-bar--open{min-height:60px;-webkit-transform:translate(-50%,0);transform:translate(-50%,0)}.command-bar--preview{opacity:.7}.command-bar__filter{background-color:#3c3c3c;border:1px solid #174a6c;color:#fff;font-family:Consolas,monospace;font-size:16px;margin:6px 0 0 5px;padding:8px;width:365px;z-index:1}.command-bar__filter::-webkit-input-placeholder{color:#a9a9a9}.command-bar__filter::-moz-placeholder{color:#a9a9a9}.command-bar__filter::-ms-input-placeholder{color:#a9a9a9}.command-bar__filter::placeholder{color:#a9a9a9}.command-bar__filter:-ms-input-placeholder{color:#a9a9a9}.command-bar__filter::-moz-focus-inner{border:0;padding:0}.command-bar__sandboxes{border-top:1px solid rgba(255,255,255,.1);margin-top:9px;overflow:auto;position:relative;max-height:calc(100vh - 109px)}.command-bar__sandboxes::-webkit-scrollbar{background-color:transparent;width:6px}.command-bar__sandboxes::-webkit-scrollbar-track{border-left:1px solid #000;background:rgba(255,255,255,.1)}.command-bar__sandboxes::-webkit-scrollbar-thumb{background-color:rgba(255,255,255,.1);margin-left:2px;width:4px}.command-bar__sandbox{border-bottom:1px solid #000;border-top:1px solid rgba(255,255,255,.1);padding:8px 6px 14px}.command-bar__sandbox:first-child{border-top:none}.command-bar__sandbox:last-child{border-bottom:none;padding-bottom:12px}.command-bar__title{-webkit-box-align:center;align-items:center;color:rgba(255,255,255,.5);display:-webkit-box;display:flex;font-family:Consolas,monospace;font-size:12px;font-weight:400;-webkit-box-pack:justify;justify-content:space-between;margin:0 0 5px;padding:5px 0 0}.command-bar__title ::ng-deep mark{background:0 0;color:#0097fb;font-weight:700}.command-bar__name{max-width:100%;overflow:hidden;text-overflow:ellipsis;white-space:nowrap}.command-bar__label{background:rgba(255,255,255,.1);border-radius:2px;display:block;font-size:10px;margin-left:10px;padding:4px 5px 3px}.command-bar__scenarios{display:-webkit-box;display:flex}.command-bar__scenario-link{-webkit-box-align:center;align-items:center;border-radius:2px;color:rgba(255,255,255,.5);cursor:pointer;display:-webkit-box;display:flex;padding:4px 3px;width:100%}.command-bar__scenario-link:active,.command-bar__scenario-link:focus,.command-bar__scenario-link:hover{background-color:#0097fb;color:#fff;outline-style:none}.command-bar__scenario-link:active .command-bar__scenario-icon,.command-bar__scenario-link:focus .command-bar__scenario-icon,.command-bar__scenario-link:hover .command-bar__scenario-icon{opacity:.5}.command-bar__scenario-label{line-height:1;max-width:calc(100% - 26px);min-width:calc(100% - 26px);padding-bottom:2px}.command-bar__scenario-link--selected{background:rgba(255,255,255,.1);color:#fff}.command-bar__scenario-link:active .command-bar__scenario-icon--selected,.command-bar__scenario-link:focus .command-bar__scenario-icon--selected,.command-bar__scenario-link:hover .command-bar__scenario-icon--selected{fill:#fff}.command-bar__brand{border-top:1px solid #000;display:block;position:relative;padding:9px 0 3px;text-align:center}.command-bar__brand::before{border-bottom:1px solid rgba(255,255,255,.1);content:\"\";display:block;left:0;position:absolute;top:0;width:100%}.command-bar__brand:hover .command-bar__logo__box{fill:rgba(255,255,255,.2)}.command-bar__brand:hover .command-bar__logo__letter{fill:rgba(255,255,255,.75)}.content__none{-webkit-box-align:center;align-items:center;border:0;display:-webkit-box;display:flex;min-height:calc(100vh - 4em);-webkit-box-pack:center;justify-content:center;padding-top:2em;padding-bottom:2em;position:relative;width:100%}.content__none-message{font-family:Arial,sans-serif;max-width:50%;min-width:450px;text-align:center}.content__none-message em{color:#666}.content__none-message p{font-size:20px}.content__shortcuts{border-top:1px solid #ccc;margin-top:2em;padding:30px 0 0 100px;width:520px}.content__shortcut{display:-webkit-box;display:flex}.content__shortcut-label{-webkit-box-align:center;align-items:center;display:-webkit-box;display:flex;font-size:11px;-webkit-box-pack:end;justify-content:flex-end;max-width:150px;min-width:150px;padding:8px 12px 8px 0;white-space:nowrap}.content__shortcut-label code{background:#eee;border:1px solid #ccc;border-radius:4px;padding:3px 7px}.content__shortcut-value{-webkit-box-align:center;align-items:center;display:-webkit-box;display:flex;font-size:11px;line-height:1.75;text-align:left;white-space:nowrap}"]
        }),
        __param(5, Inject(MIDDLEWARE))
    ], AppComponent);
    return AppComponent;
}());

var HighlightSearchMatchPipe = /** @class */ (function () {
    function HighlightSearchMatchPipe() {
    }
    HighlightSearchMatchPipe.prototype.transform = function (value, indexMatches, offset) {
        if (offset === void 0) { offset = 0; }
        // Match null and undefined, but not 0 or ''
        if (value == null || indexMatches == null) {
            return value;
        }
        var transformedValue = '';
        var indexes = indexMatches.reduce(function (a, n) { return n >= offset ? __spread(a, [n - offset]) : a; }, []);
        var _loop_1 = function (i) {
            if (indexes.some(function (item) { return item === i; })) {
                transformedValue += "<mark>" + value[i] + "</mark>";
            }
            else {
                transformedValue += value[i];
            }
        };
        for (var i = 0; i < value.length; i++) {
            _loop_1(i);
        }
        return transformedValue;
    };
    HighlightSearchMatchPipe = __decorate([
        Pipe({ name: 'apHighlightSearchMatch', pure: false })
    ], HighlightSearchMatchPipe);
    return HighlightSearchMatchPipe;
}());

var LogoComponent = /** @class */ (function () {
    function LogoComponent() {
    }
    LogoComponent = __decorate([
        Component({
            selector: 'ap-logo',
            template: "<svg class=\"command-bar__logo\" viewBox=\"0 0 645.9 136.4\">\n          <path class=\"command-bar__logo__box\"\n                d=\"M57.1,46.2c-0.1-0.3-0.1-0.5-0.1-0.8l-0.6,0.3c0.1-0.3,0.1-0.5,0.4-0.5c-0.4-0.3-1,0.2-1.5-0.4\n              c0.3-0.4-0.3-1.2,0.5-1.3c-0.3,1.2,0.8-0.1,0.9,0.9c-0.1-0.2-0.1-0.4,0-0.7c0.1,0,0.2,0,0.2,0.1c0.7-1.2-0.9-0.6-0.8-1.8\n              c0.4,0.2,0.8-0.3,0.7-0.5c0,0.5-0.7,0.2-1.1-0.2c0.4-0.8-0.2-0.9-0.2-1.3l-0.2,0.5c-0.2-0.1-0.3-0.2-0.4-0.4c0-0.1,0-0.2-0.1-0.3\n              c0-0.2,0-0.3-0.1-0.5c0.3-0.1,0.2,0.7,0.4,0.4c-0.4-0.4-0.2-0.9,0.1-1.5c0.2-0.6,0.5-1.4,0.3-1.9c0.3-0.8,0.8-0.5,1-1.3\n              c-0.5-1,0.3-1.9,0.3-3c-0.2,0-0.5,0.1-0.9,0.2c0,0.6,0.1,0.4-1.5,1.1l1.5-0.3l0.2,1.1c-0.2-0.3-0.4-0.5-0.7-0.6l-0.5,0.2\n              c0.1,0,0.2,0,0.4,0c0.1,0.5,0.3,0.9,0.2,1.4c-0.5-0.4-0.3-0.3-0.8,0.4c0.4-0.7,0.4-1.1,0.2-1.4c-0.7,0.2-1.4,0.5-1.7,0.2l1.1-1.5\n              c-1.6-0.6-4.1,1-5.6,0.7l0.5,0.2c-0.2,0.6-0.9,0.1-1.4,0.4c0-0.5-0.3-0.7-0.5-0.9c0,0.4-1.1,0-1.2,1l-0.7-0.6\n              c-1.3-0.2-1.3,1.5-2.5,1.4c0.3-0.5-0.2-0.9,0.6-1.3c-0.4,0-0.8-0.1-0.7,0.3c-0.2-0.3-1.1,0.7-1.5,0.3c-0.2,0.2-0.2,0.7-0.6,0.7\n              c-0.1-0.1,0-0.2,0-0.2c0,0.2-0.7,0.4-0.3,0.9c-1.3-1.5-3.4,0.3-4.5-1c-0.9,0.2-1.9,0.4-2.7,0.5c0-0.1,0-0.2,0.1-0.2\n              c-1.1-0.4-0.6,1.4-1.9,1c-0.7-0.8,0.6-0.5,0.2-0.7c-0.2-1.6-1.5,0.3-2.4-0.6l0.2-0.1c-0.7-0.5-1.5,0.7-2.6,0.6c0,0,0-0.1,0-0.2\n              c-0.7,0.9-2,1.2-3,1.5c0.3-0.6,0.4-0.4,0.2-1c-0.4,0.1,0.2,1-0.7,1.2c-0.3-0.3-1-0.6-1.1-1.3l0.8-0.1c-0.5-0.9-1.1,0.1-1.5,0l0-0.4\n              c-1.3,0.2-1.3,1.2-2.6,1.2L19,36c-0.7,1-0.7-0.5-1.4,0.2l-0.3-1c-0.5,0.3-1.7,1-2.6,1.3c0.5-0.8,1.5-1.2,2.1-1.6\n              c-0.5,0-1.7,0.2-1.9,0.6c0.2-0.1,0.4-0.4,0.6-0.2c-0.6,0.8-1.4,1.2-2.3,1.2c0.1-1.9-2.6-0.4-3.4-1.8C9.4,35,9,35.1,8.5,35.1\n              c-0.2,0-0.4,0-0.7,0c-0.4,0-0.7,0-1.1,0c-1.5-0.1-3.3-0.2-5-0.1c2.6,1.4-1,0.1,0.9,1.5c0,0-0.1,0-0.1,0c0.1,0.6,0.1,0.8,0,0.7\n              l0.1-0.1c0,0,0,0.1-0.1,0.1l0,0c-0.3,0.4-0.5,0.9-0.8,1.5c-0.4,0.4-0.4-0.9-0.8-1c0,1.8-0.6,0.5-0.9,1.7c0.5-0.1,0.9,0.9,0.8,1.9\n              c0.2-0.5,0.4-1,0.6-1.4c0.5,1.3,0.2,1.7,0,2.3C1,42.6,0.7,43,1.2,43.7l-0.5-0.4C1,44,0.9,46,2.2,46.6c-0.2,0.1-0.6,0.4-0.7,0\n              c0,1.8,1.7,3.9,1,5.3l-0.4-0.1c0.1,0.3,0.7,0.5,0.4,1c-0.2,0-0.4-0.1-0.4,0.1c0.1,0.2,0.7,0.8,0.6,1.3l-0.5-0.3\n              c-0.3,1.8,1.3,2.6,1.6,4c-1.1,0.1-1.3,1.5-1.4,2.2l-0.2,0c0.1,2,1.2,3.4,1.8,5.2c-1.1,1.3-0.1,4-0.6,5.9c0.2-0.3,0.9,0,1,0.3\n              c-0.2,0.5,0.9,1.1-0.1,1.2l0-0.2c-0.9,0.4-0.6,1.3-0.6,2l1.3,0c-1.7,2,1.1,4.8-0.5,5.9c0.4,0.4,0.4,1,0.8,1.4l-0.7,0\n              c-0.1,1.3-0.1,1.9,0.2,3.4L4.5,85c0,0.9,2.1,0.9,2.2,1.7l-1,0.3c0.2,1.5-0.6,3.1,0.5,4.5c-0.2,0-0.5,0-0.5-0.3c0,0.7-0.5,2.8,1,2.4\n              c-0.9,0.1,0.1,1.6,0.5,2.3L6.4,96c0.9,0.6,1.2,0.6,1.5,1.5c-0.1,0.2-0.4,0.2-0.4,0.1c-0.1,0.6,0.7,0.5,0.5,1.4l-0.5-0.3\n              c0,0.8,0.2,0.7,0.8,1.1c0.3,0.3,0,1.8-0.6,2.2c0.4,0,0.9,0.4,0.8,1.1c-0.2,0.2-0.4,0.1-0.5,0c-0.1,0.9,0,1.5,0.2,1.9\n              c1-0.1,1.7-0.3,2.2,0.2c-0.1,0.2-0.1,0.3,0,0.4c-1.2-0.2-2.1-0.3-2.5-0.4c0-0.2,0-0.2,0-0.2c-0.1,0.1-0.3,0.3-0.3,0.5\n              c0,0.3,0.5,0.6-1,1.1c0.9-0.1,2-0.4,2.9-0.5c1-0.2,1.8-0.2,2.4,0c0,0.1,0.1,0.1,0.1,0.2l-0.2,0.1c0.3,0.1,0.6,0.1,0.8,0\n              c0.2,0.1,0.4,0.2,0.6,0.3c0,0.5,0.9,0.4,1.3,0.5c0.1-0.7,0.6-0.5,0.2-1.2c0.1,0.1-0.1,0.1-0.4,0.1c-0.1,0-0.1,0-0.2,0c0,0,0,0,0-0.1\n              c-0.3,0-0.6,0-1-0.1c0.4-0.3,0.8-0.6,1.2-0.1c0.3-0.4,0.6-0.8,0.7-1c0.4,1,1.5,1.7,1.5,2.3c1.3-0.2,3,0.1,4-1\n              c0.4-0.1,0.4,0.4,0.5,0.7c-0.2,0.1-0.5,0-0.6,0.2c0.8-0.2,2.1,0.2,2.4-0.7c0.9,0.5-0.6,0.8,0.6,0.8l-0.2,0.3\n              c2.3,1.8,4.3-1.8,6.8,0.4c-0.2-0.1-0.3-0.2-0.3-0.4c3,1.2,6.5,0.1,9.4,1.8c0.8,0.4,1-0.2,1.6-0.6l0.1,0.8l1-0.7\n              c1.4-0.5,2,2.1,3.8,1.2l-0.1-0.5c0.6,0.2,1.9,0,1.7,0.5c0.1-0.1,0.3-0.5,0.5-0.2l0,0.4l1.5-0.5c1.1,0.4-0.4,1.5,1.2,1.8\n              c0.6-0.1,1.9-0.9,1.7-1.2c-0.2-0.1-0.6-0.3-0.6-0.7l0.6-0.1c0-0.1,0-0.1,0.1-0.2l0.1,0l0-0.4c0.2-0.7,0.3-1.3,0-2l0.1-4.2l0.2,0.1\n              l-0.2-0.2l0-0.8c0.1,0,0.3,0.1,0.3,0.3c0.5-0.6-0.4-0.6,0.2-1.1c0.1-0.1,0.1,0,0.2,0l-0.3-0.6c0.2,0.1,0.7-0.1,0.6,0.3\n              c0.6-1.1-0.7-1.8,0-2.2l0.1,0.1c-0.2-1.1-0.5-0.7-1-0.8l0-0.7l1.3-0.5l-0.8-0.2c0-0.2,0.1-0.5,0.4-0.4c-0.5-0.6-0.3,0.1-0.8-0.2\n              c0.7-0.7-0.3-2.1,0.9-2.2c-0.6-0.3-0.5,0.7-0.8-0.1c1.3-0.2-0.2-0.9,0.4-1.6c0.7,0.2,0.3,0.9,0.9,0.2c-1.1-0.4,0.3-1.3,0.1-1.9\n              c-0.9,0-0.4-1.3-0.6-2.2c0.3,0.1,0.5-1.8,0.9-2.7l0.2,0c-0.9-0.2-0.7-1.1-0.9-1.8c0.5,0,0.8,0.2,1,0.5l0-0.6\n              c-0.3-0.2-0.4-0.4-0.5-0.8l0.7,0.2c0-0.8-0.8-1.7-0.3-2.8c-0.4,0-1.1,0.3-1.6,0.2l0-0.6c0.2,0,0.4,0.1,0.5,0.2\n              c0-0.1-0.3-0.4-0.1-0.6l0.6,0.5c0.3-0.5-0.5-0.3-0.4-0.7c0.5,0,0.4-0.3,0.7-0.2c0,0.1-0.2,0.3,0,0.5c0.1-0.4,0.4-1,0.8-1.2\n              c-0.3,0-0.7,0-0.7-0.3c0-0.8,0.6-0.3,0.9-0.8c-0.3-0.2,0-0.7-0.7-0.8c0-0.2,0.4-0.3,0.5-0.4c-0.3-0.4-0.1-2-0.8-1.6l-0.1,0.1\n              c0,0,0,0,0-0.1c-0.7-1.4,0.5-3,0.2-4.4l0.2,0c-0.3-0.9-0.1-1.9-0.7-2.5c0-0.3,0.5-0.6,0.7-0.5c-0.2-0.7-0.4-0.7,0-1.7\n              c0.4,0.7,0.4-0.2,0.3,0.7c0.2-0.8,0.6-1,0-1.9c0.6,0.3,0.8-0.6,0.6-1l-0.5,0.6c-0.1-0.5,0-0.8,0-1.2l0.8,0.2c1-1.7-1.7-3.1,0-4.4\n              c-0.6-0.5-0.4,0.9-1.1,0.5c0.1-0.8,1.3-1.6,1.3-1.8c0.5-1.1-1.2-1-0.9-2.2c-0.3,0.1-1.1,0.3-1.3-0.6c0.1-0.8,1.3-1,1.5-0.5\n              c0,0.2-0.1,0.3-0.2,0.3c0.1,0.3,0.2,0.6,0.5,0.3l-0.3-0.1c0.3-0.4,0.3-1.5,0.8-1.2c-0.3-0.6-0.5-0.3-0.6,0.2c0.4-1,0-1.6,0.3-2.6\n              c-0.2,0.2-0.5,0.3-0.5,0.1c0.4-1.5-0.5-0.7-0.7-1.7c0.5-0.6,1.1,0.2,0.7-0.8c0.3-0.6,0.8-0.1,1,0.1c-0.6-1,0.4-1.8,0.5-2.3\n              L57.1,46.2z M14.1,106.6C14.1,106.6,14.1,106.6,14.1,106.6c0,0,0,0.2-0.1,0.3c-0.3,0.1-0.4-0.1-0.4-0.2\n              C13.7,106.6,13.8,106.6,14.1,106.6z M52.7,100.4c0,0-0.1,0.1-0.1,0.1l0-0.3C52.6,100.2,52.6,100.3,52.7,100.4z\"></path>\n          <path class=\"command-bar__logo__box\"\n                d=\"M115.7,90.3L115.7,90.3c-0.3-1-0.5-0.5-1-0.6l-0.1-0.7l1.2-0.6l-0.8-0.1c0-0.2,0-0.5,0.4-0.4\n              c-0.6-0.5-0.3,0.1-0.8-0.2c0.6-0.8-0.5-2.1,0.7-2.3c-0.6-0.2-0.4,0.8-0.8,0c1.3-0.3-0.3-0.9,0.2-1.6c0.7,0.2,0.4,0.8,0.9,0.1\n              c-1.2-0.3,0.1-1.4-0.1-1.9c-0.9,0-0.5-1.2-0.8-2.2c0.3,0,0.4-1.9,0.6-2.8l0.2,0c-0.9-0.1-0.8-1.1-1.1-1.7c0.5,0,0.8,0.1,1,0.4\n              l-0.1-0.6c-0.3-0.1-0.5-0.3-0.6-0.7l0.7,0.1c-0.1-0.8-0.9-1.7-0.6-2.8c-0.4,0-1.1,0.4-1.6,0.4l0-0.6c0.2,0,0.4,0.1,0.5,0.2\n              c-0.1-0.1-0.3-0.3-0.2-0.5l0.6,0.4c0.2-0.5-0.5-0.2-0.5-0.6c0.5,0,0.3-0.3,0.7-0.3c0.1,0.1-0.2,0.4,0,0.5c0.1-0.5,0.3-1,0.7-1.2\n              c-0.3,0.1-0.7,0.1-0.7-0.3c-0.1-0.8,0.6-0.3,0.8-0.9c-0.4-0.2,0-0.8-0.8-0.7c0-0.2,0.3-0.3,0.5-0.4c-0.4-0.4-0.3-2.1-1-1.5l0,0.1\n              c0,0,0,0,0-0.1c-0.8-1.4,0.2-3-0.2-4.4l0.2,0c-0.3-0.8-0.3-1.9-0.9-2.5c0-0.3,0.4-0.6,0.7-0.5c-0.2-0.6-0.5-0.7-0.2-1.7\n              c0.5,0.6,0.4-0.3,0.4,0.6c0.1-0.9,0.5-1.1-0.2-1.9c0.6,0.3,0.7-0.7,0.5-1.1l-0.4,0.6c-0.1-0.4-0.1-0.8-0.1-1.3l0.8,0.2\n              c0.9-1.8-1.9-3-0.4-4.5c-0.6-0.4-0.3,1-1,0.6c0-0.8,1.2-1.7,1.1-2c0.4-1.1-1.3-0.9-1.1-2.1c-0.3,0.2-1.1,0.4-1.4-0.5\n              c0-0.8,1.2-1.1,1.4-0.6c0,0.2,0,0.3-0.1,0.3c0.1,0.3,0.3,0.6,0.6,0.2l-0.3-0.1c0.3-0.4,0.2-1.5,0.7-1.3c-0.3-0.6-0.5-0.2-0.6,0.2\n              c0.3-1.1-0.1-1.6,0.1-2.7c-0.1,0.2-0.5,0.4-0.5,0.2c0.2-1.6-0.6-0.6-0.8-1.7c0.4-0.6,1.1,0.1,0.6-0.9c0.3-0.7,0.8-0.2,1,0\n              c-0.6-0.9,0.2-1.8,0.3-2.4l0.3,0.5c-0.2-0.3-0.2-0.5-0.2-0.8l-0.6,0.3c0.1-0.3,0.1-0.6,0.3-0.6c-0.4-0.3-1,0.3-1.5-0.3\n              c0.3-0.4-0.4-1.2,0.4-1.4c-0.2,1.3,0.8-0.2,1,0.8c-0.1-0.2-0.1-0.4,0-0.8c0.1-0.1,0.2,0,0.2,0.1c0.6-1.2-0.9-0.5-1-1.7\n              c0.5,0.1,0.8-0.4,0.7-0.5c0,0.5-0.7,0.3-1.1-0.1c0.3-0.8-0.3-0.8-0.3-1.3l-0.1,0.5c-0.5-0.1-0.4-0.5-0.6-1.1c0.1,0,0.2,0,0.2,0.1\n              c-0.2-0.6,0.8-2,0.2-2.9c0.2-0.8,0.8-0.5,0.9-1.2c-0.3-0.6-0.2-1.1-0.1-1.6c0.1-0.5,0.2-1.1,0.1-1.7c-0.2,0-0.4,0.1-0.6,0.1\n              c0,0.6,0.1,0.4-1.4,1.1l1.1-0.2l0.4,1.4c-0.3-0.3-0.6-0.6-0.9-0.8l-0.1,0.1l0.1,0c0.1,0.6,0.5,1.1,0.4,1.6c-0.5-0.4-0.4-0.4-0.8,0.3\n              c0.4-0.8,0.2-1.2,0-1.5c-0.6,0.2-1.1,0.4-1.4,0.1l1-1.5c-0.8-0.3-1.9,0-2.8,0.3c-1,0.3-2.1,0.6-2.7,0.5l0.5,0.2\n              c-0.2,0.6-1,0.1-1.4,0.4c0-0.5-0.3-0.7-0.5-0.9c0,0.4-1.1,0-1.2,1l-0.7-0.6c-1.3-0.2-1.3,1.5-2.6,1.4c0.3-0.6-0.2-0.9,0.5-1.3\n              c-0.4,0-0.8,0-0.8,0.3c-0.3-0.3-1.1,0.7-1.5,0.3c-0.2,0.2-0.2,0.7-0.6,0.7c-0.1-0.1,0-0.2,0-0.2c0,0.2-0.7,0.4-0.3,0.9\n              c-1.3-1.4-3.4,0.3-4.6-0.9c-0.9,0.2-1.9,0.4-2.7,0.5c0-0.1,0-0.2,0.1-0.2c-1.1-0.4-0.6,1.4-1.9,1c-0.7-0.7,0.6-0.5,0.2-0.7\n              c-0.3-1.6-1.5,0.3-2.4-0.6l0.2-0.1c-0.7-0.5-1.5,0.7-2.6,0.7c0,0,0-0.1,0-0.2c-0.7,0.9-2,1.2-3,1.5c0.3-0.6,0.4-0.4,0.2-1\n              c-0.4,0.1,0.2,1-0.7,1.3c-0.3-0.3-1-0.6-1.1-1.3l0.8-0.1c-0.5-0.9-1.1,0.1-1.6,0l0-0.4c-1.3,0.2-1.3,1.2-2.7,1.2L75,28\n              c-0.4,0.5-0.5,0.4-0.7,0.2c-0.2-0.2-0.5-0.3-0.9,0c-0.2-0.3-0.4-0.6-0.6-0.9c-0.3,0.1-0.8,0.4-1.3,0.6c-0.5,0.3-1,0.5-1.4,0.6\n              c0,0.1,0,0.1,0,0.2c0-0.1-0.1-0.2-0.1-0.3c0.1-0.6,1.2-1.1,2-1.5c-0.8,0-2.7,0.2-2.6,0.7c0.2-0.1,0.4-0.4,0.8-0.2\n              c-0.1,0.3-0.3,0.5-0.5,0.7c0.2,0.9,0.3,1.6,0.3,2.4c-0.9-0.7-1.1,0.2-1.2,1.4c0,0.3-0.1,0.6-0.1,0.8c0,0.2-0.1,0.4-0.1,0.6\n              c-0.1,0.4-0.3,0.6-0.7,0.8c0.6,1.8-0.5,3.5-0.6,5.5c1.5-0.6,0,0.6,1.5,0.6c0.1,0.5,0,0.7-0.1,0.7l0.1-0.1c0,0-0.1,0.1-0.1,0.1l0,0\n              l-1,1.4c-0.5,0.3-0.3-0.5-0.6-0.5c-0.2,1-0.7,0.4-1.1,1.1c0.6-0.1,0.8,0.4,0.5,1l0.8-0.8c0.7,1.5-1.4,1.2-0.7,2.7l-0.4-0.5\n              c0.2,0.8-0.2,2.8,1,3.6c-0.2,0.1-0.6,0.3-0.7-0.1c-0.3,1.8,1,4.1,0.2,5.5l-0.4-0.2c0.1,0.3,0.7,0.6,0.3,1c-0.2,0-0.4-0.2-0.4,0\n              c0.1,0.2,0.6,0.9,0.3,1.4L66,56.2c-0.6,1.7,0.8,2.8,1,4.3c-1.1-0.1-1.5,1.3-1.8,2l-0.2,0c-0.3,2.1,0.7,3.6,1,5.5\n              c-1.3,1.1-0.8,4-1.6,5.8c0.2-0.3,0.8,0.1,0.9,0.5c-0.3,0.4,0.7,1.3-0.3,1.1l0-0.2c-1,0.3-0.8,1.2-0.9,1.9l1.3,0.2\n              c-2,1.7,0.4,5-1.4,5.9c0.3,0.5,0.3,1,0.5,1.5l-0.7-0.1c-0.3,1.2-0.4,1.9-0.3,3.4l-0.5-0.4C63,88.6,65,89,65,89.8l-1.1,0.2\n              c-0.1,1.5-1.1,3-0.3,4.6c-0.2,0-0.5-0.1-0.5-0.4c0,0.4-0.3,1-0.3,1.6c0,0.1,0,0.1,0,0.2c0,0.1,0,0.2,0,0.3c0,0.2,0,0.4,0.1,0.6\n              c0.1,0.3,0.3,0.4,0.7,0.3c-0.5,0-0.5,0.7-0.5,1.5c0.1,0.8,0.3,1.6,0.5,2.1c-0.2,0.1-0.5,0.2-0.7,0.2c0.7,0.5,1,0.4,1.2,1.1\n              c0.1,0,0.3-0.1,0.5-0.1c0.2,0.1-0.1,0.4-0.4,0.4c0.7,0.2,1.4-0.6,2.3-0.3l-0.8,0.4c1.1,0.1,1.2-0.2,2-0.7c0.4-0.1,0.8-0.1,1.2,0.1\n              c0.4,0.2,0.8,0.4,0.9,0.8c0.2-0.4,0.6-0.9,1.1-0.7c0.1,0.2-0.1,0.4-0.2,0.5c1.3,0.5,2-0.8,2.9-0.1c0,0.2,0.1,0.3,0.1,0.4\n              c-1,0-1.9-0.1-1.8-0.1c0,0.6,0.8,0.8,0.1,1.6c0.7-0.5,1.9-1.5,2.7-1.2c0,0.1,0.1,0.1,0.1,0.2l-0.1,0.1c0.2,0.1,0.4,0,0.5-0.1\n              c0.3,0.1,0.4,0.2,0.7,0.2c0.1,0.5,1,0.3,1.4,0.3c0-0.7,0.5-0.6,0-1.2c0.1,0.1-0.1,0.1-0.4,0.1c-0.1,0-0.1,0-0.2,0.1c0,0,0,0,0-0.1\n              c-0.3,0-0.6,0-1.1,0c0.3-0.3,0.7-0.6,1.2-0.2c0.2-0.4,0.5-0.9,0.6-1.1c0.6,1,1.7,1.5,1.8,2.1c1.3-0.4,3-0.2,3.9-1.5\n              c0.4-0.2,0.4,0.4,0.6,0.6c-0.2,0.1-0.5,0.1-0.6,0.3c0.7-0.3,2.2-0.1,2.4-1c0.9,0.4-0.5,0.8,0.7,0.7l-0.2,0.3\n              c2.5,1.5,4.1-2.3,6.8-0.4c-0.2-0.1-0.3-0.2-0.3-0.3c3.1,0.9,6.5-0.7,9.6,0.7c0.9,0.3,1-0.3,1.5-0.7l0.2,0.8l0.9-0.9\n              c1.3-0.6,2.2,1.9,3.9,0.7l-0.2-0.5c0.6,0.1,1.9-0.3,1.8,0.3c0.1-0.1,0.2-0.5,0.4-0.3l0,0.4l1.5-0.6c1.1,0.3-0.2,1.6,1.5,1.7\n              c0.6-0.1,1.8-1.1,1.5-1.4c-0.2,0-0.6-0.3-0.6-0.6l0.6-0.2c0-0.1,0-0.1,0-0.2l0.1,0l0-0.4c0.1-0.6,0.2-1.3-0.1-1.8l-0.3-4.4l0.2,0.1\n              l-0.2-0.2l-0.1-0.8c0.1,0,0.3,0.1,0.4,0.2c0.5-0.7-0.4-0.5,0.1-1.1c0.1-0.1,0.1,0,0.2,0l-0.4-0.6c0.2,0.1,0.7-0.2,0.6,0.2\n              C116.4,91.3,115,90.8,115.7,90.3z M76.8,103.4C76.8,103.4,76.8,103.4,76.8,103.4c0,0,0,0.2-0.1,0.3c-0.3,0.1-0.4,0-0.4-0.2\n              C76.4,103.5,76.5,103.5,76.8,103.4z M114.9,92.7l0-0.3c0,0.1,0.1,0.2,0.1,0.3C115,92.7,115,92.7,114.9,92.7z\"></path>\n          <path class=\"command-bar__logo__box\"\n                d=\"M295.1,34.4l-0.6,0.3c0.1-0.3,0.1-0.6,0.3-0.6c-0.4-0.3-1,0.2-1.5-0.4c0.3-0.4-0.3-1.2,0.4-1.4\n              c-0.3,1.3,0.8-0.1,0.9,0.9c-0.1-0.2-0.1-0.5,0-0.8c0.1,0,0.2,0,0.2,0.1c0.7-1.2-0.9-0.5-0.9-1.8c0.4,0.2,0.8-0.3,0.7-0.5\n              c0,0.5-0.7,0.3-1.1-0.2c0.2-0.3,0.1-0.8,0-1c-0.1-0.3-0.2-0.4-0.2-0.7l-0.2,0.7c-0.4-0.2-0.4-0.7-0.5-1.4c0.3-0.2,0.2,0.6,0.4,0.3\n              c-0.4-0.3-0.2-0.8,0-1.5c0.2-0.7,0.5-1.5,0.2-2c0.3-0.9,0.8-0.8,1-1.6c0,0,0,0,0,0c-1.1,0.3-1.3,0.1-1.7-0.1\n              c-0.3-0.2-0.8-0.3-1.8-0.1c0.2,0.6,0.3,0.4-1.1,1l2.9-0.4c-0.8,0.3-1.6,0.6-2.2,0.9c0.7,0,1.8-0.4,2.2-0.3c-1.2,0.5-0.9,0.3-1,0.8\n              c-0.1-0.4-0.7-0.3-1.3-0.1c-0.6,0.2-1.1,0.4-1.5,0.1l0.5-1.5c-2.3-0.7-2.8,0.8-4.5,0.4l0.5,0.3c-0.2,0.6-1,0.1-1.5,0.3\n              c0-0.5-0.3-0.7-0.5-1c0,0.4-1.1,0-1.3,0.9l-0.7-0.6c-1.3-0.3-1.4,1.4-2.7,1.2c0.4-0.5-0.2-0.9,0.6-1.3c-0.4,0-0.8-0.1-0.8,0.2\n              c-0.2-0.3-1.1,0.7-1.5,0.2c-0.2,0.2-0.2,0.7-0.7,0.7c0-0.1,0-0.2,0-0.2c0,0.2-0.7,0.3-0.3,0.9c-1.3-1.5-3.5,0.1-4.6-1.2\n              c-0.9,0.2-2,0.3-2.8,0.4c0.1-0.1,0-0.2,0.2-0.2c-1.1-0.5-0.7,1.4-1.9,0.9c-0.6-0.8,0.7-0.4,0.3-0.7c-0.2-1.6-1.5,0.2-2.4-0.7\n              l0.2-0.1c-0.6-0.5-1.6,0.6-2.6,0.5c0,0,0.1-0.1,0-0.2c-0.7,0.8-2.1,1.1-3.1,1.3c0.3-0.6,0.5-0.3,0.3-1c-0.5,0.1,0.1,1-0.8,1.2\n              c-0.3-0.3-1-0.6-1-1.3l0.9,0c-0.4-0.9-1.1,0-1.6-0.1l0.1-0.4c-1.3,0.1-1.3,1.1-2.8,1l0.1,0.3c-0.8,1-0.6-0.5-1.4,0.2l-0.2-1\n              c-0.3,0.1-0.6,0.4-1.3,0.6c-0.6,0.2-1.2,0.5-1.7,0.5c0.1-0.4,0.5-0.6,1.1-0.9c0.3-0.1,0.6-0.2,0.9-0.3c0.1-0.1,0.3-0.1,0.4-0.2\n              c0.1-0.1,0.2-0.1,0.3-0.2c-0.2,0-0.9,0-1.5,0.1c-0.6,0.1-1.1,0.2-1.2,0.5c0.2-0.1,0.6-0.3,0.9-0.2c-0.7,0.8-1.4,1.1-2.6,1.1\n              c-0.3-0.4-0.7-0.7-1.1-0.8c-0.3,0.2-0.4,1.2-0.5,2c-0.1,0.9-0.3,1.7-0.9,2.1c0.1,0.4,0.1,0.8,0.1,1.1c0,0.2,0,0.4,0,0.7\n              c0,0.4-0.1,0.9-0.2,1.4c-0.2,0.9-0.5,1.9-0.5,2.9c1.5-0.6,0,0.6,1.5,0.6c0.1,0.5,0,0.7-0.1,0.7l0.1-0.1c0,0-0.1,0.1-0.1,0.1l0,0\n              l-1,1.4c-0.5,0.3-0.3-0.5-0.6-0.5c-0.2,1.1-0.7,0.4-1.1,1.1c0.6-0.1,0.8,0.4,0.5,1l0.8-0.8c0.7,1.5-1.4,1.2-0.7,2.7l-0.4-0.5\n              c0.2,0.8-0.2,2.8,1,3.7c-0.2,0.1-0.6,0.3-0.7-0.1c-0.3,1.8,1,4.2,0.2,5.5l-0.4-0.2c0.1,0.3,0.7,0.6,0.3,1c-0.2,0-0.4-0.2-0.4,0\n              c0.1,0.2,0.6,1,0.3,1.4l-0.4-0.4c-0.6,1.8,0.8,2.8,1,4.3c-1.1-0.1-1.5,1.3-1.8,2l-0.2,0c-0.3,2.1,0.7,3.6,1,5.5\n              c-1.3,1.1-0.8,4-1.6,5.9c0.2-0.3,0.8,0.1,0.9,0.5c-0.3,0.4,0.7,1.3-0.3,1.2l0-0.2c-1,0.3-0.8,1.2-0.9,1.9l1.3,0.2\n              c-2,1.8,0.4,5-1.4,5.9c0.3,0.5,0.3,1,0.5,1.5l-0.7-0.1c-0.3,1.3-0.4,1.9-0.3,3.5l-0.5-0.4c-0.1,0.9,1.9,1.3,1.9,2.1l-1.1,0.2\n              c-0.1,1.5-1.1,3-0.3,4.6c-0.2,0-0.5-0.1-0.5-0.4c-0.1,0.7-0.9,2.8,0.6,2.6c-0.5,0-0.5,0.3-0.4,0.9c0.1,0.9,0.3,1.8,0.5,2.4\n              c-0.2,0-0.5,0-0.7,0c0.4,0.5,0.7,0.6,0.9,0.8c0.2,0.2,0.3,0.5,0.4,1.1c-0.1,0.3-0.5,0.6-0.4,0.3c-0.2,0.8,0.3,0.6,0.3,1\n              c0.2,0,0.4,0,0.6,0.1l-0.9,0.4c1.1,0.1,1.3-0.1,2.3-0.6c0.4-0.1,0.9-0.1,1.2,0.1c0.4,0.2,0.7,0.5,0.6,0.8c0.3-0.4,1-0.8,1.6-0.6\n              c0.1,0.2-0.2,0.4-0.3,0.4c0.5,0.1,0.8,0.2,1,0.1c0.3,0,0.5-0.1,0.7-0.2c0.5-0.2,0.9-0.3,1.3,0.1c0,0.2,0.1,0.3,0.1,0.4\n              c-1.1-0.1-1.9-0.1-1.8-0.2c0,0.6,0.8,0.8,0.1,1.6c0.7-0.4,1.9-1.4,2.8-1.1c0,0.1,0.1,0.1,0.1,0.2l-0.1,0.1c0.2,0.1,0.4,0,0.5-0.1\n              c0.3,0.1,0.4,0.2,0.7,0.2c0.1,0.5,1,0.3,1.4,0.4c0-0.7,0.6-0.6,0.1-1.2c0.1,0.1-0.1,0.1-0.4,0.1c-0.1,0-0.1,0-0.2,0.1c0,0,0,0,0-0.1\n              c-0.3,0-0.6,0-1.1,0c0.4-0.3,0.7-0.6,1.2-0.2c0.2-0.4,0.5-0.9,0.7-1.1c0.5,1,1.6,1.6,1.8,2.2c1.3-0.3,3.1-0.1,4-1.4\n              c0.4-0.1,0.4,0.4,0.5,0.6c-0.2,0.1-0.5,0.1-0.6,0.3c0.8-0.3,2.2,0,2.4-0.9c0.9,0.5-0.6,0.8,0.7,0.7l-0.2,0.3\n              c2.4,1.6,4.2-2.1,6.9-0.1c-0.2-0.1-0.3-0.2-0.3-0.3c3.1,1,6.6-0.5,9.7,1c0.9,0.3,1-0.3,1.6-0.7l0.2,0.8l1-0.8c1.4-0.6,2.2,2,3.9,0.8\n              l-0.2-0.5c0.6,0.1,1.9-0.2,1.8,0.4c0.1-0.1,0.2-0.5,0.4-0.2l0,0.4l1.5-0.6c1.1,0.3-0.3,1.5,1.4,1.7c0.6-0.1,1.8-1.1,1.6-1.3\n              c-0.2,0-0.6-0.3-0.6-0.7l0.6-0.2c0-0.1,0-0.1,0-0.2l0.1,0l0-0.5c0.1-0.5,0.2-1.1,0-1.6l-0.1-4.7l0.2,0.1l-0.2-0.2l0-0.9\n              c0.1,0,0.3,0.1,0.3,0.3c0.5-0.6-0.4-0.6,0.2-1.1c0.1-0.1,0.1,0,0.2,0l-0.3-0.6c0.2,0.1,0.7-0.1,0.6,0.3c0.6-1.1-0.8-1.8,0-2.2\n              l0.1,0.1c-0.3-1.1-0.6-0.7-1.1-0.8l0-0.7l1.3-0.5l-0.8-0.1c0-0.2,0.1-0.5,0.4-0.4c-0.5-0.6-0.3,0.1-0.8-0.2c0.7-0.8-0.3-2.1,0.9-2.3\n              c-0.6-0.3-0.4,0.7-0.8-0.1c1.3-0.2-0.2-0.9,0.4-1.6c0.7,0.2,0.3,0.9,0.9,0.2c-1.1-0.4,0.2-1.4,0-1.9c-0.9,0-0.4-1.3-0.7-2.3\n              c0.3,0.1,0.5-1.9,0.8-2.8l0.2,0c-0.9-0.2-0.8-1.1-0.9-1.8c0.5,0,0.8,0.1,1,0.5l0-0.6c-0.3-0.1-0.5-0.3-0.6-0.8l0.7,0.2\n              c0-0.9-0.8-1.7-0.4-2.8c-0.4,0-1.1,0.4-1.6,0.3l0-0.6c0.2,0,0.4,0.1,0.5,0.2c0-0.1-0.3-0.4-0.1-0.6l0.6,0.5c0.2-0.5-0.5-0.3-0.4-0.7\n              c0.5,0,0.3-0.3,0.7-0.2c0,0.1-0.2,0.4,0,0.5c0.1-0.5,0.4-1,0.7-1.2c-0.3,0-0.7,0-0.7-0.3c0-0.9,0.6-0.3,0.8-0.8\n              c-0.3-0.2,0-0.8-0.7-0.8c0-0.3,0.3-0.3,0.5-0.4c-0.3-0.4-0.1-2.1-0.9-1.6l0,0.1c0,0,0,0,0-0.1c-0.7-1.5,0.4-3,0.1-4.5l0.2,0\n              c-0.3-0.9-0.2-1.9-0.7-2.5c0-0.3,0.4-0.6,0.7-0.5c-0.2-0.7-0.5-0.7-0.1-1.7c0.4,0.7,0.4-0.2,0.3,0.7c0.2-0.9,0.6-1.1-0.1-2\n              c0.6,0.3,0.8-0.7,0.6-1.1l-0.4,0.6c-0.1-0.5,0-0.9,0-1.3l0.8,0.2c1-1.8-1.7-3.2-0.1-4.5c-0.6-0.5-0.4,1-1.1,0.6\n              c0.1-0.8,1.3-1.7,1.3-1.9c0.4-1.1-1.2-1-0.9-2.2c-0.3,0.2-1.1,0.3-1.4-0.6c0.1-0.8,1.2-1,1.5-0.5c0,0.2-0.1,0.3-0.2,0.3\n              c0.1,0.3,0.2,0.6,0.6,0.3l-0.3-0.1c0.3-0.4,0.3-1.5,0.7-1.3c-0.3-0.6-0.5-0.3-0.6,0.2c0.4-1,0-1.6,0.3-2.7c-0.2,0.2-0.5,0.3-0.5,0.2\n              c0.3-1.6-0.6-0.7-0.7-1.7c0.4-0.6,1.1,0.2,0.7-0.9c0.3-0.7,0.8-0.1,1,0.1c-0.6-1,0.4-1.8,0.5-2.4l0.2,0.5\n              C295.1,34.9,295.1,34.6,295.1,34.4z M253,100.3C253,100.2,253,100.2,253,100.3c0,0,0,0.2-0.1,0.3c-0.3,0.1-0.4-0.1-0.4-0.2\n              C252.6,100.3,252.8,100.3,253,100.3z M292.3,90.7c0,0-0.1,0.1-0.1,0.1l0-0.3C292.2,90.5,292.2,90.6,292.3,90.7z\"></path>\n          <path class=\"command-bar__logo__box\"\n                d=\"M354.9,39.5c-0.1-0.3-0.1-0.6-0.1-0.8l-0.6,0.3c0.1-0.3,0.1-0.5,0.4-0.5c-0.4-0.3-1,0.2-1.5-0.4\n              c0.1-0.1,0.1-0.2,0.1-0.4c0-0.2,0-0.3,0-0.5c0-0.4,0-0.7,0.4-0.9c-0.3,1.8,0.6,0.2,0.8,1c0,0,0-0.1,0-0.1c0-0.2,0-0.5,0.1-0.7\n              c0.1-0.1,0.2,0.1,0.2,0.2c0.8-2.4-0.9-0.9-0.8-2.8c0.4,0,0.9-1,0.8-1.3c0,0.9-0.8,0.8-1.1,0.4c0.2-1,0.1-1.2-0.1-1.3\n              c-0.2,0.1-0.5,0.1-0.7,0.1l0.4,0.1l0,0.2c-0.1,0-0.1,0-0.2,0c-0.5,0.2-1,0.2-1.7,0.2c0.2-0.3,0.9-0.2,0.7-0.3\n              c-0.8,0.3-1,0.1-1.4-0.1c-0.4-0.3-0.9-0.5-1.7-0.3c-0.5-0.3,0.2-0.8-0.4-1.1c-0.4,0.1-0.8,0.1-1,0.1c-0.2,0-0.3-0.1-0.4-0.2\n              c-0.3-0.2-0.5-0.4-1-0.2c0.3,0.6,0.3,0.5-0.4,1l1.7-0.3l-1.1,0.8c0.5,0,1-0.3,1.4-0.2c-0.6,0.4-0.5,0.3-0.3,0.8\n              c-0.6-0.9-1.4,0.4-2.1-0.2l0.1-1.5c-1.2-0.7-2.2,0.6-3.7,0.2l0.5,0.3c-0.2,0.6-1,0-1.5,0.2c0.1-0.5-0.2-0.7-0.4-1\n              c0,0.4-1.1-0.1-1.3,0.9l-0.7-0.7c-1.2-0.3-1.5,1.3-2.7,1.1c0.4-0.5-0.1-0.9,0.7-1.2c-0.4,0-0.8-0.1-0.8,0.2\n              c-0.2-0.3-1.1,0.6-1.5,0.1c-0.2,0.2-0.3,0.7-0.7,0.6c0-0.1,0-0.2,0-0.2c0,0.2-0.7,0.3-0.4,0.9c-1.2-1.6-3.5-0.1-4.5-1.5\n              c-0.9,0.1-1.9,0.2-2.7,0.2c0.1-0.1,0-0.2,0.2-0.2c-1.1-0.5-0.8,1.4-2,0.8c-0.6-0.8,0.7-0.4,0.3-0.7c-0.1-1.6-1.5,0.1-2.3-0.9\n              l0.2-0.1c-0.6-0.6-1.6,0.5-2.6,0.3c0,0,0.1-0.1,0-0.2c-0.8,0.8-2.1,1-3.2,1.1c0.3-0.6,0.5-0.3,0.3-1c-0.5,0.1,0.1,1-0.8,1.2\n              c-0.3-0.4-0.9-0.7-1-1.4l0.9,0c-0.4-0.9-1.1,0-1.5-0.2l0.1-0.4c-1.3,0.1-1.4,1-2.8,0.9l0.1,0.3c-0.4,0.5-0.6,0.3-0.7,0.1\n              c-0.1-0.1-0.1-0.2-0.3-0.2c-0.1,0-0.3,0-0.5,0.2c-0.1-0.3-0.2-0.7-0.4-1c-0.4,0.1-1,0.3-1.5,0.5c-0.6,0.2-1.1,0.4-1.5,0.5\n              c0-0.7,1.4-1,2.3-1.4c-0.8-0.1-2.7-0.1-2.7,0.4c0.2-0.1,0.5-0.3,0.8-0.1c-0.3,0.5-0.7,0.8-1.3,1c0.1,0.4,0.1,0.8,0,1.2\n              c-0.9-0.9-1.1-0.1-1.2,1c-0.1,0.5-0.1,1.1-0.3,1.7c-0.1,0.3-0.2,0.5-0.3,0.7c-0.1,0.2-0.3,0.3-0.5,0.4c0.6,1.8-0.5,3.5-0.6,5.5\n              c1.5-0.6,0,0.6,1.5,0.6c0.1,0.5,0,0.7-0.1,0.7l0.1-0.1c0,0-0.1,0.1-0.1,0.1l0,0l-1,1.4c-0.5,0.3-0.3-0.5-0.6-0.5\n              c-0.2,1-0.7,0.4-1.1,1.1c0.6-0.1,0.8,0.4,0.5,1l0.8-0.8c0.7,1.5-1.4,1.2-0.7,2.7l-0.4-0.5c0.2,0.8-0.2,2.8,1,3.6\n              c-0.2,0.1-0.6,0.3-0.7-0.1c-0.3,1.8,1,4.1,0.2,5.5l-0.4-0.2c0.1,0.3,0.7,0.6,0.3,1c-0.2,0-0.4-0.2-0.4,0c0.1,0.2,0.6,0.9,0.3,1.4\n              l-0.4-0.4c-0.6,1.7,0.8,2.8,1,4.3c-1.1-0.1-1.5,1.3-1.8,2l-0.2,0c-0.3,2,0.7,3.6,1,5.5c-1.3,1.1-0.8,4-1.6,5.8\n              c0.2-0.3,0.8,0.1,0.9,0.5c-0.3,0.4,0.7,1.3-0.3,1.1l0-0.2c-1,0.3-0.8,1.2-0.9,1.9l1.3,0.2c-2,1.7,0.4,5-1.4,5.9\n              c0.3,0.5,0.3,1,0.5,1.5l-0.7-0.1c-0.3,1.2-0.4,1.9-0.3,3.4l-0.5-0.4c-0.1,0.9,1.9,1.2,1.9,2l-1.1,0.2c-0.1,1.5-1.1,3-0.3,4.6\n              c-0.2,0-0.5-0.1-0.5-0.4c-0.1,0.7-0.9,2.7,0.6,2.6c-0.5,0-0.5,0.4-0.4,0.9c0.1,0.3,0.1,0.9,0.2,1.2c0.1,0.4,0.2,0.8,0.3,1.1\n              c-0.2,0-0.5,0-0.7,0c0.4,0.5,0.7,0.6,0.9,0.9c0.2,0.2,0.3,0.5,0.4,1.1c-0.1,0.3-0.5,0.6-0.4,0.3c-0.3,1,0.6,0.3,0.3,1.6\n              c0,0,0.1,0,0.1,0l-0.3,0.1c-0.1,0-0.2,0-0.3-0.1c0,0.2,0,0.3,0,0.4c0.8,0,1.1-0.3,2-0.7c0.4-0.1,0.9-0.1,1.2,0.1\n              c0.3,0.2,0.6,0.4,0.6,0.7c0.4-0.4,1.1-0.9,1.7-0.7c0,0.2-0.2,0.4-0.4,0.5c0.5,0.1,0.9,0.1,1.3,0.1c0.3,0,0.5-0.1,0.8-0.2\n              c0.4-0.2,0.8-0.3,1.3,0c0,0.2,0.1,0.3,0.2,0.4c-1,0-1.9-0.1-1.8-0.1c0,0.6,0.8,0.7,0.2,1.6c0.7-0.5,1.8-1.5,2.7-1.2\n              c0,0.1,0.1,0.1,0.1,0.2l-0.1,0.1c0.2,0.1,0.4,0,0.5-0.1c0.3,0.1,0.4,0.2,0.7,0.2c0.1,0.5,1,0.3,1.4,0.3c0-0.7,0.5-0.6,0-1.2\n              c0.1,0.1-0.1,0.1-0.4,0.1c-0.1,0-0.1,0-0.2,0.1c0,0,0,0,0-0.1c-0.3,0-0.6,0-1,0c0.3-0.4,0.7-0.7,1.1-0.3c0.2-0.4,0.5-0.9,0.6-1.1\n              c0.6,1,1.7,1.5,1.9,2.1c1.3-0.4,3-0.3,3.8-1.6c0.4-0.2,0.4,0.4,0.6,0.6c-0.2,0.1-0.5,0.1-0.6,0.3c0.7-0.3,2.2-0.1,2.3-1\n              c0.9,0.4-0.5,0.8,0.8,0.7l-0.2,0.3c2.5,1.5,4.1-2.3,6.8-0.5c-0.2-0.1-0.3-0.2-0.3-0.3c3.2,0.8,6.5-0.8,9.6,0.5\n              c0.9,0.3,1-0.3,1.5-0.8l0.2,0.7l0.9-0.9c1.3-0.6,2.3,1.8,3.9,0.6l-0.2-0.5c0.6,0.1,1.9-0.3,1.8,0.3c0.1-0.1,0.2-0.5,0.4-0.3l0,0.4\n              l1.5-0.7c1.1,0.3-0.2,1.6,1.5,1.6c0.6-0.2,1.8-1.1,1.5-1.4c-0.2,0-0.6-0.2-0.7-0.6l0.6-0.2c0-0.1,0-0.1,0-0.2h0.1l0-1\n              c0,0,0-0.1,0-0.1l0.1-5.5l0.2,0.1l-0.2-0.2l0-0.8c0.1,0,0.3,0.1,0.3,0.3c0.5-0.6-0.4-0.6,0.2-1.1c0.1-0.1,0.1,0,0.2,0l-0.3-0.6\n              c0.2,0.1,0.7-0.1,0.6,0.3c0.6-1.1-0.7-1.8,0.1-2.2l0.1,0.1c-0.2-1.1-0.5-0.7-1-0.8l0-0.7l1.3-0.5l-0.8-0.2c0-0.2,0.1-0.5,0.4-0.4\n              c-0.5-0.6-0.3,0.1-0.8-0.3c0.7-0.7-0.3-2.1,0.9-2.2c-0.6-0.3-0.5,0.7-0.8-0.1c1.3-0.2-0.2-0.9,0.4-1.6c0.7,0.2,0.3,0.9,0.9,0.2\n              c-1.1-0.4,0.3-1.3,0.1-1.9c-0.9,0-0.3-1.3-0.6-2.3c0.3,0.1,0.5-1.8,0.9-2.7l0.2,0c-0.9-0.2-0.7-1.1-0.9-1.8c0.5,0,0.8,0.2,1,0.5\n              l0-0.6c-0.3-0.2-0.4-0.4-0.5-0.8l0.7,0.2c0-0.9-0.8-1.7-0.3-2.8c-0.4-0.1-1.1,0.3-1.6,0.2l0-0.6c0.2,0,0.4,0.1,0.5,0.3\n              c0-0.1-0.3-0.4-0.1-0.6l0.6,0.5c0.3-0.5-0.5-0.3-0.4-0.7c0.5,0,0.4-0.3,0.7-0.2c0,0.1-0.2,0.3,0,0.5c0.1-0.4,0.4-1,0.8-1.2\n              c-0.3,0-0.7,0-0.7-0.3c0-0.9,0.6-0.3,0.9-0.8c-0.3-0.2,0-0.8-0.7-0.8c0-0.2,0.4-0.3,0.6-0.4c-0.3-0.4,0-2.1-0.8-1.6l-0.1,0.1\n              c0,0,0,0,0-0.1c-0.7-1.5,0.5-3,0.2-4.4l0.2,0c-0.3-0.9-0.1-1.9-0.6-2.5c0-0.3,0.5-0.6,0.8-0.5c-0.2-0.7-0.4-0.7,0-1.7\n              c0.4,0.7,0.4-0.2,0.3,0.7c0.2-0.8,0.6-1,0-2c0.6,0.3,0.8-0.6,0.6-1l-0.5,0.6c-0.1-0.5,0-0.9,0-1.3l0.8,0.2c1-1.7-1.6-3.2,0.1-4.5\n              c-0.6-0.5-0.4,0.9-1.1,0.5c0.1-0.8,1.3-1.6,1.3-1.9c0.5-1.1-1.2-1.1-0.8-2.2c-0.3,0.1-1.1,0.3-1.3-0.6c0.1-0.8,1.3-1,1.5-0.4\n              c0,0.2-0.1,0.3-0.2,0.3c0.1,0.3,0.2,0.6,0.5,0.3l-0.3-0.1c0.3-0.4,0.3-1.5,0.8-1.2c-0.3-0.6-0.5-0.3-0.6,0.2c0.5-1,0.1-1.6,0.4-2.7\n              c-0.2,0.2-0.5,0.3-0.5,0.1c0.4-1.5-0.5-0.7-0.6-1.8c0.5-0.6,1.1,0.2,0.7-0.8c0.3-0.6,0.8-0.1,1,0.1c-0.5-1,0.4-1.8,0.5-2.3\n              L354.9,39.5z M311,105.7C311,105.7,311,105.7,311,105.7c0,0,0,0.2-0.1,0.3c-0.3,0.1-0.4,0-0.4-0.2\n              C310.6,105.8,310.8,105.8,311,105.7z M350.1,94.3c0,0-0.1,0.1-0.1,0.1l0-0.3C350,94.1,350,94.2,350.1,94.3z\"></path>\n          <path class=\"command-bar__logo__box\"\n                d=\"M406.4,101.2L406.4,101.2c-0.3-1-0.5-0.5-1-0.7l0-0.7l1.3-0.6l-0.8-0.1c0-0.2,0.1-0.5,0.4-0.4\n              c-0.6-0.6-0.3,0.1-0.8-0.2c0.6-0.8-0.4-2.1,0.8-2.3c-0.6-0.2-0.4,0.8-0.8,0c1.3-0.3-0.3-0.9,0.3-1.7c0.7,0.2,0.4,0.9,0.9,0.1\n              c-1.2-0.4,0.2-1.4-0.1-1.9c-0.9,0-0.4-1.3-0.8-2.2c0.3,0,0.4-1.9,0.7-2.8l0.2,0c-0.9-0.1-0.8-1.1-1-1.8c0.5,0,0.8,0.1,1,0.5l0-0.6\n              c-0.3-0.1-0.5-0.3-0.6-0.7l0.7,0.1c-0.1-0.9-0.9-1.7-0.5-2.8c-0.4,0-1.1,0.4-1.6,0.4l0-0.6c0.2,0,0.4,0.1,0.5,0.2\n              c-0.1-0.1-0.3-0.3-0.2-0.6l0.6,0.4c0.2-0.5-0.5-0.2-0.5-0.6c0.5,0,0.3-0.3,0.7-0.3c0.1,0.1-0.2,0.4,0,0.5c0.1-0.5,0.3-1,0.7-1.2\n              c-0.3,0.1-0.7,0-0.7-0.3c-0.1-0.9,0.6-0.3,0.8-0.8c-0.4-0.2,0-0.8-0.8-0.7c0-0.3,0.3-0.3,0.5-0.4c-0.3-0.4-0.2-2.1-1-1.5l0,0.1\n              c0,0,0,0,0-0.1c-0.8-1.4,0.2-3-0.1-4.5l0.2,0c-0.3-0.8-0.3-1.9-0.8-2.5c0-0.3,0.4-0.6,0.7-0.5c-0.2-0.6-0.5-0.7-0.1-1.7\n              c0.5,0.6,0.4-0.2,0.4,0.7c0.2-0.9,0.5-1.1-0.2-2c0.6,0.3,0.8-0.7,0.5-1.1l-0.4,0.6c-0.1-0.5-0.1-0.9-0.1-1.3l0.8,0.2\n              c0.9-1.8-1.9-3.1-0.3-4.5c-0.6-0.4-0.4,1-1.1,0.6c0.1-0.8,1.2-1.7,1.2-2c0.4-1.1-1.3-1-1-2.1c-0.3,0.2-1.1,0.3-1.4-0.5\n              c0.1-0.8,1.2-1.1,1.4-0.6c0,0.2-0.1,0.3-0.2,0.3c0.1,0.3,0.3,0.6,0.6,0.3l-0.3-0.1c0.3-0.4,0.2-1.5,0.7-1.3\n              c-0.3-0.6-0.5-0.2-0.6,0.2c0.4-1.1-0.1-1.6,0.2-2.7c-0.2,0.2-0.5,0.4-0.5,0.2c0.3-1.6-0.6-0.7-0.8-1.7c0.4-0.6,1.1,0.2,0.6-0.9\n              c0.3-0.7,0.8-0.2,1,0.1c-0.6-0.9,0.3-1.8,0.3-2.4l0.3,0.5c-0.2-0.3-0.2-0.5-0.2-0.8l-0.6,0.3c0.1-0.3,0.1-0.6,0.3-0.6\n              c-0.4-0.3-1,0.3-1.5-0.3c0.3-0.4-0.4-1.2,0.4-1.4c-0.2,1.3,0.8-0.2,1,0.9c-0.1-0.2-0.1-0.5,0-0.8c0.1-0.1,0.2,0,0.2,0.1\n              c0.6-1.2-0.9-0.5-1-1.8c0.4,0.1,0.8-0.3,0.7-0.5c0,0.5-0.7,0.3-1.1-0.2c0.3-0.8-0.3-0.9-0.3-1.3l-0.1,0.5c-0.4-0.1-0.4-0.5-0.6-1.1\n              c0.3-0.1,0.3,0.5,0.4,0.3c-0.2-0.1-0.3-0.3-0.2-0.5c0-0.1,0-0.2,0.1-0.3c0-0.1,0.1-0.3,0.1-0.4c0.2-0.6,0.4-1.3,0.1-1.9\n              c0.3-0.8,0.8-0.4,1-1.1c-0.3-0.7-0.2-1.2-0.1-1.8c0.1-0.6,0.3-1.3,0-1.9c0,0-0.1,0-0.1,0c0,0,0,0.1,0,0.1c-0.3,0.7-0.4,0.8-0.6,0.6\n              c-0.2,0.1-0.5,0.2-0.9,0.4l0.7-0.1l0.4,1.7c-0.3-0.4-0.6-0.7-0.9-0.9c0,0.7,0.4,1.2,0.4,1.8c-0.5-0.4-0.4-0.4-0.8,0.3\n              c0.5-1,0.1-1.4-0.1-1.7c-0.4,0.1-0.8,0.2-1,0l1.1-1.5c-1.6-0.6-4.1,0.9-5.8,0.6l0.5,0.2c-0.2,0.6-1,0.1-1.5,0.4c0-0.5-0.3-0.7-0.5-1\n              c0,0.4-1.1,0-1.2,1l-0.7-0.6c-1.3-0.2-1.4,1.5-2.6,1.3c0.3-0.5-0.2-0.9,0.6-1.3c-0.4,0-0.8-0.1-0.8,0.3c-0.2-0.3-1.1,0.7-1.5,0.3\n              c-0.2,0.2-0.2,0.7-0.7,0.7c-0.1-0.1,0-0.2,0-0.2c0,0.2-0.7,0.4-0.3,0.9c-1.3-1.5-3.5,0.3-4.6-1c-0.9,0.2-1.9,0.4-2.7,0.5\n              c0-0.1,0-0.2,0.1-0.2c-1.1-0.4-0.6,1.4-1.9,1c-0.7-0.8,0.6-0.5,0.2-0.7c-0.2-1.6-1.5,0.3-2.4-0.6l0.2-0.1c-0.7-0.5-1.6,0.7-2.6,0.6\n              c0,0,0-0.1,0-0.2c-0.7,0.9-2,1.2-3.1,1.5c0.3-0.6,0.4-0.4,0.2-1c-0.5,0.1,0.2,1-0.7,1.2c-0.3-0.3-1-0.6-1.1-1.3l0.9-0.1\n              c-0.5-0.9-1.1,0.1-1.6-0.1l0-0.4c-1.3,0.2-1.3,1.2-2.7,1.1l0.2,0.3c-0.7,1-0.8-0.5-1.6,0.2c-0.2-0.3-0.4-0.6-0.6-1\n              c-0.3,0.1-0.8,0.4-1.3,0.6c-0.5,0.2-0.9,0.5-1.3,0.6c0,0.1,0,0.2,0,0.2c-0.1-0.1-0.1-0.2-0.2-0.4c0.1-0.6,1.2-1,1.9-1.4\n              c-0.8,0-2.7,0.2-2.6,0.7c0.2-0.1,0.4-0.4,0.8-0.2c-0.1,0.3-0.2,0.5-0.4,0.7c0.3,0.9,0.4,1.7,0.4,2.5c-0.9-0.7-1.1,0.2-1.2,1.4\n              c-0.1,1.3-0.1,1.9-0.9,2.4c0.7,1.8-0.4,3.5-0.3,5.6c1.5-0.7,0,0.6,1.5,0.5c0.1,0.5,0.1,0.7,0,0.7l0.1-0.1c0,0,0,0.1-0.1,0.1l0,0\n              l-0.9,1.4c-0.4,0.3-0.3-0.5-0.7-0.4c-0.1,1.1-0.7,0.4-1.1,1.1c0.5-0.1,0.8,0.4,0.5,1l0.7-0.8c0.8,1.5-1.3,1.3-0.5,2.8l-0.5-0.5\n              c0.3,0.8-0.1,2.8,1.1,3.6c-0.2,0.1-0.6,0.3-0.7,0c-0.2,1.8,1.2,4.1,0.4,5.5l-0.4-0.1c0.1,0.3,0.7,0.6,0.3,1c-0.2,0-0.4-0.1-0.4,0\n              c0.1,0.2,0.6,0.9,0.4,1.4l-0.4-0.4c-0.5,1.8,1,2.8,1.2,4.3c-1.1,0-1.5,1.4-1.7,2.1l-0.2,0c-0.2,2.1,0.8,3.5,1.2,5.5\n              c-1.2,1.2-0.6,4-1.3,5.9c0.2-0.3,0.9,0.1,1,0.4c-0.2,0.4,0.7,1.3-0.3,1.2l0-0.2c-1,0.3-0.7,1.2-0.8,2l1.3,0.1c-1.9,1.8,0.6,5-1.1,6\n              c0.3,0.5,0.3,1,0.6,1.5l-0.7,0c-0.2,1.3-0.3,1.9-0.2,3.5l-0.5-0.3c-0.1,0.9,2,1.2,2,2l-1.1,0.2c0,1.5-1,3.1-0.1,4.6\n              c-0.2,0-0.5-0.1-0.5-0.3c0,0.7-0.8,2.8,0.7,2.6c-0.9,0-0.1,1.6,0.3,2.4l-0.7-0.1c0.9,0.7,1.1,0.7,1.3,1.7c-0.1,0.2-0.4,0.2-0.4,0\n              c-0.1,0.3,0,0.4,0.2,0.7c0.2,0.2,0.3,0.5,0.1,1.1c-0.1-0.1-0.3-0.3-0.4-0.4c-0.1,1.2,0.2,1.1,0.7,1.4c0.1,0.3,0.1,0.8-0.1,1.3\n              c-0.2,0.5-0.5,1.2-0.8,1.5c0.4-0.1,0.9,0,0.7,0.7l0,0c-0.1,0.1-0.2,0.2-0.3,0.3c-0.1,0-0.1,0-0.2,0c0,0.1,0,0.2-0.1,0.3\n              c0.8,0.1,1.6-0.2,2.3-0.4c0.7-0.2,1.2-0.4,1.7-0.1c-0.1,0.2,0,0.3,0.1,0.4c-1.6,0.1-2.9,0.2-2.7,0.1c-0.2,0.3,0,0.5,0,0.7\n              c0.1,0.2,0,0.5-0.9,1c0.8-0.3,1.9-0.8,2.8-1.1c0.9-0.4,1.7-0.6,2.4-0.5c0.1,0.1,0.1,0.1,0.2,0.2l-0.1,0.1c0.3,0,0.6,0,0.8-0.2\n              c0.3,0.1,0.4,0.1,0.7,0.1c0.1,0.5,1,0.2,1.4,0.2c-0.1-0.7,0.5-0.6-0.1-1.2c0.1,0-0.1,0.1-0.4,0.1c-0.1,0-0.1,0-0.2,0.1\n              c0,0,0,0,0-0.1c-0.3,0-0.6,0.1-1.1,0.1c0.3-0.4,0.6-0.7,1.1-0.4c0.2-0.5,0.4-0.9,0.5-1.2c0.7,0.9,1.8,1.4,2,2c1.3-0.5,3-0.5,3.7-1.9\n              c0.4-0.2,0.5,0.3,0.6,0.5c-0.2,0.1-0.5,0.1-0.6,0.3c0.7-0.4,2.2-0.3,2.3-1.2c1,0.3-0.5,0.9,0.8,0.6l-0.2,0.3c2.6,1.3,3.9-2.6,6.8-1\n              c-0.2-0.1-0.3-0.1-0.4-0.3c3.2,0.6,6.5-1.3,9.7-0.2c0.9,0.2,1-0.4,1.5-0.9l0.2,0.7l0.9-0.9c1.3-0.7,2.4,1.7,4,0.3l-0.3-0.5\n              c0.6,0,1.9-0.4,1.8,0.1c0.1-0.1,0.2-0.5,0.4-0.3l0,0.4l1.4-0.8c1.2,0.2-0.1,1.6,1.6,1.5c0.6-0.2,1.7-1.3,1.4-1.5\n              c-0.3,0-0.7-0.2-0.7-0.6l0.5-0.2c0-0.1,0-0.1,0-0.2l0.1,0l-0.1-1c0-0.1,0-0.2,0-0.3l-0.3-5.4l0.2,0.1l-0.2-0.2l0-0.8\n              c0.1,0,0.3,0.1,0.3,0.2c0.5-0.7-0.4-0.5,0.1-1.1c0.1-0.1,0.1,0,0.2,0l-0.3-0.6c0.2,0.1,0.7-0.1,0.6,0.2\n              C407,102.2,405.6,101.6,406.4,101.2z M367.3,117.9C367.3,117.9,367.3,117.8,367.3,117.9c0,0,0,0.2-0.1,0.3c-0.3,0.1-0.4,0-0.4-0.1\n              C366.9,118,367.1,117.9,367.3,117.9z M405.5,103.6l0-0.3c0,0.1,0.1,0.2,0.1,0.3C405.6,103.5,405.6,103.5,405.5,103.6z\"></path>\n          <path class=\"command-bar__logo__box\"\n                d=\"M463.5,98.7L463.5,98.7c-0.3-1-0.5-0.5-1-0.6l-0.1-0.7l1.2-0.6l-0.8-0.1c0-0.2,0-0.5,0.4-0.4\n              c-0.6-0.5-0.3,0.1-0.8-0.2c0.6-0.8-0.5-2,0.7-2.3c-0.6-0.2-0.4,0.7-0.8,0c1.3-0.3-0.3-0.9,0.2-1.6c0.7,0.2,0.4,0.8,0.9,0.1\n              c-1.2-0.3,0.1-1.3-0.1-1.9c-0.9,0-0.5-1.2-0.9-2.1c0.3,0,0.4-1.8,0.6-2.7l0.2,0c-0.9-0.1-0.8-1-1.1-1.7c0.5,0,0.8,0.1,1,0.4\n              l-0.1-0.6c-0.3-0.1-0.5-0.3-0.6-0.7l0.7,0.1c-0.1-0.8-0.9-1.6-0.6-2.7c-0.4,0-1.1,0.4-1.6,0.4l0-0.6c0.2,0,0.4,0,0.5,0.2\n              c-0.1-0.1-0.3-0.3-0.2-0.5l0.6,0.4c0.2-0.5-0.5-0.2-0.5-0.6c0.5,0,0.3-0.3,0.7-0.3c0.1,0.1-0.2,0.4,0,0.5c0.1-0.4,0.3-1,0.7-1.2\n              c-0.3,0.1-0.7,0.1-0.7-0.3c-0.1-0.8,0.6-0.3,0.8-0.8c-0.4-0.2,0-0.7-0.8-0.7c0-0.2,0.3-0.3,0.5-0.4c-0.4-0.4-0.3-2-1-1.5l0,0.1\n              c0,0,0,0,0-0.1c-0.8-1.3,0.2-3-0.2-4.3l0.2,0c-0.3-0.8-0.3-1.8-0.9-2.4c0-0.2,0.4-0.6,0.7-0.5c-0.2-0.6-0.5-0.7-0.2-1.6\n              c0.5,0.6,0.4-0.3,0.4,0.6c0.1-0.8,0.5-1.1-0.2-1.9c0.7,0.3,0.7-0.7,0.5-1.1l-0.4,0.6c-0.1-0.4-0.1-0.8-0.1-1.2l0.8,0.1\n              c0.9-1.8-2-2.9-0.4-4.3c-0.6-0.4-0.3,1-1,0.6c0-0.8,1.2-1.7,1.1-1.9c0.4-1.1-1.3-0.9-1.1-2c-0.3,0.2-1.1,0.4-1.4-0.4\n              c0-0.8,1.2-1.1,1.4-0.6c0,0.2,0,0.3-0.1,0.3c0.1,0.2,0.3,0.6,0.6,0.2l-0.3-0.1c0.3-0.4,0.2-1.5,0.7-1.3c-0.3-0.6-0.5-0.2-0.6,0.2\n              c0.3-1-0.1-1.6,0.1-2.6c-0.1,0.2-0.5,0.4-0.5,0.2c0.2-1.5-0.6-0.6-0.8-1.6c0.4-0.6,1.1,0.1,0.6-0.9c0.3-0.7,0.8-0.2,1,0\n              c-0.6-0.9,0.2-1.8,0.3-2.3l0.3,0.5c-0.2-0.3-0.2-0.5-0.2-0.8l-0.6,0.3c0.1-0.3,0.1-0.5,0.3-0.6c-0.4-0.3-1,0.3-1.5-0.2\n              c0.3-0.4-0.4-1.2,0.4-1.3c-0.2,1.2,0.8-0.2,1,0.8c-0.1-0.2-0.1-0.4,0-0.7c0.1-0.1,0.2,0,0.2,0.1c0.6-1.2-0.9-0.5-1-1.7\n              c0.5,0.1,0.8-0.3,0.7-0.5c0,0.5-0.7,0.3-1.1-0.1c0.3-0.8-0.3-0.8-0.3-1.2l-0.1,0.5c-0.5-0.1-0.4-0.5-0.6-1.2\n              c0.3-0.1,0.3,0.6,0.4,0.3c-0.4-0.4-0.3-0.8-0.1-1.4c0.2-0.6,0.4-1.4,0.1-1.9c0.2-0.8,0.8-0.6,0.9-1.4c-0.4-0.7-0.2-1.3-0.1-2.1\n              c-0.3-0.1-0.7-0.2-1.6,0.1c0.1,0.6,0.2,0.4-1.3,1.1l2.2-0.4l0.1,0.5c-0.1-0.1-0.2-0.2-0.3-0.3c-0.5,0.2-1,0.4-1.4,0.6\n              c0.3,0,0.7-0.1,1.1-0.2c0.1,0.3,0.2,0.6,0.2,0.8c-0.5-0.3-0.4-0.3-0.8,0.5c0.2-0.4,0.2-0.7,0.2-0.9c-0.3,0-0.8,0.2-1.3,0.3\n              c-0.4,0.1-0.8,0.2-1.1,0l0.8-1.5c-0.8-0.3-1.8,0-2.8,0.2c-0.2,0.1-0.5,0.1-0.7,0.2c-0.1,0-0.2,0.1-0.3,0.1l-0.2,0.1\n              c-0.3,0.1-0.6,0.1-1,0.1l0.5,0.2c-0.2,0.6-0.9,0.1-1.4,0.4c0-0.5-0.3-0.7-0.5-0.9c0,0.4-1.1,0-1.2,1l-0.7-0.6\n              c-1.2-0.2-1.3,1.5-2.5,1.4c0.3-0.6-0.2-0.9,0.5-1.3c-0.4,0-0.7-0.1-0.7,0.3c-0.2-0.3-1,0.7-1.4,0.3c-0.2,0.2-0.2,0.7-0.6,0.7\n              c-0.1-0.1,0-0.2,0-0.2c0,0.2-0.7,0.4-0.3,0.9c-1.3-1.5-3.4,0.3-4.5-0.9c-0.8,0.2-1.9,0.4-2.6,0.5c0-0.1,0-0.2,0.1-0.2\n              c-1.1-0.4-0.6,1.4-1.8,1c-0.7-0.8,0.6-0.5,0.2-0.7c-0.3-1.6-1.5,0.3-2.3-0.6l0.2-0.1c-0.7-0.5-1.5,0.7-2.5,0.6c0,0,0-0.1,0-0.2\n              c-0.7,0.9-2,1.2-2.9,1.5c0.3-0.6,0.4-0.4,0.2-1c-0.4,0.1,0.2,1-0.7,1.3c-0.3-0.3-1-0.6-1.1-1.3l0.8-0.1c-0.5-0.9-1.1,0.1-1.5,0\n              l0-0.4c-1.3,0.2-1.2,1.2-2.6,1.2l0.2,0.3c-0.7,1-0.7-0.5-1.4,0.3l-0.3-1c-0.5,0.3-1.7,1-2.5,1.3c0.5-0.8,1.5-1.2,2.1-1.7\n              c-0.5,0-1.7,0.2-1.9,0.7c0.2-0.1,0.4-0.4,0.6-0.2c-0.3,0.4-0.6,0.7-1,0.9c-0.2,0.1-0.4,0.2-0.7,0.3s-0.5,0.1-0.8,0.1\n              c-0.2-0.8-0.9-1-1.9-1c-1,0-2,0.1-3.2-0.2c-0.1,0.1-0.2,0.2-0.4,0.2c0.4,1.7,0.4,3,0.3,4.4c0,0.4,0,0.7-0.1,1.1l0,0.6l0,0.4\n              c0,0.5-0.1,1,0,1.5c1.4-0.8,0.1,0.6,1.6,0.4c0.1,0.5,0.1,0.6,0,0.7l0.1-0.1c0,0,0,0.1-0.1,0.1l0,0l-0.8,1.5\n              c-0.4,0.4-0.4-0.4-0.7-0.4c-0.1,1-0.6,0.5-1,1.2c0.5-0.2,0.8,0.3,0.6,0.9l0.7-0.9c0.9,1.4-1.2,1.3-0.3,2.7l-0.5-0.4\n              c0.3,0.7,0.1,2.7,1.4,3.4c-0.2,0.1-0.6,0.3-0.7,0c-0.1,1.8,1.5,3.9,0.8,5.3l-0.4-0.1c0.1,0.3,0.7,0.5,0.4,1c-0.2,0-0.4-0.1-0.4,0\n              c0.1,0.2,0.7,0.8,0.5,1.3l-0.4-0.4c-0.4,1.8,1.1,2.6,1.5,4c-1.1,0-1.4,1.4-1.5,2.1l-0.2,0c0,2,1.1,3.4,1.6,5.2\n              c-1.1,1.2-0.3,3.9-0.9,5.8c0.2-0.3,0.9,0,1,0.3c-0.2,0.4,0.8,1.2-0.2,1.1l0-0.2c-1,0.4-0.7,1.3-0.7,2l1.3,0.1\n              c-1.8,1.9,0.9,4.8-0.7,5.8c0.3,0.4,0.4,1,0.7,1.4l-0.7,0c-0.1,1.2-0.2,1.8,0.1,3.4l-0.5-0.3c0,0.9,2.1,1,2.2,1.8l-1,0.3\n              c0.1,1.5-0.8,3,0.3,4.5c-0.2,0-0.5,0-0.5-0.3c0,0.7-0.6,2.7,0.8,2.4c-0.9,0.1,0,1.5,0.4,2.3l-0.7-0.1c0.9,0.6,1.2,0.8,1.5,2\n              c-0.1,0.3-0.4,0.4-0.4,0.2c-0.1,1,0.7,0.5,0.4,1.8c-0.2-0.1-0.3-0.1-0.5-0.2c0,1.2,0.2,1,0.8,1.1c0.1,0.1,0.1,0.3,0.1,0.5\n              c0.6,0,1.1,0.4,0.9,0.9c0.4-0.4,1.3-0.8,1.8-0.6c0,0.2-0.3,0.4-0.5,0.4c0.8,0.3,1.7,0.1,2.3-0.1c0.4-0.1,0.5-0.2,0.7-0.2\n              c0.2,0,0.4,0,0.6,0.2c0,0.2,0.1,0.3,0.1,0.4c-0.5,0-1-0.1-1.3-0.1c-0.5,0-0.8-0.1-0.7-0.1c-0.1,0.3,0.2,0.5,0.4,0.7\n              c0.2,0.2,0.3,0.5-0.4,0.9c0.8-0.2,1.1-0.6,1.6-0.8c0.5-0.3,1-0.4,1.4-0.3c0,0.1,0.1,0.1,0.1,0.2l-0.1,0.1c0.2,0.1,0.4,0,0.5-0.1\n              c0.3,0.1,0.4,0.2,0.7,0.2c0.1,0.5,0.9,0.3,1.3,0.4c0-0.7,0.5-0.6,0-1.2c0.1,0.1-0.1,0.1-0.4,0.1c-0.1,0-0.1,0-0.2,0.1c0,0,0,0,0-0.1\n              c-0.3,0-0.6,0-1,0c0.3-0.3,0.7-0.6,1.1-0.2c0.2-0.4,0.5-0.9,0.6-1.1c0.5,1,1.6,1.6,1.7,2.2c1.3-0.3,3-0.2,3.8-1.4\n              c0.4-0.2,0.4,0.4,0.5,0.6c-0.2,0.1-0.5,0.1-0.6,0.3c0.7-0.3,2.1-0.1,2.3-1c0.9,0.4-0.5,0.8,0.7,0.7l-0.2,0.3c2.4,1.5,4-2.2,6.7-0.2\n              c-0.2-0.1-0.3-0.2-0.3-0.3c3.1,0.9,6.4-0.6,9.4,0.9c0.9,0.3,1-0.3,1.5-0.7l0.2,0.8l0.9-0.8c1.3-0.6,2.1,1.9,3.8,0.8l-0.2-0.5\n              c0.6,0.1,1.9-0.2,1.7,0.3c0.1-0.1,0.2-0.5,0.4-0.2l0,0.4l1.5-0.6c1.1,0.3-0.2,1.5,1.4,1.7c0.6-0.1,1.7-1.1,1.5-1.4\n              c-0.2,0-0.6-0.3-0.6-0.7l0.6-0.2c0-0.1,0-0.1,0-0.2l0.1,0l0-0.4c0.1-0.7,0.2-1.4-0.2-2l-0.3-4.2l0.2,0.1l-0.2-0.2l-0.1-0.8\n              c0.1,0,0.3,0.1,0.4,0.2c0.5-0.6-0.4-0.5,0.1-1.1c0.1-0.1,0.1,0,0.2,0l-0.4-0.6c0.2,0.1,0.7-0.2,0.6,0.2\n              C464.1,99.8,462.7,99.2,463.5,98.7z M425.4,110.9C425.4,110.9,425.5,110.8,425.4,110.9c0,0,0,0.2-0.1,0.3c-0.3,0.1-0.4-0.1-0.4-0.2\n              C425.1,110.9,425.2,110.9,425.4,110.9z M462.7,101.1l0-0.3c0,0.1,0.1,0.2,0.1,0.2C462.7,101,462.7,101.1,462.7,101.1z\"></path>\n          <path class=\"command-bar__logo__box\"\n                d=\"M524.3,35.4c0-0.1,0-0.1,0-0.2l0-0.4c-0.2,0.2-0.4,0.4-0.6,0.5c0.1-0.4,0.1-0.9,0.3-0.9c-0.2-0.2-0.4-0.1-0.7,0\n              c-0.3,0.1-0.5,0.1-0.8-0.3c0.1-0.3,0.1-0.7,0-1.1c0-0.4,0-0.7,0.4-1.1c-0.1,1,0.1,0.9,0.3,0.7c0.2-0.2,0.4-0.4,0.5,0.1\n              c0-0.3,0-0.6,0-1c0.1-0.1,0.2,0,0.2,0.1c0.6-2.6-0.9-0.5-0.9-2.5c0.2-0.1,0.5-0.4,0.6-0.8c-0.1-0.1-0.3-0.2-0.5-0.1\n              c0.9,0-0.3,0.7-1.3,1c-1-0.4-1.5,0.2-2.1,0.1l0.6,0.2c-0.5,0.4-1,0.3-1.8,0.5c0.1-0.3,0.8-0.2,0.6-0.4c-0.6,0.4-0.9,0.2-1.4-0.1\n              c-0.2-0.1-0.5-0.3-0.8-0.3c-0.2-0.1-0.4-0.1-0.6,0c-0.4-0.3,0-0.8-0.3-1c-1,0.5-1-0.6-2-0.2c0.3,0.6,0.3,0.5-0.4,1l1.7-0.4l-1.1,0.8\n              c0.5,0,1.1-0.3,1.3-0.2c-0.5,0.5-0.5,0.3-0.2,0.8c-0.6-0.9-1.4,0.5-2.1-0.2l0.1-1.5c-1.2-0.7-2.1,0.7-3.6,0.3l0.5,0.3\n              c-0.2,0.6-0.9,0-1.4,0.3c0.1-0.5-0.2-0.7-0.4-1c0,0.4-1.1-0.1-1.3,0.9l-0.7-0.7c-1.2-0.3-1.5,1.3-2.6,1.1c0.4-0.5-0.2-0.9,0.7-1.2\n              c-0.4,0-0.7-0.1-0.8,0.2c-0.2-0.3-1.1,0.6-1.5,0.1c-0.2,0.2-0.2,0.7-0.7,0.7c0-0.1,0-0.2,0-0.2c0,0.2-0.7,0.3-0.3,0.9\n              c-1.2-1.6-3.4,0-4.4-1.4c-0.9,0.1-1.9,0.3-2.7,0.3c0.1-0.1,0-0.2,0.2-0.2c-1.1-0.5-0.7,1.4-1.9,0.8c-0.6-0.8,0.7-0.4,0.3-0.7\n              c-0.1-1.6-1.5,0.2-2.3-0.8l0.2-0.1c-0.6-0.6-1.6,0.6-2.6,0.4c0,0,0.1-0.1,0-0.2c-0.7,0.8-2.1,1-3.1,1.2c0.3-0.6,0.5-0.3,0.3-1\n              c-0.4,0.1,0.1,1-0.8,1.2c-0.3-0.4-0.9-0.7-1-1.4l0.8,0c-0.4-0.9-1.1,0-1.5-0.2l0.1-0.4c-1.3,0.1-1.3,1.1-2.7,0.9l0.1,0.3\n              c-0.8,0.9-0.6-0.6-1.4,0.1l-0.2-1c-0.3,0.1-0.7,0.3-1.2,0.5c-0.5,0.2-1,0.4-1.6,0.5c0.1-0.2,0.4-0.3,0.6-0.5\n              c0.2-0.1,0.4-0.3,0.6-0.4c0.4-0.2,0.8-0.4,1.2-0.6c-0.5,0-1.6,0-2,0.5c0.3-0.1,0.5-0.3,0.7-0.2c-0.2,0.2-0.3,0.3-0.5,0.5\n              c-0.2,0.1-0.5,0.2-0.7,0.3c-0.5,0.2-1,0.2-1.6,0.2c-0.4-1.4-2.3-1.1-4.2-1.4c-0.1,0.5-0.3,0.8-0.7,0.8c0.8,3.2-0.1,6.1,0,7.8\n              c1.5-0.8,0.1,0.6,1.6,0.4c0.1,0.5,0.1,0.6,0,0.7l0.1-0.1c0,0,0,0.1-0.1,0.1l0,0l-0.9,1.5c-0.4,0.4-0.3-0.4-0.7-0.4\n              c-0.1,1-0.6,0.5-1,1.1c0.5-0.2,0.8,0.3,0.6,0.9l0.7-0.8c0.8,1.4-1.2,1.3-0.4,2.7l-0.5-0.4c0.3,0.7,0.1,2.8,1.3,3.5\n              c-0.2,0.1-0.6,0.3-0.7,0c-0.1,1.8,1.4,3.9,0.7,5.4l-0.4-0.1c0.1,0.3,0.7,0.6,0.3,1c-0.2,0-0.4-0.1-0.4,0c0.1,0.2,0.7,0.9,0.5,1.3\n              l-0.4-0.4c-0.4,1.8,1.1,2.7,1.4,4.1c-1.1,0-1.4,1.4-1.6,2.1l-0.2,0c-0.1,2,1,3.4,1.5,5.3c-1.2,1.2-0.4,4-1,5.9c0.2-0.3,0.9,0,1,0.4\n              c-0.2,0.4,0.8,1.2-0.2,1.2l0-0.2c-1,0.4-0.7,1.3-0.7,2l1.3,0.1c-1.9,1.9,0.8,4.8-0.9,5.9c0.3,0.4,0.3,1,0.7,1.4l-0.7,0\n              c-0.1,1.2-0.2,1.9,0,3.4l-0.5-0.3c0,0.9,2.1,1,2.1,1.8l-1,0.3c0.1,1.5-0.8,3,0.2,4.5c-0.2,0-0.5-0.1-0.5-0.3c0,0.4-0.2,1-0.1,1.6\n              c0,0.3,0.1,0.5,0.2,0.7c0.1,0.2,0.4,0.4,0.7,0.2c-0.5,0-0.5,0.7-0.3,1.5c0.2,0.8,0.5,1.6,0.7,2.1c-0.2,0.1-0.5,0.2-0.7,0.2\n              c0.8,0.5,1.1,0.4,1.3,1.1c0.1,0,0.3,0,0.5-0.1c0.2,0.1-0.1,0.4-0.4,0.4c0.7,0.2,1.3-0.6,2.2-0.2c-0.2,0.1-0.5,0.3-0.8,0.4\n              c1.1,0.1,1.2-0.1,2-0.6c0.7-0.2,1.9,0.3,1.9,0.9c0.1-0.4,0.6-0.9,1-0.7c0,0.2-0.1,0.4-0.2,0.4c1.3,0.5,2-0.7,2.8,0\n              c0,0.2,0.1,0.3,0.1,0.4c-1-0.1-1.9-0.1-1.8-0.2c0,0.6,0.8,0.8,0.1,1.6c0.7-0.4,1.9-1.4,2.7-1.1c0,0.1,0.1,0.1,0.1,0.2l-0.1,0.1\n              c0.2,0.1,0.4,0,0.5-0.1c0.3,0.1,0.4,0.2,0.7,0.2c0.1,0.5,0.9,0.3,1.4,0.4c0-0.7,0.5-0.6,0.1-1.2c0.1,0.1-0.1,0.1-0.4,0.1\n              c-0.1,0-0.1,0-0.2,0.1c0,0,0,0,0-0.1c-0.3,0-0.6,0-1,0c0.3-0.3,0.7-0.6,1.1-0.2c0.2-0.4,0.5-0.9,0.6-1.1c0.5,1,1.6,1.6,1.7,2.2\n              c1.3-0.3,3-0.1,3.9-1.4c0.4-0.2,0.4,0.4,0.5,0.6c-0.2,0.1-0.5,0.1-0.6,0.3c0.7-0.3,2.1,0,2.4-0.9c0.9,0.5-0.6,0.8,0.7,0.7l-0.2,0.3\n              c2.4,1.6,4.1-2.1,6.7-0.1c-0.2-0.1-0.3-0.2-0.3-0.3c3.1,1,6.4-0.5,9.4,1c0.9,0.3,1-0.3,1.5-0.7l0.2,0.8l1-0.8\n              c1.3-0.6,2.1,1.9,3.8,0.8l-0.2-0.5c0.6,0.1,1.9-0.2,1.7,0.4c0.1-0.1,0.2-0.5,0.4-0.2l0,0.4l1.5-0.6c1.1,0.3-0.3,1.5,1.4,1.7\n              c0.6-0.1,1.8-1.1,1.6-1.3c-0.2,0-0.6-0.3-0.6-0.7l0.6-0.2c0-0.1,0-0.1,0-0.2l0.1,0l0-0.4c0.1-0.6,0.2-1.1,0-1.7l-0.1-4.4l0.2,0.1\n              l-0.2-0.2l0-0.8c0.1,0,0.3,0.1,0.3,0.2c0.5-0.6-0.4-0.5,0.2-1.1c0.1-0.1,0.1,0,0.2,0l-0.3-0.6c0.2,0.1,0.7-0.1,0.6,0.3\n              c0.6-1.1-0.8-1.7,0-2.1l0.1,0.1c-0.3-1.1-0.6-0.6-1.1-0.8l0-0.7l1.3-0.5l-0.8-0.1c0-0.2,0.1-0.5,0.4-0.4c-0.5-0.6-0.3,0.1-0.8-0.2\n              c0.7-0.8-0.4-2,0.8-2.2c-0.6-0.3-0.4,0.7-0.8-0.1c1.3-0.3-0.2-0.9,0.3-1.6c0.7,0.2,0.3,0.8,0.9,0.1c-1.1-0.4,0.2-1.3,0-1.9\n              c-0.9,0-0.4-1.2-0.7-2.2c0.3,0.1,0.5-1.8,0.8-2.7l0.2,0c-0.9-0.2-0.8-1.1-1-1.7c0.5,0,0.8,0.1,1,0.5l0-0.6c-0.3-0.1-0.5-0.3-0.6-0.7\n              l0.7,0.1c-0.1-0.8-0.8-1.7-0.4-2.8c-0.4,0-1.1,0.4-1.6,0.3l0-0.6c0.2,0,0.4,0.1,0.5,0.2c0-0.1-0.3-0.3-0.2-0.5l0.6,0.4\n              c0.2-0.5-0.5-0.2-0.4-0.6c0.5,0,0.3-0.3,0.7-0.3c0,0.1-0.2,0.3,0,0.5c0.1-0.4,0.4-1,0.7-1.2c-0.3,0-0.7,0-0.7-0.3\n              c-0.1-0.8,0.6-0.3,0.8-0.8c-0.3-0.2,0-0.7-0.7-0.7c0-0.2,0.3-0.3,0.5-0.4c-0.3-0.4-0.1-2-0.9-1.5l0,0.1c0,0,0,0,0-0.1\n              c-0.8-1.4,0.3-3,0-4.4l0.2,0c-0.3-0.8-0.2-1.9-0.8-2.5c0-0.2,0.4-0.6,0.7-0.5c-0.2-0.6-0.5-0.7-0.1-1.6c0.4,0.6,0.4-0.2,0.3,0.7\n              c0.2-0.8,0.6-1-0.1-1.9c0.6,0.3,0.8-0.7,0.6-1l-0.4,0.6c-0.1-0.4-0.1-0.8,0-1.2l0.8,0.2c1-1.7-1.8-3.1-0.1-4.4\n              c-0.6-0.4-0.4,0.9-1.1,0.6c0.1-0.8,1.3-1.6,1.3-1.9c0.4-1.1-1.2-1-0.9-2.1c-0.3,0.2-1.1,0.3-1.4-0.5c0.1-0.8,1.2-1,1.5-0.5\n              c0,0.2-0.1,0.3-0.2,0.3c0.1,0.3,0.2,0.6,0.6,0.3l-0.3-0.1c0.3-0.4,0.2-1.5,0.7-1.2c-0.3-0.6-0.5-0.2-0.6,0.2c0.4-1,0-1.6,0.2-2.6\n              c-0.2,0.2-0.5,0.3-0.5,0.2c0.3-1.5-0.6-0.7-0.7-1.7c0.4-0.6,1.1,0.2,0.7-0.8c0.3-0.6,0.8-0.2,1,0.1c-0.6-0.9,0.3-1.8,0.4-2.3\n              l0.2,0.5C524.4,35.7,524.3,35.5,524.3,35.4z M484,99.4C484,99.4,484,99.3,484,99.4c0,0,0,0.2-0.1,0.3c-0.3,0.1-0.4-0.1-0.4-0.2\n              C483.7,99.5,483.8,99.4,484,99.4z M522.2,90c0,0-0.1,0.1-0.1,0.1l0-0.3C522.1,89.8,522.2,89.9,522.2,90z\"></path>\n          <path class=\"command-bar__logo__box\"\n                d=\"M582.3,88.7L582.3,88.7c-0.3-1-0.5-0.5-1-0.7l0-0.7l1.2-0.6l-0.8-0.1c0-0.2,0.1-0.5,0.4-0.4\n              c-0.6-0.6-0.3,0.1-0.8-0.2c0.6-0.8-0.4-2.1,0.8-2.3c-0.6-0.2-0.4,0.8-0.8,0c1.3-0.3-0.3-0.9,0.3-1.7c0.7,0.2,0.4,0.9,0.9,0.1\n              c-1.2-0.4,0.2-1.4-0.1-1.9c-0.9,0-0.4-1.3-0.8-2.2c0.3,0,0.4-1.9,0.7-2.8l0.2,0c-0.9-0.1-0.8-1.1-1-1.8c0.5,0,0.8,0.1,1,0.5l0-0.6\n              c-0.3-0.1-0.5-0.3-0.6-0.7l0.7,0.1c-0.1-0.9-0.9-1.7-0.5-2.8c-0.4,0-1.1,0.4-1.6,0.4l0-0.6c0.2,0,0.4,0.1,0.5,0.2\n              c-0.1-0.1-0.3-0.3-0.2-0.6l0.6,0.4c0.2-0.5-0.5-0.2-0.5-0.6c0.5,0,0.3-0.3,0.7-0.3c0.1,0.1-0.2,0.4,0,0.5c0.1-0.5,0.3-1,0.7-1.2\n              c-0.3,0.1-0.7,0-0.7-0.3c-0.1-0.9,0.6-0.3,0.8-0.8c-0.4-0.2,0-0.8-0.8-0.7c0-0.3,0.3-0.3,0.5-0.4c-0.3-0.4-0.2-2.1-1-1.5l0,0.1\n              c0,0,0,0,0-0.1c-0.8-1.4,0.2-3.1-0.2-4.5l0.2,0c-0.3-0.9-0.3-1.9-0.8-2.5c0-0.3,0.4-0.6,0.7-0.5c-0.2-0.7-0.5-0.7-0.2-1.7\n              c0.5,0.6,0.4-0.3,0.4,0.7c0.2-0.9,0.5-1.1-0.2-2c0.6,0.3,0.8-0.7,0.5-1.1l-0.4,0.6c-0.1-0.5-0.1-0.9-0.1-1.3l0.8,0.2\n              c0.9-1.8-1.9-3.1-0.3-4.5c-0.6-0.4-0.4,1-1,0.6c0.1-0.8,1.2-1.7,1.2-2c0.4-1.1-1.3-1-1-2.1c-0.3,0.2-1.1,0.4-1.4-0.5\n              c0.1-0.8,1.2-1.1,1.4-0.6c0,0.2-0.1,0.3-0.2,0.3c0.1,0.3,0.3,0.6,0.6,0.3l-0.3-0.1c0.3-0.4,0.2-1.5,0.7-1.3\n              c-0.3-0.6-0.5-0.2-0.6,0.2c0.4-1.1-0.1-1.6,0.1-2.7c-0.2,0.2-0.5,0.4-0.5,0.2c0.3-1.6-0.6-0.7-0.8-1.7c0.4-0.6,1.1,0.2,0.6-0.9\n              c0.3-0.7,0.8-0.2,1,0.1c-0.6-0.9,0.3-1.8,0.3-2.4l0.3,0.5c-0.2-0.3-0.2-0.6-0.2-0.8l-0.6,0.3c0.1-0.3,0.1-0.6,0.3-0.6\n              c-0.4-0.3-1,0.3-1.5-0.3c0.3-0.4-0.4-1.2,0.4-1.4c-0.2,1.3,0.8-0.2,1,0.9c-0.1-0.2-0.1-0.5,0-0.8c0.1-0.1,0.2,0,0.2,0.1\n              c0.6-1.3-0.9-0.5-1-1.8c0.2,0.1,0.4,0,0.6-0.1c0.1-0.1,0.1-0.1,0.1-0.2c0-0.1,0-0.2,0-0.3c0,0.5-0.2,0.4-0.4,0.4\n              c-0.2,0-0.5-0.3-0.7-0.7c0.3-1.4-0.3-1.3-0.3-1.9l-0.1,0.8c-0.4-0.1-0.4-0.6-0.6-1.3c0.1-0.1,0.2-0.1,0.2,0c-0.1-0.2-0.1-0.5,0-0.9\n              c0.1-0.5,0.3-1.2,0.4-1.8c-0.2,0-0.5,0-0.9,0.1c-0.4-0.3,0.6-0.8,0.1-1c-1,0.3-1.4,0.1-1.8,0c-0.4-0.2-0.9-0.3-1.8-0.1\n              c0.4,0.6,0.4,0.4-0.8,1l2.8-0.4c-0.7,0.3-1.4,0.6-2,0.9c0.7,0,1.7-0.4,2.1-0.3c-1,0.5-0.8,0.3-0.8,0.8c-0.2-0.4-0.8-0.3-1.3-0.1\n              c-0.5,0.2-1,0.4-1.5,0.1l0-1.5c-1.2-0.7-2.2,0.8-3.7,0.4l0.5,0.3c-0.2,0.6-1,0.1-1.5,0.3c0-0.5-0.3-0.7-0.5-1c0,0.4-1.1,0-1.3,0.9\n              l-0.7-0.6c-1.3-0.3-1.4,1.4-2.7,1.2c0.4-0.5-0.2-0.9,0.6-1.3c-0.4,0-0.8-0.1-0.8,0.2c-0.2-0.3-1.1,0.7-1.5,0.2\n              c-0.2,0.2-0.2,0.7-0.7,0.7c0-0.1,0-0.2,0-0.2c0,0.2-0.7,0.3-0.3,0.9c-1.3-1.5-3.5,0.1-4.6-1.2c-0.9,0.2-2,0.3-2.8,0.4\n              c0.1-0.1,0-0.2,0.2-0.2c-1.1-0.5-0.7,1.4-2,0.9c-0.6-0.8,0.7-0.4,0.3-0.7c-0.2-1.6-1.5,0.2-2.4-0.7l0.2-0.1\n              c-0.7-0.5-1.6,0.6-2.7,0.5c0,0,0.1-0.1,0-0.2c-0.7,0.8-2.1,1.1-3.1,1.3c0.3-0.6,0.5-0.3,0.3-1c-0.5,0.1,0.1,1-0.8,1.2\n              c-0.3-0.3-1-0.6-1-1.3l0.9,0c-0.4-0.9-1.1,0-1.6-0.1l0.1-0.4c-1.3,0.1-1.3,1.1-2.8,1l0.1,0.3c-0.8,1-0.8-0.5-1.7,0.2\n              c-0.2-0.3-0.4-0.7-0.6-1c-0.3,0.1-0.8,0.3-1.3,0.6c-0.5,0.2-1,0.4-1.4,0.5c0,0.1,0,0.1,0,0.2c0-0.1-0.1-0.2-0.1-0.3\n              c0-0.6,1.2-1,2-1.4c-0.8,0-2.7,0.1-2.6,0.6c0.2-0.1,0.4-0.3,0.8-0.2c-0.1,0.3-0.3,0.5-0.5,0.7c0.3,0.9,0.4,1.6,0.4,2.4\n              c-0.9-0.7-1.1,0.2-1.2,1.3c0,0.3,0,0.6-0.1,0.9c0,0.1,0,0.3-0.1,0.4c0,0.1,0,0.2-0.1,0.3c-0.1,0.4-0.3,0.7-0.7,0.8\n              c0.7,1.8-0.4,3.6-0.3,5.6c1.5-0.7,0,0.6,1.5,0.5c0.1,0.5,0.1,0.7,0,0.7l0.1-0.1c0,0,0,0.1-0.1,0.1l0,0l-0.9,1.5\n              c-0.4,0.3-0.3-0.5-0.7-0.4c-0.1,1.1-0.7,0.4-1.1,1.1c0.5-0.1,0.8,0.4,0.5,1l0.7-0.8c0.8,1.5-1.3,1.3-0.5,2.8l-0.5-0.5\n              c0.3,0.8-0.1,2.8,1.1,3.6c-0.2,0.1-0.6,0.3-0.7,0c-0.2,1.8,1.2,4.1,0.4,5.5l-0.4-0.1c0.1,0.3,0.7,0.6,0.3,1c-0.2,0-0.4-0.1-0.4,0\n              c0.1,0.2,0.6,0.9,0.4,1.4l-0.4-0.4c-0.5,1.8,1,2.8,1.2,4.3c-1.1,0-1.5,1.4-1.7,2.1l-0.2,0c-0.2,2.1,0.8,3.6,1.2,5.5\n              c-1.2,1.2-0.6,4.1-1.3,6c0.2-0.3,0.9,0.1,1,0.4c-0.2,0.4,0.7,1.3-0.3,1.2l0-0.2c-1,0.3-0.7,1.3-0.8,2l1.3,0.1c-1.9,1.9,0.6,5-1.1,6\n              c0.3,0.5,0.3,1,0.6,1.5l-0.7,0c-0.2,1.3-0.3,1.9-0.2,3.5l-0.5-0.3c-0.1,0.9,2,1.2,2,2l-1.1,0.2c0,1.5-1,3.1-0.1,4.6\n              c-0.2,0-0.5-0.1-0.5-0.3c0,0.7-0.8,2.8,0.7,2.6c-0.5,0-0.5,0.4-0.4,0.9c0.1,0.3,0.2,0.5,0.3,0.8c0.1,0.4,0.2,0.8,0.3,1.1\n              c-0.2,0-0.5-0.1-0.7-0.1c0.4,0.5,0.7,0.7,0.9,1c0.2,0.3,0.3,0.6,0.4,1.2c-0.1,0.3-0.4,0.5-0.4,0.2c-0.1,0.5,0,0.6,0.2,0.7\n              c0.2,0.1,0.3,0.3,0.1,1.1c-0.1,0-0.3-0.1-0.4-0.1c-0.1,0.9,0,1,0.3,1c0.2-0.1,0.5-0.2,0.8-0.4c0.5-0.2,0.9-0.1,1.2,0\n              c0.3,0.1,0.6,0.4,0.5,0.7c0.4-0.4,1.2-0.9,1.8-0.8c0,0.2-0.3,0.4-0.4,0.5c0.9,0.2,1.8,0,2.3-0.2c0.4-0.2,0.8-0.4,1.3,0\n              c0,0.2,0.1,0.3,0.2,0.4c-0.5,0-1,0-1.3,0c-0.2,0-0.4,0-0.5,0c-0.1,0-0.2,0-0.2,0c0,0.3,0.4,0.5,0.5,0.7c0.1,0.1,0.1,0.2,0.1,0.4\n              c0,0.1,0,0.2-0.1,0.2c0,0.1-0.1,0.2-0.3,0.3c0.2-0.1,0.3-0.1,0.4-0.2c0.1-0.1,0.2-0.2,0.3-0.2c0.2-0.2,0.4-0.3,0.7-0.5\n              c0.5-0.3,1-0.5,1.4-0.4c0,0.1,0.1,0.1,0.1,0.2L541,104c0.2,0.1,0.4,0,0.5-0.1c0.3,0.1,0.4,0.2,0.7,0.2c0.1,0.5,1,0.2,1.4,0.3\n              c-0.1-0.7,0.5-0.6,0-1.2c0.1,0-0.1,0.1-0.4,0.1c-0.1,0-0.1,0-0.2,0.1c0,0,0,0,0-0.1c-0.3,0-0.6,0.1-1.1,0.1c0.3-0.4,0.7-0.7,1.2-0.3\n              c0.2-0.5,0.4-0.9,0.6-1.1c0.6,1,1.8,1.4,1.9,2c1.3-0.4,3.1-0.4,3.8-1.7c0.4-0.2,0.4,0.4,0.6,0.6c-0.2,0.1-0.5,0.1-0.6,0.3\n              c0.7-0.4,2.2-0.2,2.3-1.1c1,0.4-0.5,0.9,0.8,0.7l-0.2,0.3c2.6,1.4,4-2.5,6.9-0.7c-0.2-0.1-0.3-0.2-0.3-0.3c3.2,0.7,6.6-1,9.8,0.2\n              c0.9,0.2,1-0.4,1.5-0.8l0.2,0.7l0.9-0.9c1.3-0.7,2.4,1.8,4,0.5l-0.2-0.5c0.6,0.1,1.9-0.4,1.8,0.2c0.1-0.1,0.2-0.5,0.4-0.3l0,0.4\n              l1.5-0.7c1.2,0.2-0.1,1.6,1.6,1.6c0.6-0.2,1.7-1.2,1.5-1.5c-0.3,0-0.7-0.2-0.7-0.6l0.6-0.2c0-0.1,0-0.1,0-0.2l0.1,0l0-0.6\n              c0-0.4,0-0.8-0.1-1.2l-0.3-5l0.2,0.1l-0.2-0.2l0-0.9c0.1,0,0.3,0.1,0.3,0.2c0.5-0.7-0.4-0.5,0.1-1.1c0.1-0.1,0.1,0,0.2,0l-0.3-0.6\n              c0.2,0.1,0.7-0.1,0.6,0.2C583,89.7,581.6,89.1,582.3,88.7z M542.8,103.9C542.8,103.9,542.8,103.9,542.8,103.9c0,0,0,0.2-0.1,0.3\n              c-0.3,0.1-0.4,0-0.4-0.2C542.4,104,542.6,104,542.8,103.9z M581.5,91.1l0-0.3c0,0.1,0.1,0.2,0.1,0.3C581.6,91,581.5,91.1,581.5,91.1\n              z\"></path>\n          <path class=\"command-bar__logo__box\"\n                d=\"M645.9,39.9c-0.1-0.3-0.1-0.5-0.1-0.8l-0.6,0.3c0.1-0.3,0.1-0.5,0.4-0.5c-0.4-0.3-1,0.2-1.5-0.4\n              c0.3-0.4-0.3-1.2,0.5-1.3c-0.3,1.2,0.8-0.1,0.9,0.9c-0.1-0.2-0.1-0.4,0-0.7c0.1,0,0.2,0,0.2,0.1c0.2-0.3,0.2-0.5,0.2-0.6\n              c0-0.2-0.2-0.3-0.3-0.4c-0.3-0.2-0.7-0.5-0.7-1.4c0.4,0.2,0.9-0.7,0.8-1.1c0,0.9-0.7,0.6-1.1,0.1c0.4-1.4-0.2-1.2-0.2-1.8l-0.2,0.8\n              c-0.4,0-0.4-0.5-0.5-1.1c0.4-0.4,0.2,0.5,0.4,0.1c-0.2-0.1-0.2-0.2-0.2-0.5c-0.3-0.1-0.5-0.3-0.8-0.5c-0.3-0.2-0.7-0.3-1.3-0.1\n              c-0.4-0.3,0.4-0.8-0.1-1c-1,0.2-1.3,0.1-1.7-0.1c-0.2-0.1-0.4-0.2-0.7-0.2c-0.3,0-0.7-0.1-0.9,0c0.3,0.6,0.3,0.5-0.4,1l0.4-0.1\n              l0.2,0l0.3,0l1.4-0.1c-0.7,0.3-1.2,0.5-1.8,0.8c0.7,0,1.6-0.3,2-0.2c-0.9,0.4-0.7,0.3-0.6,0.8c-0.3-0.5-0.8-0.4-1.3-0.2\n              c-0.4,0.1-0.7,0.3-1.1,0l0.1-1.5c-1.1-0.7-2.1,0.6-3.6,0.2l0.5,0.3c-0.2,0.6-0.9,0-1.4,0.2c0.1-0.5-0.2-0.7-0.4-1\n              c0,0.4-1.1-0.1-1.3,0.9l-0.6-0.7c-1.2-0.3-1.5,1.3-2.7,1.1c0.4-0.5-0.1-0.9,0.7-1.2c-0.4,0-0.7-0.1-0.8,0.2\n              c-0.2-0.3-1.1,0.6-1.5,0.1c-0.2,0.2-0.3,0.7-0.7,0.7c0-0.1,0-0.2,0-0.2c0,0.2-0.7,0.3-0.4,0.9c-1.1-1.6-3.4-0.1-4.4-1.4\n              c-0.9,0.1-1.9,0.2-2.7,0.2c0.1-0.1,0-0.2,0.2-0.2c-1.1-0.5-0.7,1.4-1.9,0.8c-0.6-0.8,0.7-0.4,0.3-0.7c-0.1-1.6-1.5,0.1-2.3-0.8\n              l0.2-0.1c-0.6-0.6-1.6,0.5-2.6,0.4c0,0,0.1-0.1,0-0.2c-0.8,0.8-2.1,1-3.1,1.2c0.3-0.6,0.5-0.3,0.3-1c-0.4,0.1,0.1,1-0.8,1.2\n              c-0.3-0.4-0.9-0.7-0.9-1.4l0.8,0c-0.4-0.9-1.1,0-1.5-0.2l0.1-0.4c-1.3,0.1-1.4,1-2.7,0.9l0.1,0.3c-0.8,0.9-0.6-0.6-1.4,0.1l-0.2-1\n              c-0.3,0.1-0.7,0.3-1.2,0.5c-0.6,0.2-1.3,0.4-1.8,0.4c0.3-0.7,2.1-1,2.5-1.4c-0.1,0-0.3,0-0.5,0c-0.2,0-0.5,0-0.8,0\n              c-0.6,0-1.1,0.2-1.2,0.4c0.3-0.1,0.6-0.3,0.9-0.1c-0.8,0.7-1.6,1.1-2.8,1c-0.4-0.9-1.2-1.1-2.2-1.2c-0.2,1.1-0.2,2.6-1.1,2.9\n              c0.2,1.6,0,3.1-0.1,3.8c-0.2,0.9-0.5,1.8-0.5,2.8c1.5-0.6,0,0.6,1.5,0.6c0.1,0.5,0,0.6-0.1,0.7l0.1-0.1c0,0-0.1,0.1-0.1,0.1l0,0\n              l-1,1.4c-0.5,0.3-0.3-0.5-0.6-0.5c-0.2,1-0.7,0.4-1.1,1c0.6-0.1,0.7,0.4,0.5,1l0.8-0.8c0.7,1.5-1.4,1.2-0.7,2.7l-0.4-0.5\n              c0.2,0.8-0.2,2.7,1,3.6c-0.2,0.1-0.6,0.3-0.7-0.1c-0.3,1.8,1,4.1,0.2,5.4l-0.4-0.2c0.1,0.3,0.7,0.6,0.3,1c-0.2,0-0.4-0.2-0.4,0\n              c0.1,0.2,0.6,0.9,0.3,1.4l-0.4-0.4c-0.6,1.7,0.8,2.7,1,4.2c-1.1-0.1-1.5,1.3-1.8,1.9l-0.2,0c-0.3,2,0.7,3.5,0.9,5.4\n              c-1.3,1.1-0.8,3.9-1.6,5.7c0.2-0.3,0.8,0.1,0.9,0.4c-0.3,0.4,0.7,1.3-0.3,1.1l0-0.2c-1,0.3-0.8,1.2-0.9,1.9l1.3,0.2\n              c-2,1.7,0.3,4.9-1.4,5.8c0.3,0.5,0.3,1,0.5,1.5l-0.7-0.1c-0.3,1.2-0.4,1.8-0.3,3.4l-0.5-0.3c-0.1,0.9,1.9,1.2,1.9,2l-1.1,0.2\n              c-0.1,1.5-1.1,2.9-0.3,4.5c-0.2,0-0.5-0.1-0.5-0.3c-0.1,0.7-0.9,2.7,0.5,2.5c-0.2,0-0.4,0.1-0.4,0.2c-0.1,0.3-0.1,0.6,0,1.1\n              c0.1,0.8,0.3,1.7,0.5,2.2c-0.2,0-0.5,0.1-0.7,0.1c0.4,0.4,0.7,0.5,0.9,0.7c0.2,0.2,0.3,0.5,0.4,1c-0.1,0.3-0.5,0.6-0.4,0.4\n              c-0.1,0.2-0.1,0.4-0.1,0.5c0.6-0.1,1.2-0.5,1.9-0.2l-0.9,0.4c1.1,0.1,1.2-0.1,2.1-0.5c0.4-0.1,0.8,0,1.2,0.2\n              c0.4,0.2,0.7,0.5,0.7,0.8c0.1-0.2,0.4-0.4,0.6-0.5c0.3-0.1,0.4-0.2,0.7-0.1c0,0.2-0.1,0.4-0.2,0.4c1.3,0.6,2-0.6,2.8,0.1\n              c0,0.2,0,0.3,0.1,0.4c-1-0.1-1.9-0.2-1.8-0.3c-0.1,0.6,0.8,0.8,0,1.6c0.7-0.4,1.9-1.4,2.7-1c0,0.1,0.1,0.1,0.1,0.2l-0.1,0.1\n              c0.2,0.1,0.4,0,0.5-0.1c0.2,0.1,0.4,0.2,0.7,0.2c0.1,0.5,0.9,0.4,1.3,0.4c0-0.7,0.6-0.5,0.1-1.2c0.1,0.1-0.1,0.1-0.4,0.1\n              c-0.1,0-0.1,0-0.2,0.1c0,0,0,0,0-0.1c-0.3,0-0.6,0-1,0c0.4-0.3,0.7-0.6,1.1-0.2c0.3-0.4,0.5-0.9,0.7-1.1c0.5,1,1.5,1.6,1.6,2.2\n              c1.3-0.3,3,0,3.9-1.2c0.4-0.1,0.4,0.4,0.5,0.6c-0.2,0.1-0.5,0.1-0.6,0.3c0.7-0.3,2.1,0.1,2.4-0.8c0.9,0.5-0.6,0.8,0.7,0.8l-0.2,0.3\n              c2.3,1.7,4.2-2,6.7,0.1c-0.2-0.1-0.3-0.2-0.3-0.3c3,1.1,6.4-0.2,9.4,1.3c0.8,0.3,1-0.2,1.6-0.6l0.1,0.8l1-0.8c1.3-0.5,2.1,2,3.8,1\n              l-0.2-0.5c0.6,0.1,1.9-0.1,1.7,0.4c0.1-0.1,0.2-0.5,0.4-0.2l0,0.4l1.5-0.5c1.1,0.4-0.3,1.5,1.3,1.8c0.6-0.1,1.8-1,1.6-1.3\n              c-0.2,0-0.6-0.3-0.6-0.7l0.6-0.2c0-0.1,0-0.1,0-0.2l0.1,0l0-0.5c0.1-0.5,0.2-1.1,0-1.6l0.1-4.5l0.2,0.1l-0.2-0.2l0-0.8\n              c0.1,0,0.3,0.1,0.3,0.3c0.5-0.6-0.4-0.6,0.2-1.1c0.1-0.1,0.1,0,0.2,0l-0.3-0.6c0.2,0.1,0.7-0.1,0.6,0.3c0.6-1.1-0.7-1.8,0.1-2.1\n              l0.1,0.1c-0.2-1.1-0.5-0.7-1-0.8l0-0.7l1.3-0.5l-0.8-0.2c0-0.2,0.1-0.5,0.4-0.4c-0.5-0.6-0.3,0.1-0.8-0.3c0.7-0.7-0.3-2.1,0.9-2.2\n              c-0.6-0.3-0.5,0.7-0.8-0.1c1.3-0.2-0.2-0.9,0.4-1.6c0.7,0.2,0.3,0.9,0.9,0.2c-1.1-0.4,0.3-1.3,0.1-1.9c-0.9,0-0.3-1.3-0.6-2.2\n              c0.3,0.1,0.5-1.8,0.9-2.7l0.2,0c-0.9-0.2-0.7-1.1-0.9-1.8c0.5,0,0.8,0.2,1,0.5l0-0.6c-0.3-0.2-0.4-0.4-0.5-0.8l0.7,0.2\n              c0-0.8-0.8-1.7-0.3-2.8c-0.4-0.1-1.1,0.3-1.6,0.2l0-0.6c0.2,0,0.4,0.1,0.5,0.2c0-0.1-0.3-0.4-0.1-0.5l0.6,0.5\n              c0.3-0.5-0.5-0.3-0.4-0.7c0.5,0,0.4-0.3,0.7-0.2c0,0.1-0.2,0.3,0,0.5c0.1-0.4,0.4-1,0.8-1.1c-0.3,0-0.7,0-0.7-0.3\n              c0-0.8,0.6-0.3,0.9-0.8c-0.3-0.2,0-0.7-0.7-0.8c0-0.2,0.4-0.3,0.6-0.4c-0.3-0.4,0-2-0.8-1.6l-0.1,0.1c0,0,0,0,0-0.1\n              c-0.7-1.4,0.5-2.9,0.2-4.4l0.2,0c-0.3-0.8-0.1-1.9-0.6-2.5c0-0.2,0.5-0.6,0.8-0.5c-0.2-0.6-0.4-0.7,0-1.6c0.4,0.7,0.4-0.2,0.3,0.7\n              c0.2-0.8,0.6-1,0-1.9c0.6,0.3,0.8-0.6,0.6-1l-0.5,0.6c-0.1-0.4,0-0.8,0-1.2l0.8,0.2c1-1.7-1.6-3.1,0.1-4.4c-0.6-0.5-0.4,0.9-1.1,0.5\n              c0.1-0.8,1.3-1.6,1.3-1.8c0.5-1.1-1.2-1-0.8-2.2c-0.3,0.1-1.1,0.3-1.3-0.6c0.1-0.8,1.3-0.9,1.5-0.4c0,0.2-0.1,0.3-0.2,0.3\n              c0.1,0.3,0.2,0.6,0.5,0.3l-0.3-0.1c0.3-0.4,0.3-1.5,0.8-1.2c-0.3-0.6-0.5-0.3-0.6,0.2c0.5-1,0.1-1.6,0.4-2.6\n              c-0.2,0.2-0.5,0.3-0.5,0.1c0.4-1.5-0.5-0.7-0.6-1.7c0.5-0.6,1.1,0.2,0.7-0.8c0.3-0.6,0.8-0.1,1,0.1c-0.5-1,0.4-1.8,0.5-2.3\n              L645.9,39.9z M602.5,101.8C602.5,101.8,602.6,101.7,602.5,101.8c0,0,0,0.2-0.1,0.3c-0.3,0.1-0.4-0.1-0.4-0.2\n              C602.2,101.8,602.3,101.8,602.5,101.8z M641,93.8c0,0-0.1,0.1-0.1,0.1l0-0.3C641,93.6,641,93.7,641,93.8z\"></path>\n          <path class=\"command-bar__logo__letter\"\n                d=\"M20.3,58.8c-0.1-0.6-0.2-1.2-0.2-1.7s-0.2-1.1-0.2-1.7c-0.1-0.7-0.1-1.4,0-2.1c0.1-0.7,0.2-1.4,0.4-2.1\n              c0.2-0.7,0.6-1.3,1-1.8c0.4-0.5,1-0.9,1.8-1.1c1.2-0.3,2.4-0.3,3.6,0.1c1.2,0.4,2.3,0.9,3.4,1.4c1,0.5,2,1.1,3,1.7\n              c1,0.6,2,1.1,3.1,1.7c0.5,0.3,1.1,0.5,1.6,0.8c0.5,0.3,1,0.6,1.4,1s0.8,0.8,1,1.4c0.3,0.5,0.4,1.2,0.5,1.9c0,0.7-0.1,1.5-0.4,2.2\n              c-0.3,0.7-0.6,1.4-0.8,2c-1.6,3.7-3.7,7.1-6.3,10.4c-0.9,1.2-2,2.3-3.2,3.3c-1.2,1-2.5,1.7-4,2c0.3,2.1,0.7,4.2,1.1,6.2\n              c0.4,2,0.9,4.1,1.4,6.1c0.2,0.7,0.1,1.3-0.2,1.7c-0.3,0.5-0.8,0.8-1.3,0.9c-0.5,0.1-1,0.1-1.6-0.1s-0.9-0.7-1.1-1.4\n              c-1.2-4.5-2.1-9-2.7-13.5s-1-9.1-1.2-13.8c0-0.6-0.1-1.2-0.1-1.8c0-0.6,0-1.2-0.1-1.8c0-0.3,0-0.5,0-0.7s0-0.5-0.1-0.7V58.8z\n              M24.2,53.4c0,0.8,0,1.7,0.1,2.6c0,0.4,0.1,0.7,0.1,0.8c0,0.1,0,0.2,0,0.3l0.2,1c0.1,1.1,0.2,2.2,0.2,3.4c0.1,1.6,0.1,3.2,0.2,4.8\n              s0.2,3.2,0.3,4.8c0,0.4,0.1,0.9,0.1,1.3c0,0.4,0.1,0.9,0.1,1.3c0.4-0.1,0.6-0.2,0.8-0.3c0.1-0.1,0.3-0.2,0.4-0.2\n              c0.2-0.1,0.4-0.3,0.5-0.4c0.9-0.7,1.7-1.5,2.3-2.2c0.6-0.7,1.2-1.5,1.7-2.3c1.2-1.7,2.3-3.5,3.2-5.3c0.2-0.4,0.4-0.8,0.5-1.1\n              c0.2-0.3,0.4-0.7,0.5-1.1c0.1-0.2,0.1-0.4,0.2-0.5s0.1-0.3,0.2-0.5c0.2-0.4,0.3-0.8,0.4-1.1c0-0.3-0.1-0.4-0.2-0.6\n              c-0.5-0.3-0.9-0.5-1.1-0.6c-1.1-0.5-2.1-1-3.1-1.6c-1-0.5-2-1.1-3.1-1.7c-0.5-0.3-1-0.6-1.6-0.8c-0.5-0.3-1.1-0.5-1.7-0.7\n              c-0.1,0-0.2-0.1-0.4-0.1c-0.1,0-0.2-0.1-0.4-0.1c-0.3,0-0.4,0-0.4,0c0,0-0.1,0-0.1,0c-0.1,0.1-0.1,0.2-0.1,0.3c0,0.1,0,0.2-0.1,0.3\n              V53.4z\"></path>\n          <path class=\"command-bar__logo__letter\"\n                d=\"M78.4,88.5c-0.6,0-1-0.2-1.4-0.5c-0.3-0.3-0.6-0.7-0.7-1.1c-0.1-0.4-0.1-0.9,0-1.3c0.1-0.4,0.4-0.8,0.7-1.1\n              c0.3-1.3,0.4-2.5,0.2-3.8c-0.1-1.3-0.3-2.5-0.4-3.8c-0.2-1.5-0.2-3.1-0.2-4.6c0.1-1.5,0.2-3,0.4-4.6c0.2-1.5,0.5-3.1,0.7-4.6\n              c0.3-1.5,0.5-3.1,0.7-4.6c0.2-1.7,0.3-3.3,0.5-4.9c0.1-1.6,0.2-3.3,0.3-4.9c0-0.7,0.2-1.2,0.7-1.6c0.5-0.4,1-0.6,1.5-0.6\n              s1,0.2,1.5,0.6c0.4,0.4,0.6,0.9,0.6,1.6c-0.1,1.7-0.2,3.5-0.3,5.1c-0.1,1.7-0.3,3.4-0.5,5.2c-0.2,1.6-0.4,3.2-0.7,4.8\n              c-0.3,1.6-0.5,3.1-0.7,4.8c-0.2,1.5-0.3,3.1-0.3,4.6c0,1.5,0.1,3,0.3,4.6c0.1,1.1,0.2,2.1,0.3,3.2c0.1,1.1,0.1,2.1-0.1,3.2\n              c2.7,0.1,5.5,0.2,8.2,0.4c2.7,0.2,5.5,0.4,8.2,0.5c0.7,0,1.2,0.3,1.6,0.8c0.3,0.5,0.5,1,0.5,1.6s-0.2,1.1-0.5,1.5\n              c-0.3,0.4-0.9,0.6-1.6,0.6c-3.3-0.1-6.6-0.3-9.8-0.6C85,88.6,81.7,88.5,78.4,88.5z\"></path>\n          <path class=\"command-bar__logo__letter\"\n                d=\"M253.7,49.2c-0.6-0.3-1-0.8-1-1.4c-0.1-0.6,0-1.1,0.3-1.6s0.7-0.8,1.2-1.1c0.5-0.2,1.1-0.2,1.7,0.2\n              c1.9,1,3.5,2.3,4.8,3.9c1.3,1.6,2.3,3.3,3.2,5.3c0.5,1.3,1.1,2.6,1.6,3.8s1.2,2.4,2,3.5c1.3-1.2,2.6-2.5,3.7-3.9\n              c1.1-1.4,2.1-2.9,3.1-4.5c0.9-1.6,1.8-3.2,2.7-4.8c0.9-1.6,1.7-3.2,2.6-4.7c0.4-0.6,0.8-0.9,1.4-1s1.1,0,1.6,0.3\n              c0.5,0.3,0.8,0.7,1,1.2s0.1,1.1-0.2,1.7c-0.9,1.6-1.9,3.3-2.8,5c-0.9,1.7-1.9,3.3-2.9,5c-1,1.6-2.1,3.2-3.2,4.7\n              c-1.1,1.5-2.4,2.9-3.8,4.2c-0.1,3.4-0.2,6.7-0.3,10c-0.1,3.3,0,6.6,0.3,10c0,0.6-0.2,1.1-0.7,1.5s-1,0.6-1.5,0.6\n              c-0.6,0-1.1-0.2-1.5-0.6s-0.6-0.9-0.7-1.5c-0.2-3-0.3-6-0.3-9c0-3,0.1-6,0.2-9c-0.3-0.2-0.6-0.5-0.8-0.8c-1.2-1.4-2.2-2.8-3-4.3\n              c-0.7-1.5-1.4-3.1-2.1-4.8c-0.7-1.7-1.5-3.2-2.5-4.5S255.4,50,253.7,49.2z\"></path>\n          <path class=\"command-bar__logo__letter\"\n                d=\"M486.1,70.3c-0.4-3.7-0.7-7.4-0.8-11s-0.4-7.3-0.8-10.9c-0.1-0.6,0.1-1.2,0.6-1.6c0.5-0.4,1-0.6,1.6-0.6\n              c0.7,0,1.2,0.2,1.5,0.6c0.4,0.4,0.6,1,0.7,1.6c0.3,2.2,0.4,4.4,0.6,6.6c0.1,2.2,0.2,4.3,0.4,6.6c0.1,3.2,0.4,6.5,0.8,9.7\n              c0,0.4,0.1,0.9,0.1,1.4c0.1,0.5,0.1,0.9,0.1,1.2c0.1,0.5,0.2,1,0.2,1.5c0.1,0.5,0.2,1,0.3,1.4c0.2,0.9,0.5,1.8,0.7,2.5\n              c0.3,0.8,0.7,1.4,1.2,1.9l0.5,0.3c0.3,0.1,0.5,0.1,0.8,0.1c0.3,0,0.6-0.1,1.1-0.3c0.5-0.2,0.9-0.4,1.3-0.6c1-0.7,1.8-1.5,2.5-2.5\n              c0.7-1,1.3-2,1.8-3.1c0.6-1.2,1.1-2.4,1.5-3.7s0.8-2.5,1.1-3.8c0.7-3.2,1-6.5,1-9.8c-0.1-3.3-0.3-6.6-0.7-9.8\n              c-0.1-0.6,0.1-1.1,0.6-1.5c0.5-0.4,1-0.7,1.5-0.7c0.7,0,1.2,0.2,1.5,0.7c0.4,0.4,0.6,1,0.7,1.5c0.5,3.5,0.7,7.1,0.8,10.8\n              c0,3.6-0.4,7.2-1.3,10.7c-0.3,1.3-0.7,2.5-1.1,3.9c-0.4,1.3-1,2.6-1.6,3.8c-0.6,1.2-1.3,2.4-2.1,3.5c-0.8,1.1-1.7,2.1-2.7,2.9\n              c-1,0.8-2.2,1.5-3.4,1.9c-1.2,0.5-2.5,0.6-3.8,0.3c-1-0.2-1.9-0.6-2.7-1.3c-0.8-0.7-1.4-1.4-1.9-2.3c-0.9-1.8-1.6-3.8-1.9-5.8\n              C486.6,74.3,486.4,72.3,486.1,70.3z\"></path>\n          <path class=\"command-bar__logo__letter\"\n                d=\"M319.5,89c-1.1-0.7-2.1-1.6-3-2.6c-0.9-1-1.6-2.2-2.3-3.4c-0.7-1.2-1.2-2.5-1.6-3.8c-0.4-1.3-0.8-2.6-1-3.9\n              c-0.2-1.2-0.3-2.5-0.4-3.7c0-1.3,0.1-2.5,0.5-3.7c0.3-1,0.7-1.9,1.2-2.8c0.5-0.9,1-1.7,1.6-2.5c1.4-2,3-3.9,4.8-5.6s3.7-3.3,5.7-4.7\n              c0.9-0.6,1.9-1.2,3-1.7c1-0.5,2.1-0.8,3.2-1.1c1.1-0.2,2.2-0.3,3.3-0.2c1.1,0.1,2.2,0.4,3.3,0.9c0.5,0.3,0.9,0.7,1,1.3\n              c0.1,0.6,0.1,1.2-0.2,1.7c-0.3,0.6-0.8,0.9-1.3,1c-0.6,0.1-1.1,0-1.7-0.2c-0.6-0.3-1.4-0.4-2.3-0.3c-0.4,0-0.8,0.1-1.3,0.2\n              c-0.5,0.1-1,0.3-1.6,0.5c-1.1,0.4-2.1,1-3.1,1.7c-1,0.7-2,1.5-2.9,2.2c-1.6,1.4-3.1,2.9-4.5,4.6c-1.4,1.6-2.6,3.4-3.5,5.3\n              c-0.1,0.3-0.2,0.6-0.3,1.2c-0.1,0.5-0.2,1-0.2,1.3c-0.1,1.1,0,2.1,0.2,3.2c0.2,1.2,0.5,2.4,0.9,3.6c0.4,1.2,0.9,2.3,1.5,3.4\n              c0.6,1,1.3,2,2,3c0.8,1,1.8,1.6,2.9,1.9c1.2,0.3,2.5,0.3,3.8-0.2c0.3-0.1,0.6-0.2,0.8-0.4c0.3-0.1,0.5-0.3,0.8-0.4\n              c0.1-0.1,0.2-0.1,0.3-0.2c0.1-0.1,0.2-0.1,0.3-0.2c0.8-0.7,1.6-1.5,2.2-2.3c0.7-0.8,1.3-1.7,1.9-2.5c0.3-0.5,0.6-1,0.9-1.6\n              s0.6-1.1,0.7-1.6c0.2-0.5,0.3-0.9,0.4-1.3s0.2-0.8,0.2-1.2c-0.4-0.1-0.7-0.2-1.1-0.2s-0.7,0-1.1,0c-1,0-2,0.1-3,0.2\n              c-1,0.1-2,0.2-3.1,0.2c-0.7,0-1.2-0.2-1.6-0.7c-0.3-0.5-0.5-1-0.5-1.5s0.2-1.1,0.5-1.5c0.3-0.4,0.9-0.7,1.6-0.7c1,0,2-0.1,3-0.2\n              c1-0.1,2-0.2,3-0.2c1,0,1.9,0,2.9,0.1c0.9,0.1,1.9,0.4,2.8,0.8c0.9,0.4,1.7,1,2.4,2c0.4,0.6,0.6,1.3,0.5,2.1\n              c-0.1,0.8-0.1,1.5-0.1,2.3c-0.1,1.6-0.1,3.2-0.1,4.7c0,1.6,0,3.1-0.1,4.7c0,0.7-0.2,1.3-0.7,1.6c-0.5,0.4-1,0.5-1.5,0.5\n              s-1-0.2-1.5-0.5c-0.4-0.4-0.7-0.9-0.7-1.6c0-0.7,0.1-1.5,0.1-2.2s0-1.5,0.1-2.2c-1.1,1.7-2.2,3.3-3.6,4.7c-0.9,1-1.9,1.8-3.1,2.5\n              s-2.4,1.2-3.6,1.5s-2.5,0.4-3.9,0.2S320.7,89.8,319.5,89z\"></path>\n          <path class=\"command-bar__logo__letter\"\n                d=\"M631.4,73.7c-0.2,2.3-0.8,4.3-2,6c-1.1,1.8-2.6,3.2-4.5,4.5c-1.2,0.8-2.6,1.5-4,2.1c-1.4,0.6-2.8,1.1-4.3,1.4\n              c-1.3,0.3-2.5,0.4-3.7,0.4c-1.2,0-2.5-0.1-3.7-0.3c-0.3,0.3-0.6,0.6-1.1,0.7c-0.4,0.1-0.8,0.1-1.2,0c-0.8-0.2-1.3-0.8-1.5-1.6\n              c-0.4-0.4-0.7-1-0.7-1.6c0-0.8,0.4-1.5,1.1-1.9c0.2-1.5,0.4-3,0.5-4.4c0.1-1.5,0.2-3,0.3-4.5c0.1-1.8,0.2-3.5,0.3-5.3\n              c0.1-1.7,0.1-3.5,0.1-5.3c0-1.1,0-2.3-0.1-3.4c-0.1-1.2-0.1-2.4-0.1-3.5c0-1.2,0.1-2.3,0.2-3.5c0.1-1.1,0.4-2.3,0.8-3.3\n              c-0.2-0.6-0.1-1.2,0.1-1.8c0.3-0.6,0.7-1,1.5-1.2c0.4-0.5,1-0.7,1.6-0.7c0.6,0,1.1,0.2,1.5,0.6c1.8,0.1,3.5,0.5,5.2,1\n              c1.7,0.5,3.3,1.4,4.7,2.6c1.7,1.4,3.2,3.1,4.2,5.1c1.1,2,1.9,4,2.6,6.1c0.3,0.9,0.6,1.9,0.9,2.8c0.3,1,0.6,1.9,0.8,2.9\n              c0.2,1,0.4,2,0.5,3C631.5,71.7,631.5,72.7,631.4,73.7z M622.3,80.7c0.5-0.3,0.9-0.6,1.3-0.9c0.4-0.3,0.8-0.6,1.2-1.1\n              c0.3-0.4,0.6-0.7,0.8-0.9c0.2-0.2,0.4-0.6,0.7-1c0.6-1.3,0.9-2.7,0.9-4.4c0-0.9-0.1-1.8-0.2-2.6c-0.1-0.8-0.4-1.7-0.7-2.7\n              c-0.3-1-0.6-2.1-0.9-3.1c-0.3-1-0.7-2-1.1-3c-0.3-0.8-0.7-1.7-1.2-2.6s-1-1.7-1.5-2.4c-1.3-1.5-2.7-2.7-4.3-3.3\n              c-1.6-0.7-3.4-1.1-5.2-1.2c-0.4,1-0.6,2.1-0.7,3.2c-0.1,1.1-0.1,2.3-0.1,3.4c0,1.1,0.1,2.3,0.2,3.4c0.1,1.1,0.1,2.3,0.1,3.3\n              c-0.1,3.1-0.3,6.2-0.4,9.3c-0.1,3.1-0.4,6.1-0.8,9.2c0.7,0.1,1.3,0.1,2,0.2c0.7,0,1.3,0,2-0.1c1.4-0.1,2.8-0.5,4.2-1.1\n              C619.8,82,621.1,81.4,622.3,80.7z\"></path>\n          <path class=\"command-bar__logo__letter\"\n                d=\"M394.6,96.8c-0.3,0.5-0.7,0.9-1.3,1.1c-0.6,0.2-1.2,0.1-1.7-0.3c-1-0.7-1.9-1.5-2.8-2.3c-0.9-0.8-1.9-1.5-3-2.1\n              c-0.9-0.6-1.9-1.1-2.8-1.6s-1.9-1.1-2.7-1.7c-1.1-0.8-2-1.7-3-2.7c0,1.8,0,3.7,0.1,5.5c0.1,1.8,0.2,3.7,0.3,5.5\n              c0.1,0.6-0.1,1.2-0.6,1.6c-0.5,0.4-1,0.6-1.5,0.6c-0.7,0-1.2-0.2-1.5-0.6c-0.4-0.4-0.6-1-0.7-1.6c-0.2-2.7-0.3-5.3-0.4-7.8\n              c0-2.6,0-5.2,0.2-7.8c-0.3-0.4-0.6-0.8-0.9-1.2c-0.3-0.4-0.6-0.9-0.8-1.3c-0.2-0.4-0.3-0.9-0.2-1.3c0.1-0.4,0.2-0.8,0.4-1.1\n              c0.2-0.3,0.5-0.5,0.9-0.7c0.3-0.2,0.7-0.3,1.1-0.2c0.1-0.8,0.2-1.7,0.3-2.6s0.2-1.7,0.3-2.6c0.1-0.6,0.2-1.2,0.3-1.7\n              c0.1-0.6,0.2-1.2,0.2-1.7c0.1-0.3,0.1-0.6,0.1-0.8c0-0.3,0.1-0.5,0.1-0.8c0-0.1,0-0.3,0.1-0.6c0-0.5,0.1-1,0.1-1.4\n              c0-0.5,0-0.9,0.1-1.4c0-0.6,0.1-1.2,0.2-1.9c0.1-0.7,0.2-1.3,0.4-2c0.2-0.6,0.5-1.2,0.8-1.8s0.9-1,1.5-1.3c0.6-0.3,1.2-0.4,1.8-0.4\n              c0.6,0,1.2,0,1.8,0.1c0.6,0.1,1.2,0.3,1.8,0.5c0.6,0.2,1.1,0.4,1.7,0.6c1.3,0.5,2.5,1,3.8,1.6c1.3,0.6,2.4,1.4,3.4,2.4\n              c0.9,1,1.8,2.1,2.5,3.4c0.7,1.3,1,2.7,0.9,4.1c-0.1,0.8-0.2,1.6-0.5,2.2c-0.3,0.6-0.6,1.3-1.1,2c-1.2,1.7-2.4,3.2-3.8,4.8\n              c-1.3,1.5-2.8,2.9-4.3,4.2c-0.7,0.6-1.5,1.2-2.4,1.8c-0.9,0.6-1.8,1.1-2.7,1.5c0.3,0.3,0.7,0.6,1,0.9c0.3,0.3,0.7,0.6,1,0.8\n              c0.9,0.6,1.9,1.2,2.9,1.7s1.9,1.1,2.8,1.7c0.9,0.6,1.7,1.3,2.6,2c0.8,0.7,1.7,1.4,2.5,2c0.5,0.3,0.8,0.8,1,1.3\n              S394.9,96.2,394.6,96.8z M379.9,80.3c0.9-0.5,1.7-1,2.4-1.5c0.7-0.5,1.4-1.1,2.1-1.7c0.7-0.7,1.4-1.4,2.1-2.1c0.7-0.7,1.4-1.5,2-2.2\n              c0.3-0.3,0.5-0.6,0.7-0.9c0.2-0.3,0.4-0.5,0.6-0.7c0.2-0.2,0.4-0.5,0.6-0.8c0.2-0.3,0.4-0.6,0.6-1c0.1-0.1,0.1-0.2,0.2-0.3\n              c0.1-0.1,0.1-0.2,0.2-0.3c0-1-0.2-2-0.7-2.9c-0.6-1.1-1.4-1.9-2.3-2.6c-0.9-0.7-2-1.2-3.1-1.7c-0.7-0.3-1.3-0.6-2-0.8\n              s-1.3-0.4-2.1-0.6c-0.2-0.1-0.5-0.1-0.8-0.2c-0.2,0.1-0.4,0.2-0.4,0.3c-0.1,0.5-0.2,0.8-0.2,1c0,0.1-0.1,0.3-0.2,0.6\n              c0,0.4-0.1,0.7-0.1,0.9c0,0.2,0,0.4,0,0.6c-0.1,2.5-0.4,5.1-0.8,7.5c-0.4,2.5-0.8,5-1,7.5c-0.1,0.4-0.1,0.8-0.1,1.2\n              c0,0.4,0,0.8-0.1,1.2c0.3,0,0.5,0,0.5,0c0.4-0.1,0.7-0.2,1-0.3C379.3,80.6,379.6,80.5,379.9,80.3z\"></path>\n          <path class=\"command-bar__logo__letter\"\n                d=\"M544.8,53.3c-0.1-0.1-0.1-0.1-0.2-0.2c-0.1-0.1-0.1-0.1-0.2-0.2c-0.4-0.4-0.6-0.9-0.6-1.4\n              c0-0.5,0.2-0.9,0.4-1.3s0.6-0.7,1.1-0.9c0.5-0.2,0.9-0.2,1.4,0c0.6-0.1,1.1,0.1,1.7,0.4s0.8,0.9,0.8,1.7c1.9,2.3,3.6,4.5,5.2,6.6\n              s3.1,4.5,4.5,6.9c0.6,1.1,1.2,2.2,1.7,3.4c0.5,1.2,1,2.3,1.5,3.5c0-1.5,0.1-3,0.2-4.5c0.1-1.5,0.3-3,0.5-4.5\n              c0.4-3.2,0.8-6.3,1.2-9.5c0.4-3.2,0.6-6.4,0.6-9.6c0-0.7,0.2-1.2,0.7-1.6c0.5-0.4,1-0.5,1.5-0.5c0.5,0,1,0.2,1.5,0.5\n              c0.5,0.4,0.7,0.9,0.7,1.6c0,3.3-0.3,6.6-0.7,9.8c-0.4,3.2-0.8,6.5-1.2,9.7c-0.2,1.7-0.3,3.3-0.4,4.9s-0.1,3.2-0.1,4.9\n              c0,1.6,0.1,3.2,0.1,4.7c0.1,1.6,0,3.1,0,4.7c0,0.6-0.2,1-0.5,1.4c-0.3,0.4-0.7,0.6-1.1,0.7c-0.4,0.1-0.9,0.1-1.3-0.1\n              c-0.4-0.1-0.8-0.4-1.1-0.9c-0.8-1.3-1.5-2.6-2.1-4.1c-0.6-1.4-1.1-2.8-1.6-4.2c-0.5-1.4-1-2.7-1.6-4c-0.5-1.3-1.2-2.5-1.9-3.8\n              c-0.9-1.7-1.9-3.2-2.9-4.8c-1-1.5-2.1-3-3.3-4.4c0,1.5,0,3,0,4.5s0,2.9,0.1,4.5c0.1,2.7,0.3,5.3,0.5,7.9c0.3,2.6,0.6,5.2,0.9,7.8\n              c0.1,0.6-0.1,1.2-0.6,1.6c-0.5,0.4-1,0.6-1.5,0.6c-0.7,0-1.2-0.2-1.5-0.6c-0.4-0.4-0.6-0.9-0.7-1.6c-0.4-2.6-0.7-5.2-1-7.8\n              c-0.3-2.6-0.5-5.2-0.5-7.9c-0.1-2.3-0.1-4.6-0.1-6.9C544.8,57.9,544.8,55.6,544.8,53.3z\"></path>\n          <path class=\"command-bar__logo__letter\"\n                d=\"M432.3,92.7c-0.8-0.6-1.6-1.3-2.3-2.1c-0.7-0.8-1.3-1.6-1.8-2.5c-0.5-0.8-1-1.6-1.5-2.5s-0.9-1.8-1.3-2.7\n              c-0.4-0.9-0.6-1.9-0.8-2.8c-0.2-1-0.3-1.9-0.2-2.9c0.1-1.9,0.3-3.7,0.6-5.6c0.3-1.8,0.6-3.6,1.1-5.4c0.1-0.7,0.3-1.4,0.5-2.2\n              c0.2-0.8,0.5-1.6,0.8-2.4c0.3-0.8,0.8-1.4,1.2-2.1c0.5-0.6,1.1-1.1,1.8-1.4c1.1-0.5,2.2-0.7,3.3-0.5c1.1,0.1,2.2,0.4,3.2,0.9\n              c1,0.5,2,1.1,3,1.7c0.9,0.7,1.8,1.4,2.7,2c0.8,0.6,1.5,1.3,2.3,2c0.7,0.7,1.5,1.3,2.3,2c0.9,0.8,1.9,1.7,2.9,2.5\n              c1,0.8,1.8,1.8,2.4,2.9c0.7,1.3,1.1,2.5,1.3,3.8s0,2.5-0.3,3.8c-0.3,1.2-0.7,2.4-1.4,3.6c-0.6,1.2-1.3,2.3-2,3.4c-0.7,1-1.4,2-2.3,3\n              c-0.9,1.1-1.8,2-2.8,2.9c-1,0.9-2.2,1.6-3.3,2.1c-1.2,0.5-2.4,0.7-3.7,0.6c-0.8-0.1-1.3-0.4-1.7-0.9c-0.7-0.1-1.4-0.3-2-0.4\n              C433.5,93.4,432.9,93.1,432.3,92.7z M428.8,76.8c-0.1,0.9,0,1.8,0.1,2.5c0.2,0.8,0.4,1.6,0.8,2.6c0.5,1.1,1.1,2.2,1.7,3.2\n              c0.6,1.1,1.3,2.1,2.2,3c0.7,0.8,1.6,1.3,2.5,1.5c0.6,0.1,1.2,0.2,1.8,0.3c0.6,0.1,1.2,0.2,1.7,0.4c0.2-0.1,0.4-0.2,0.7-0.2\n              c0.2-0.1,0.4-0.2,0.7-0.4c0.6-0.4,1.2-0.8,1.7-1.3c0.5-0.5,1-1,1.5-1.5c0.9-1,1.8-2.2,2.7-3.4c0.9-1.2,1.5-2.5,2-3.8\n              c0.5-1.3,0.7-2.5,0.6-3.7c-0.1-1.1-0.6-2.3-1.6-3.3c-0.9-1-1.8-1.9-2.9-2.7c-1-0.8-2-1.7-3-2.5c-0.8-0.7-1.7-1.4-2.5-2.1\n              c-0.8-0.7-1.7-1.3-2.6-1.9c-0.9-0.6-1.7-1-2.6-1.3c-0.1,0-0.2-0.1-0.3-0.1c-0.1,0-0.2,0-0.3-0.1l-0.8-0.1c-0.3,0.1-0.5,0.1-0.5,0.2\n              c-0.3,0.3-0.4,0.4-0.4,0.4c-0.3,0.4-0.4,0.8-0.5,0.9c-0.1,0.2-0.2,0.5-0.3,0.9c-0.4,1.1-0.7,2.2-0.9,3.2s-0.5,2.2-0.7,3.2\n              c-0.2,1.1-0.3,2-0.5,2.9C428.9,74.7,428.8,75.7,428.8,76.8z\"></path>\n          <path class=\"command-bar__logo__box\"\n                d=\"M238.3,22.8c-0.6-0.1-1.2-0.3-1.9-0.4l0.7-0.1c-0.5-0.4-1.4-0.7-2.4-1.1c-0.4-0.1-0.7-0.3-1.1-0.4\n              c-0.3-0.1-0.6-0.3-0.9-0.4c-0.4,0.2-1-0.1-0.8,0.5c-2.7-3-6.7-1.3-9.1-3.8c0.4,0.1,0.9,0,0.7-0.1l-2.6-0.6c0,0.6,0.2,0.4,0.6,1.1\n              c-0.4,0.2-1.2,0.2-1.8,0c1.3,0,0.3-0.9-0.1-1.3c0,0.2-0.2,0.3-0.2,0.5c-1-1.4-0.6,1-2.2-0.3l0.2-0.2c-0.7-0.1-1.5,0.1-2.2-0.3\n              c-0.1-0.7,0.8,0,1-0.5c-1-0.3-2.4,0.5-3.1-0.5c0.5,0,1.1,0.2,1.6,0.2c-0.7-1.1-0.9-0.9-1.6-1.3l0.1,0l-1.6-0.9l0.4,0.4\n              c-1.8,1.2-2.7-0.6-4.8-0.2c2-0.4,0.5-0.5,0.9-1.1c-0.4-0.4-0.9-0.8-0.7-0.9c-1,0.3-1.2-0.9-2.6-0.5c0-0.2-0.9-0.3-0.5-0.8\n              c-1.1-0.2,0.1,0.5-0.1,0.7c-1.9,0.2-3.5-2.3-6.2-2.1c-2-0.3-2.7,0.3-4.4,0.1l-0.5-0.2c-0.2-0.4,0.9-0.5,0.9-0.5\n              c-0.9-0.7-1.7-0.9-2.7-1c0.3-0.1,0.1-0.4-0.1-0.7c-0.8-0.2-0.8,0.6-1.7,0c0.3-0.2,0.6-0.5,1.1-0.6c-1-0.1-2,0-3-0.1\n              c-0.1-0.5,0.8-0.4,1.2-0.5c-1.1-0.2-2.9-0.6-3.4-0.1l0.4,0.4c-0.1-0.2,0.2-0.5,0.5-0.5c1.1-0.2,1.2,0.7,1.2,0.9l-0.7-0.1\n              c0.2,1.6,2.8,0.7,4.1,1.3L190.2,7L180,3.4l-0.1,0c-0.2-1.1-0.1-2.3-2.3-2.3l-1.2,1.2L175.6,2l1.2-0.5c-0.9-0.2-1.2,0.4-1.6,0.8\n              c-0.2-0.2-0.5-0.4-0.8-0.4l-1.1,1.6c-0.3-0.4,0.4-1.1-0.7-1c-0.5,0.7,1,0.7,0,1.4c-0.9-0.2-0.8-0.7-1.5,0c-0.2-0.2-0.1-0.5,0.2-0.7\n              c-0.6,0.6-1.9,0.8-2.1,1.6c-1-0.3-2.4,1-2.7,0c-0.1,0.3-1,1.3-1.7,1.6c0.8-1.7-2.6,0.8-2.7-0.6c-0.5,1.4-3.1,1.7-4.6,2.9\n              c0.1-0.1,0.2-0.4,0.3-0.4c-0.9,0.2-1.3,0-2,0c0.2,1-1.1,1.7-1.7,2.5c-1.4-0.5,0.3-1.3-0.4-2c0.3,0.7-1.7,1.4-1.1,1.9\n              c-1.1-0.2-0.8,0-1.5-0.6c0.3,1.2-0.7,0.5-1.2,1.4c-1.4-0.1-0.7-1-1.9-0.1c-0.8-0.1,0.3-1,0.3-1c-1.4,0.5-0.4,1.1-0.5,1.7\n              c-1.6,0.8-1-0.4-1.7-0.6c-1,0.6-1.5,1.7-2.3,1.9c-0.1-0.1-0.3-0.1-0.2-0.4c-0.9,1.1-3.2,1.2-4.2,2.5c-0.1,0-0.2-0.1-0.2-0.2\n              c0.4,0.4,0.5,0.8,0.6,1.1c-0.6,0.2-1,1-1.4,0.5c0.4-0.1,0.3-0.4,0.5-0.6l-0.6,0.5c-0.9-0.6,0.8-0.6,0.6-1.2c-1.2,0.7-2.4,1.9-3.5,2\n              c-0.5-0.7,2.2-1.1,0.6-1.3l1.6-0.2c-1.6-0.8,0.9-0.2-0.6-1.2c-2.1,0.5-2.1,1.2-4.1,1.7c-0.3,0.6,0.4,0.6,0.1,1.2\n              c-2.5,1.8-2.9-1.2-4.6,0.6c-0.3,0.7-0.2,1.8-1.9,1.8l0.2-1.1c-1,0.3-1.1,0.7-1.3,1.1c0,0.1-0.1,0.2-0.2,0.3\n              c-0.1,0.1-0.2,0.2-0.4,0.3c-0.3,0.2-0.8,0.3-1.7,0.5c0.3-0.5,2.3-1.1,1.8-1.1c-1.3,0-1.6,0.3-1.8,0.6c-0.2,0.3-0.2,0.7-1,0.9\n              c-0.3-0.2-0.7-0.3-1.1-0.5c-0.4,0.2-0.8,0.3-1.2,0.5c0.3,1.7,0.7,3.5,1,5.2l0.1,0.3l0.1,0.2l0.1,0.4l0.2,0.8c0.1,0.5,0.2,1,0.3,1.6\n              c-0.3-0.1-0.6-0.5-0.8,0.1c0.8,3,0.6,6.5,0.5,9.4l0.9,1c0,0.3-0.3,0.5-0.4,0.3c-0.1,1.2,0.5,0,0.8,0.5c0.1,0.6-0.3,0.9-0.6,0.5\n              c0.8,1.9,2,5.5,1.4,7.7l-0.3-0.4c-0.1,1.7,1.7,1.7,1.2,3.3l-0.2-0.3c0.1,3.6,1.2,7.5,0.4,10.3c1.1,0.4-0.1,3.6,1.4,3.6\n              c-0.5,2.1-0.8,1.6-1,4.5c0,0.8,1,1,1.3,2l-0.8,0.1l0.9,1.2l-0.8,0.8l0.3,0.9c0-0.4,0.3-0.8,0.5-0.8c0.5,1.3-0.1,3.4-0.5,3.5l1,3.2\n              c-0.2,0-0.4-0.3-0.6-0.7c0.6,1.2-0.9,0.2,0,1.6c0.1-0.2,0.5,0.1,0.6,0.1c-0.4,0.9-0.2,2.7,0.2,4l-0.6-0.2c0.1,2.4,0.8,5.1,1.1,7.8\n              c0.1-1,1.1-0.9,1.2-0.1l-0.8,0.4c0.6,0.5,0.2-0.9,0.9,0.1c0,1-0.8,0.8-1.2,0.2l0.4,1.2c-0.2,0.1-0.4-0.2-0.7-0.2\n              c0.3,0.2,0.1,0.8,0,1.4c0,1,0,2.3,0.5,2.8l-0.2,0.2c0,0.4,0.1,0.8,0.1,1c1,0.6,1.9,1,2.8,1.5c0.2,0.1,0.4,0.2,0.6,0.3l0.2,0.1\n              l0.2,0.1c0.2,0.1,0.4,0.2,0.6,0.3c0.8,0.4,1.5,0.7,2.5,1.2c0.4-0.6,1.1,0,1.9-0.2c1.3,1,2.7,2.4,4.4,3.3c1.4,0.4,0.6-0.3,1.4-0.4\n              c-0.2,0.6,1.5,1.5,0.4,1.4c2.1,1.1,2.7,1.4,5.6,2.4c-0.2-0.4-0.4-0.7-0.2-0.8c0.5,0.2,1,0.8,0.7,1.1l-0.2,0c3.4,0.8,5.9,5.4,8.9,5.5\n              l-0.2,0l2.8,0.7c1.1,0.7-0.9,0-0.3,0.8c3.1,0.5,4.1,2.4,6.6,3c0.8,2.6,6,3.2,7.7,5.5l0.1-0.5c0.8-0.1,0.6,0.6,1,0.9\n              c0.1-0.3-0.4-0.7,0-0.7l0.7,0.3l1.1,0.6c0.7,0.4,1.5,0.8,2.3,1.2c0.9-0.5,1.8-1,2.7-1.5l0.7-0.4l0.4-0.2l0.8-0.4\n              c1.1-0.5,2.2-1,3.3-1.5c2.3-1,4.5-1.9,6.7-2.9c1.9-0.9-0.1-2.6,2.8-3l-0.3,1.1c3.3-2.9,7.7-4.2,11-7c-0.1,0.3,0,0.5-0.4,0.7\n              c1.1,0.1,2.9-1.6,2.7-2c0.6-0.3,0.3,0.3,0.8,0.3c0.9-0.1,1.6-1.7,2.3-1.5c0.1,0,0,0.1-0.1,0.3c0.6-0.5,2.2-1.1,1.4-1.4\n              c1.5-0.5,2.5-1.7,3.8-2c0.3-1.1,2.5-1,2.2-2.1c0.4,1,2.9-0.5,3.5-1.5c-0.1,0.5,0.9-0.1,0.8,0.5c1-0.6,1.8-1.3,1.3-1.7\n              c0.4,0,0.7,0,1-0.1c0.1,0,0.3-0.1,0.4-0.2c0.1-0.1,0.4-0.2,0.6-0.4c0.9-0.6,1.6-1.2,2.3-1.7c0-1.4,0.1-2.3,0.9-3.1l0-0.3l0-0.2\n              l0-0.3l0-0.6c0-0.4,0-0.8-0.1-1.2c0-0.8,0-1.6-0.1-2.4c0-1.6,0-3.1,0-4.7c1.1,0.2-0.3,3.3,0.9,3.4c-0.1-2.9-0.9-4.7-0.3-7.5\n              c0.2-0.7,0.4,0.5,0.6,0.7c-0.1-2.9,0.7-5.5,0.4-8.1c0.1,0,0.2,0.3,0.2,0.6c0-0.5,0.3-1.1-0.2-1.1l0.1,0.5c-0.8,0.7-1.3-2.2-1.7-2.3\n              c1.4,1,0.7-2.5,2.3-0.9c-0.1-0.3-0.2-0.8-0.6-0.7c0.2-0.5,0.4-0.6,0.7-0.1c-0.1-1.8-0.7-1.3-0.5-2.9c0.2,0,0.3,0.4,0.6,0.6\n              c-0.4-0.9-0.8-1.8-0.4-3.1c0.1,0.1,0.3,0,0.3,0.4c0.2-2.4-0.8-4.2,0-6.7c1,0.9,0.5-0.8,1.4-0.9l-0.1-1.4l-0.5,0.5\n              c0.2-1.2-0.4-2.4,0-3.4l0.3,1l0.1-2.3l-0.5,1.3c-0.3-1.3-1.2,0-0.7-1.6c0.1,0.3,0.2-1.2,0.8-1.8l0,0c-0.2-0.6-0.5-1.3-0.1-2.2\n              c0.3-0.2,1,0.3,1.2,0.2c-0.7-0.7,0.1-2.9-0.6-3.7c0.2,0.3,0.6,0.3,0.8,0.3l-0.6-0.6c0.3-0.5,0.4-0.6,0.7-0.5\n              c0.3-1.7-1.2,0.4-0.5-1.6c0,0.2,0.1,0.1,0.2,0.4c0.5-2,0.1-4,0.6-5.8c1-0.4,0,1.7,0.9,1.1c0.1-2.7-0.5-5.7,0-8.3l-0.2,0.5\n              c-1.3-0.5,0.9-1.9,0.4-3.2c0.1,0.3,0.7,0.5,0.5,0.4c0.1-0.4,0.1-0.7,0.2-1.1c0-0.2,0-0.4,0-0.6c0-0.3,0-0.5,0-0.8\n              C238.4,25.2,238.4,24,238.3,22.8z M121.9,30c-0.1-0.3-0.1-0.5-0.1-0.7C121.8,29.5,121.8,29.7,121.9,30C121.9,30,121.9,30,121.9,30z\n              M122.2,32.9c0-0.4-0.1-0.8-0.1-1.3c0.2-0.1,0.5,0,0.5,0.2C122.5,31.8,122.4,32.5,122.2,32.9z M226.3,19.3c0,0.2-0.1,0.4-0.2,0.6\n              l-0.9-0.3C225.4,19.3,225.7,19.2,226.3,19.3z M198.9,9.5l0.2,0.7l-1.1-0.4C198.1,9.7,198.3,9.5,198.9,9.5z M191.7,6.9\n              c0.1,0.2,0.2,0.5,0,0.7l-1.3-0.5L191.7,6.9z\"></path>\n          <path class=\"command-bar__logo__letter\"\n                d=\"M180,16.9l-35.6,79.8l0,0h13.3l0,0l7.2-17.9H195l7.2,17.9l0,0h13.3l0,0L180,16.9L180,16.9L180,16.9L180,16.9\n              L180,16.9z M190.4,67.9h-20.8l10.4-25L190.4,67.9z\"></path>\n          <path class=\"command-bar__logo__letter\"\n                d=\"M500.3,129.5c-0.2,0-0.4-0.1-0.5-0.2c-0.2-0.2-0.2-0.3-0.2-0.5c0.2-2.1,0.3-4.2,0.3-6.3s0-4.2,0-6.3\n                c-0.1-0.2-0.2-0.3-0.2-0.5c0-0.2,0.1-0.4,0.2-0.5c0.1-0.2,0.2-0.4,0.5-0.4s0.4,0,0.7,0.1c0.6-0.1,1.2-0.2,1.8-0.2\n                c0.6,0,1.2,0,1.9,0s1.2,0,1.9,0.1c0.6,0,1.2,0,1.9,0c0.2,0,0.4,0.1,0.5,0.2s0.2,0.3,0.2,0.5s-0.1,0.4-0.2,0.6s-0.3,0.3-0.5,0.3\n                c-0.6,0-1.2,0-1.8,0c-0.6,0-1.2,0-1.8-0.1s-1.2,0-1.8,0c-0.6,0-1.2,0.1-1.7,0.2c0,0.9,0,1.8,0,2.7c0,0.9,0,1.8,0,2.7\n                c0.4,0,0.8,0,1.2,0c0.4,0,0.8,0,1.2,0s0.8,0,1.2,0c0.4,0,0.8,0.1,1.2,0.2c0.2,0.1,0.4,0.2,0.5,0.4s0.1,0.4,0,0.6\n                c-0.1,0.2-0.2,0.3-0.3,0.5c-0.2,0.1-0.4,0.2-0.6,0.1c-0.7-0.2-1.5-0.2-2.2-0.2s-1.5,0.1-2.2,0.1c0,0.9-0.1,1.8-0.1,2.8\n                c0,0.9-0.1,1.8-0.2,2.8c0,0.2-0.1,0.4-0.2,0.5S500.5,129.5,500.3,129.5z\"></path>\n          <path class=\"command-bar__logo__letter\"\n                d=\"M514.6,127.9c-0.3-0.2-0.6-0.5-0.8-0.8c-0.2-0.3-0.5-0.6-0.6-0.9c-0.2-0.3-0.4-0.6-0.5-0.9\n                c-0.2-0.3-0.3-0.6-0.4-1c-0.1-0.3-0.2-0.7-0.3-1c-0.1-0.3-0.1-0.7-0.1-1c0-0.7,0.1-1.3,0.2-2c0.1-0.6,0.2-1.3,0.4-1.9\n                c0.1-0.2,0.1-0.5,0.2-0.8s0.2-0.6,0.3-0.8s0.3-0.5,0.4-0.7s0.4-0.4,0.6-0.5c0.4-0.2,0.8-0.2,1.2-0.2s0.8,0.2,1.1,0.3\n                s0.7,0.4,1.1,0.6s0.6,0.5,0.9,0.7c0.3,0.2,0.5,0.5,0.8,0.7c0.3,0.2,0.5,0.5,0.8,0.7c0.3,0.3,0.7,0.6,1,0.9s0.6,0.6,0.8,1\n                c0.3,0.4,0.4,0.9,0.4,1.3s0,0.9-0.1,1.3s-0.3,0.9-0.5,1.3c-0.2,0.4-0.5,0.8-0.7,1.2c-0.2,0.3-0.5,0.7-0.8,1.1s-0.6,0.7-1,1\n                c-0.4,0.3-0.8,0.6-1.2,0.7s-0.9,0.2-1.3,0.2c-0.3,0-0.5-0.1-0.6-0.3c-0.2,0-0.5-0.1-0.7-0.2S514.8,128,514.6,127.9z M513.3,122.3\n                c0,0.3,0,0.6,0,0.9s0.2,0.6,0.3,0.9c0.2,0.4,0.4,0.8,0.6,1.1s0.5,0.7,0.8,1c0.3,0.3,0.6,0.5,0.9,0.5c0.2,0,0.4,0.1,0.6,0.1\n                c0.2,0,0.4,0.1,0.6,0.1c0.1,0,0.2-0.1,0.2-0.1s0.2-0.1,0.2-0.1c0.2-0.1,0.4-0.3,0.6-0.5s0.4-0.3,0.5-0.5c0.3-0.4,0.6-0.8,0.9-1.2\n                c0.3-0.4,0.5-0.9,0.7-1.3c0.2-0.5,0.3-0.9,0.2-1.3c0-0.4-0.2-0.8-0.6-1.2c-0.3-0.4-0.6-0.7-1-1s-0.7-0.6-1.1-0.9\n                c-0.3-0.3-0.6-0.5-0.9-0.7c-0.3-0.2-0.6-0.5-0.9-0.7c-0.3-0.2-0.6-0.4-0.9-0.5c0,0-0.1,0-0.1,0s-0.1,0-0.1,0l-0.3,0\n                c-0.1,0-0.2,0.1-0.2,0.1c-0.1,0.1-0.1,0.2-0.2,0.2c-0.1,0.2-0.1,0.3-0.2,0.3s-0.1,0.2-0.1,0.3c-0.1,0.4-0.2,0.8-0.3,1.1\n                s-0.2,0.8-0.2,1.1s-0.1,0.7-0.2,1S513.4,121.9,513.3,122.3z\"></path>\n          <path class=\"command-bar__logo__letter\"\n                d=\"M534.8,128.4c-0.1,0.2-0.3,0.3-0.5,0.4c-0.2,0.1-0.4,0-0.6-0.1c-0.3-0.3-0.7-0.5-1-0.8c-0.3-0.3-0.7-0.5-1-0.8\n                c-0.3-0.2-0.7-0.4-1-0.6s-0.7-0.4-1-0.6c-0.4-0.3-0.7-0.6-1-0.9c0,0.7,0,1.3,0,1.9c0,0.6,0.1,1.3,0.1,1.9c0,0.2,0,0.4-0.2,0.6\n                s-0.3,0.2-0.5,0.2c-0.2,0-0.4-0.1-0.5-0.2c-0.1-0.1-0.2-0.3-0.2-0.6c-0.1-0.9-0.1-1.9-0.1-2.8s0-1.8,0.1-2.7\n                c-0.1-0.1-0.2-0.3-0.3-0.4c-0.1-0.1-0.2-0.3-0.3-0.5c-0.1-0.2-0.1-0.3-0.1-0.4c0-0.1,0.1-0.3,0.2-0.4c0.1-0.1,0.2-0.2,0.3-0.3\n                s0.3-0.1,0.4-0.1c0-0.3,0.1-0.6,0.1-0.9c0-0.3,0.1-0.6,0.1-0.9c0-0.2,0.1-0.4,0.1-0.6s0.1-0.4,0.1-0.6c0-0.1,0-0.2,0-0.3\n                s0-0.2,0-0.3c0-0.1,0-0.1,0-0.2c0-0.2,0-0.3,0-0.5s0-0.3,0-0.5c0-0.2,0-0.4,0.1-0.7s0.1-0.5,0.2-0.7c0.1-0.2,0.2-0.4,0.3-0.6\n                c0.1-0.2,0.3-0.3,0.5-0.4c0.2-0.1,0.4-0.1,0.6-0.2s0.4,0,0.6,0c0.2,0,0.4,0.1,0.6,0.2c0.2,0.1,0.4,0.1,0.6,0.2\n                c0.5,0.2,0.9,0.4,1.3,0.6s0.8,0.5,1.2,0.8c0.3,0.3,0.6,0.7,0.9,1.2s0.4,0.9,0.3,1.4c0,0.3-0.1,0.6-0.2,0.8s-0.2,0.5-0.4,0.7\n                c-0.4,0.6-0.9,1.1-1.3,1.7s-1,1-1.5,1.5c-0.3,0.2-0.5,0.4-0.8,0.6c-0.3,0.2-0.6,0.4-1,0.5c0.1,0.1,0.2,0.2,0.3,0.3\n                c0.1,0.1,0.2,0.2,0.4,0.3c0.3,0.2,0.7,0.4,1,0.6s0.7,0.4,1,0.6c0.3,0.2,0.6,0.5,0.9,0.7c0.3,0.2,0.6,0.5,0.9,0.7\n                c0.2,0.1,0.3,0.3,0.4,0.5S534.9,128.2,534.8,128.4z M529.6,122.6c0.3-0.2,0.6-0.3,0.8-0.5c0.3-0.2,0.5-0.4,0.7-0.6\n                c0.3-0.2,0.5-0.5,0.8-0.7c0.2-0.3,0.5-0.5,0.7-0.8c0.1-0.1,0.2-0.2,0.3-0.3s0.1-0.2,0.2-0.3s0.1-0.2,0.2-0.3s0.1-0.2,0.2-0.3\n                c0,0,0.1-0.1,0.1-0.1s0.1-0.1,0.1-0.1c0-0.4-0.1-0.7-0.2-1c-0.2-0.4-0.5-0.7-0.8-0.9c-0.3-0.2-0.7-0.4-1.1-0.6\n                c-0.2-0.1-0.5-0.2-0.7-0.3s-0.5-0.1-0.7-0.2c-0.1,0-0.2-0.1-0.3-0.1c-0.1,0-0.1,0.1-0.2,0.1c-0.1,0.2-0.1,0.3-0.1,0.4\n                c0,0,0,0.1-0.1,0.2c0,0.1,0,0.2,0,0.3c0,0.1,0,0.1,0,0.2c0,0.9-0.1,1.8-0.3,2.7c-0.1,0.9-0.3,1.8-0.4,2.7c0,0.1,0,0.3,0,0.4\n                s0,0.3,0,0.4c0.1,0,0.2,0,0.2,0c0.1,0,0.3-0.1,0.3-0.1C529.4,122.6,529.5,122.6,529.6,122.6z\"></path>\n          <path class=\"command-bar__logo__letter\"\n                d=\"M551.5,128.6c0,0.2-0.1,0.4-0.2,0.6c-0.2,0.1-0.3,0.2-0.5,0.2s-0.4-0.1-0.5-0.2s-0.2-0.3-0.2-0.6\n                c0-1,0.1-2,0.3-2.9s0.5-1.9,0.9-2.8c-0.1-0.2-0.1-0.4-0.1-0.6c0.1-0.3,0.2-0.4,0.5-0.5c0.3-0.6,0.5-1.2,0.8-1.9s0.5-1.2,0.8-1.9\n                c0.3-0.6,0.5-1.3,0.7-1.9s0.4-1.3,0.5-2c0-0.2,0.1-0.4,0.2-0.5c0.1-0.1,0.3-0.2,0.5-0.2c0.2,0,0.4,0.1,0.6,0.2s0.2,0.3,0.2,0.6\n                c0,0.8,0,1.4,0,2.1s0.2,1.3,0.3,1.9s0.3,1.2,0.4,1.9c0.2,0.6,0.3,1.2,0.5,1.8c0.1,0.6,0.3,1.2,0.4,1.8c0.1,0.6,0.3,1.2,0.6,1.7\n                c0.1,0.3,0.3,0.5,0.4,0.8s0.3,0.5,0.5,0.8c0.2,0.2,0.3,0.5,0.4,0.7c0.1,0.3,0.2,0.5,0.3,0.8c0,0.2,0,0.4-0.1,0.6\n                c-0.1,0.2-0.2,0.3-0.5,0.4c-0.2,0.1-0.4,0-0.6-0.1c-0.2-0.1-0.3-0.3-0.4-0.5c-0.1-0.3-0.2-0.6-0.3-0.8c-0.1-0.2-0.3-0.5-0.5-0.7\n                c-0.2-0.3-0.3-0.6-0.5-0.9s-0.3-0.6-0.4-0.9c-0.1-0.3-0.2-0.7-0.3-1s-0.2-0.7-0.3-1c-0.6,0-1.1-0.1-1.7-0.1c-0.6,0-1.1-0.1-1.7-0.1\n                c-0.3,0.8-0.6,1.7-0.8,2.5S551.5,127.7,551.5,128.6z M553.3,121.7c0,0,0,0.1,0,0.1s0,0.1,0,0.1c0.4,0,0.8,0.1,1.2,0.1s0.8,0,1.2,0\n                c-0.1-0.6-0.3-1.2-0.5-1.9c-0.2-0.6-0.3-1.2-0.4-1.9c-0.2,0.6-0.5,1.1-0.7,1.7C553.8,120.5,553.6,121.1,553.3,121.7z\"></path>\n          <path class=\"command-bar__logo__letter\"\n                d=\"M564.8,117.4C564.8,117.4,564.8,117.3,564.8,117.4c-0.1-0.1-0.1-0.1-0.1-0.2c-0.2-0.2-0.2-0.3-0.2-0.5\n                c0-0.2,0.1-0.3,0.2-0.5c0.1-0.1,0.2-0.2,0.4-0.3s0.3-0.1,0.5,0c0.2,0,0.4,0,0.6,0.1c0.2,0.1,0.3,0.3,0.3,0.6\n                c0.7,0.8,1.3,1.6,1.9,2.3c0.6,0.8,1.1,1.6,1.6,2.4c0.2,0.4,0.4,0.8,0.6,1.2s0.4,0.8,0.5,1.2c0-0.5,0-1.1,0.1-1.6s0.1-1.1,0.2-1.6\n                c0.1-1.1,0.3-2.2,0.4-3.3s0.2-2.2,0.2-3.4c0-0.2,0.1-0.4,0.2-0.6s0.3-0.2,0.5-0.2s0.4,0.1,0.5,0.2c0.2,0.1,0.2,0.3,0.2,0.6\n                c0,1.2-0.1,2.3-0.2,3.5c-0.1,1.1-0.3,2.3-0.4,3.4c-0.1,0.6-0.1,1.2-0.2,1.7c0,0.6-0.1,1.1,0,1.7c0,0.6,0,1.1,0,1.7\n                c0,0.5,0,1.1,0,1.7c0,0.2-0.1,0.4-0.2,0.5s-0.2,0.2-0.4,0.2c-0.2,0-0.3,0-0.5,0s-0.3-0.2-0.4-0.3c-0.3-0.5-0.5-0.9-0.8-1.4\n                c-0.2-0.5-0.4-1-0.6-1.5c-0.2-0.5-0.4-0.9-0.6-1.4c-0.2-0.4-0.4-0.9-0.7-1.3c-0.3-0.6-0.7-1.1-1-1.7s-0.8-1.1-1.2-1.6\n                c0,0.5,0,1.1,0,1.6s0,1,0,1.6c0,0.9,0.1,1.9,0.2,2.8s0.2,1.8,0.3,2.8c0,0.2,0,0.4-0.2,0.6c-0.2,0.1-0.4,0.2-0.5,0.2\n                c-0.2,0-0.4-0.1-0.5-0.2c-0.1-0.1-0.2-0.3-0.2-0.6c-0.1-0.9-0.2-1.8-0.3-2.8c-0.1-0.9-0.2-1.8-0.2-2.8c0-0.8,0-1.6,0-2.5\n                C564.8,119,564.8,118.2,564.8,117.4z\"></path>\n          <path class=\"command-bar__logo__letter\"\n                d=\"M580.8,129.2c-0.4-0.2-0.7-0.6-1-0.9c-0.3-0.4-0.6-0.8-0.8-1.2c-0.2-0.4-0.4-0.9-0.6-1.3s-0.3-0.9-0.3-1.4\n                c-0.1-0.4-0.1-0.9-0.1-1.3c0-0.4,0-0.9,0.2-1.3c0.1-0.3,0.2-0.7,0.4-1s0.4-0.6,0.6-0.9c0.5-0.7,1.1-1.4,1.7-2s1.3-1.2,2-1.6\n                c0.3-0.2,0.7-0.4,1.1-0.6s0.7-0.3,1.1-0.4c0.4-0.1,0.8-0.1,1.1-0.1c0.4,0,0.8,0.1,1.2,0.3c0.2,0.1,0.3,0.2,0.4,0.5\n                c0.1,0.2,0,0.4-0.1,0.6c-0.1,0.2-0.3,0.3-0.5,0.4s-0.4,0-0.6-0.1c-0.2-0.1-0.5-0.1-0.8-0.1c-0.1,0-0.3,0-0.5,0.1\n                c-0.2,0-0.4,0.1-0.5,0.2c-0.4,0.1-0.7,0.3-1.1,0.6s-0.7,0.5-1,0.8c-0.6,0.5-1.1,1-1.6,1.6c-0.5,0.6-0.9,1.2-1.2,1.9\n                c0,0.1-0.1,0.2-0.1,0.4c0,0.2-0.1,0.3-0.1,0.5c0,0.4,0,0.7,0.1,1.1c0.1,0.4,0.2,0.9,0.3,1.3s0.3,0.8,0.5,1.2c0.2,0.4,0.4,0.7,0.7,1\n                s0.6,0.6,1,0.7c0.4,0.1,0.9,0.1,1.3-0.1c0.1,0,0.2-0.1,0.3-0.1c0.1,0,0.2-0.1,0.3-0.1c0,0,0.1-0.1,0.1-0.1c0,0,0.1-0.1,0.1-0.1\n                c0.3-0.3,0.6-0.5,0.8-0.8s0.5-0.6,0.7-0.9c0.1-0.2,0.2-0.4,0.3-0.6s0.2-0.4,0.3-0.6c0.1-0.2,0.1-0.3,0.2-0.5c0-0.1,0.1-0.3,0.1-0.4\n                c-0.1-0.1-0.3-0.1-0.4-0.1c-0.1,0-0.3,0-0.4,0c-0.4,0-0.7,0-1.1,0.1c-0.4,0-0.7,0.1-1.1,0.1c-0.2,0-0.4-0.1-0.5-0.2\n                c-0.1-0.2-0.2-0.3-0.2-0.5s0.1-0.4,0.2-0.5c0.1-0.2,0.3-0.2,0.5-0.2c0.3,0,0.7,0,1-0.1c0.3,0,0.7-0.1,1-0.1c0.3,0,0.7,0,1,0\n                c0.3,0,0.7,0.1,1,0.3c0.3,0.1,0.6,0.4,0.8,0.7c0.1,0.2,0.2,0.5,0.2,0.7c0,0.3,0,0.5,0,0.8c0,0.6,0,1.1,0,1.7c0,0.6,0,1.1,0,1.7\n                c0,0.3-0.1,0.5-0.3,0.6s-0.3,0.2-0.5,0.2c-0.2,0-0.4-0.1-0.5-0.2s-0.2-0.3-0.2-0.6c0-0.3,0-0.5,0-0.8s0-0.5,0-0.8\n                c-0.4,0.6-0.8,1.2-1.3,1.7c-0.3,0.3-0.7,0.6-1.1,0.9s-0.8,0.4-1.3,0.5s-0.9,0.1-1.4,0.1C581.6,129.6,581.2,129.4,580.8,129.2z\"></path>\n          <path class=\"command-bar__logo__letter\"\n                d=\"M594.1,123.5c-0.1-1.3-0.2-2.6-0.3-3.9c-0.1-1.3-0.1-2.6-0.3-3.9c0-0.2,0-0.4,0.2-0.6c0.2-0.1,0.4-0.2,0.5-0.2\n                c0.2,0,0.4,0.1,0.5,0.2c0.1,0.1,0.2,0.3,0.2,0.6c0.1,0.8,0.2,1.6,0.2,2.3s0.1,1.5,0.1,2.3c0.1,1.1,0.1,2.3,0.3,3.4\n                c0,0.1,0,0.3,0,0.5s0,0.3,0,0.4c0,0.2,0.1,0.4,0.1,0.5s0.1,0.3,0.1,0.5c0.1,0.3,0.2,0.6,0.3,0.9s0.2,0.5,0.4,0.7l0.2,0.1\n                c0.1,0,0.2,0,0.3,0c0.1,0,0.2,0,0.4-0.1c0.2-0.1,0.3-0.1,0.5-0.2c0.3-0.2,0.6-0.5,0.9-0.9s0.5-0.7,0.6-1.1c0.2-0.4,0.4-0.8,0.5-1.3\n                c0.2-0.4,0.3-0.9,0.4-1.3c0.3-1.1,0.4-2.3,0.3-3.5s-0.1-2.3-0.3-3.5c0-0.2,0-0.4,0.2-0.5c0.2-0.2,0.4-0.2,0.5-0.2\n                c0.2,0,0.4,0.1,0.5,0.2c0.1,0.2,0.2,0.3,0.2,0.5c0.2,1.3,0.3,2.5,0.3,3.8s-0.1,2.5-0.5,3.8c-0.1,0.4-0.3,0.9-0.4,1.4\n                s-0.3,0.9-0.5,1.4s-0.5,0.9-0.7,1.3c-0.3,0.4-0.6,0.7-1,1c-0.4,0.3-0.8,0.5-1.2,0.7s-0.9,0.2-1.3,0.1c-0.3-0.1-0.7-0.2-0.9-0.5\n                s-0.5-0.5-0.7-0.8c-0.3-0.7-0.6-1.3-0.7-2.1S594.2,124.2,594.1,123.5z\"></path>\n          <path class=\"command-bar__logo__letter\"\n                d=\"M608.4,129c-0.2,0-0.4-0.1-0.5-0.2c-0.1-0.1-0.2-0.2-0.2-0.4c0-0.1,0-0.3,0-0.5c0-0.2,0.1-0.3,0.3-0.4\n                c0.1-0.4,0.1-0.9,0.1-1.3s-0.1-0.9-0.2-1.3c-0.1-0.5-0.1-1.1-0.1-1.6s0.1-1.1,0.2-1.6s0.2-1.1,0.3-1.6c0.1-0.5,0.2-1.1,0.2-1.6\n                c0.1-0.6,0.1-1.2,0.2-1.7s0.1-1.2,0.1-1.7c0-0.2,0.1-0.4,0.3-0.6c0.2-0.1,0.3-0.2,0.5-0.2c0.2,0,0.4,0.1,0.5,0.2s0.2,0.3,0.2,0.6\n                c0,0.6-0.1,1.2-0.1,1.8s-0.1,1.2-0.2,1.8c-0.1,0.6-0.1,1.1-0.2,1.7c-0.1,0.5-0.2,1.1-0.3,1.7c-0.1,0.5-0.1,1.1-0.1,1.6\n                s0,1.1,0.1,1.6c0,0.4,0.1,0.8,0.1,1.1c0,0.4,0,0.8,0,1.1c1,0,1.9,0.1,2.9,0.1c1,0.1,1.9,0.1,2.9,0.2c0.2,0,0.4,0.1,0.5,0.3\n                c0.1,0.2,0.2,0.4,0.2,0.5s-0.1,0.4-0.2,0.5s-0.3,0.2-0.5,0.2c-1.2-0.1-2.3-0.1-3.5-0.2S609.6,129,608.4,129z\"></path>\n          <path class=\"command-bar__logo__letter\"\n                d=\"M621.4,128.6c0,0.2-0.1,0.4-0.2,0.6c-0.2,0.1-0.3,0.2-0.5,0.2s-0.4-0.1-0.5-0.2s-0.2-0.3-0.2-0.6\n                c0-1,0.1-2,0.3-2.9s0.5-1.9,0.9-2.8c-0.1-0.2-0.1-0.4-0.1-0.6c0.1-0.3,0.2-0.4,0.5-0.5c0.3-0.6,0.5-1.2,0.8-1.9s0.5-1.2,0.8-1.9\n                c0.3-0.6,0.5-1.3,0.7-1.9s0.4-1.3,0.5-2c0-0.2,0.1-0.4,0.2-0.5c0.1-0.1,0.3-0.2,0.5-0.2c0.2,0,0.4,0.1,0.6,0.2s0.2,0.3,0.2,0.6\n                c0,0.8,0,1.4,0,2.1s0.2,1.3,0.3,1.9s0.3,1.2,0.4,1.9c0.2,0.6,0.3,1.2,0.5,1.8c0.1,0.6,0.3,1.2,0.4,1.8c0.1,0.6,0.3,1.2,0.6,1.7\n                c0.1,0.3,0.3,0.5,0.4,0.8s0.3,0.5,0.5,0.8c0.2,0.2,0.3,0.5,0.4,0.7c0.1,0.3,0.2,0.5,0.3,0.8c0,0.2,0,0.4-0.1,0.6\n                c-0.1,0.2-0.2,0.3-0.5,0.4c-0.2,0.1-0.4,0-0.6-0.1c-0.2-0.1-0.3-0.3-0.4-0.5c-0.1-0.3-0.2-0.6-0.3-0.8c-0.1-0.2-0.3-0.5-0.5-0.7\n                c-0.2-0.3-0.3-0.6-0.5-0.9s-0.3-0.6-0.4-0.9c-0.1-0.3-0.2-0.7-0.3-1s-0.2-0.7-0.3-1c-0.6,0-1.1-0.1-1.7-0.1c-0.6,0-1.1-0.1-1.7-0.1\n                c-0.3,0.8-0.6,1.7-0.8,2.5S621.4,127.7,621.4,128.6z M623.2,121.7c0,0,0,0.1,0,0.1s0,0.1,0,0.1c0.4,0,0.8,0.1,1.2,0.1s0.8,0,1.2,0\n                c-0.1-0.6-0.3-1.2-0.5-1.9c-0.2-0.6-0.3-1.2-0.4-1.9c-0.2,0.6-0.5,1.1-0.7,1.7C623.7,120.5,623.4,121.1,623.2,121.7z\"></path>\n          <path class=\"command-bar__logo__letter\"\n                d=\"M642.2,128.4c-0.1,0.2-0.3,0.3-0.5,0.4c-0.2,0.1-0.4,0-0.6-0.1c-0.3-0.3-0.7-0.5-1-0.8c-0.3-0.3-0.7-0.5-1-0.8\n                c-0.3-0.2-0.7-0.4-1-0.6s-0.7-0.4-1-0.6c-0.4-0.3-0.7-0.6-1-0.9c0,0.7,0,1.3,0,1.9c0,0.6,0.1,1.3,0.1,1.9c0,0.2,0,0.4-0.2,0.6\n                s-0.3,0.2-0.5,0.2c-0.2,0-0.4-0.1-0.5-0.2c-0.1-0.1-0.2-0.3-0.2-0.6c-0.1-0.9-0.1-1.9-0.1-2.8s0-1.8,0.1-2.7\n                c-0.1-0.1-0.2-0.3-0.3-0.4c-0.1-0.1-0.2-0.3-0.3-0.5c-0.1-0.2-0.1-0.3-0.1-0.4c0-0.1,0.1-0.3,0.2-0.4c0.1-0.1,0.2-0.2,0.3-0.3\n                s0.3-0.1,0.4-0.1c0-0.3,0.1-0.6,0.1-0.9c0-0.3,0.1-0.6,0.1-0.9c0-0.2,0.1-0.4,0.1-0.6s0.1-0.4,0.1-0.6c0-0.1,0-0.2,0-0.3\n                s0-0.2,0-0.3c0-0.1,0-0.1,0-0.2c0-0.2,0-0.3,0-0.5s0-0.3,0-0.5c0-0.2,0-0.4,0.1-0.7s0.1-0.5,0.2-0.7c0.1-0.2,0.2-0.4,0.3-0.6\n                c0.1-0.2,0.3-0.3,0.5-0.4c0.2-0.1,0.4-0.1,0.6-0.2s0.4,0,0.6,0c0.2,0,0.4,0.1,0.6,0.2c0.2,0.1,0.4,0.1,0.6,0.2\n                c0.5,0.2,0.9,0.4,1.3,0.6s0.8,0.5,1.2,0.8c0.3,0.3,0.6,0.7,0.9,1.2s0.4,0.9,0.3,1.4c0,0.3-0.1,0.6-0.2,0.8s-0.2,0.5-0.4,0.7\n                c-0.4,0.6-0.9,1.1-1.3,1.7s-1,1-1.5,1.5c-0.3,0.2-0.5,0.4-0.8,0.6c-0.3,0.2-0.6,0.4-1,0.5c0.1,0.1,0.2,0.2,0.3,0.3\n                c0.1,0.1,0.2,0.2,0.4,0.3c0.3,0.2,0.7,0.4,1,0.6s0.7,0.4,1,0.6c0.3,0.2,0.6,0.5,0.9,0.7c0.3,0.2,0.6,0.5,0.9,0.7\n                c0.2,0.1,0.3,0.3,0.4,0.5S642.3,128.2,642.2,128.4z M637,122.6c0.3-0.2,0.6-0.3,0.8-0.5c0.3-0.2,0.5-0.4,0.7-0.6\n                c0.3-0.2,0.5-0.5,0.8-0.7c0.2-0.3,0.5-0.5,0.7-0.8c0.1-0.1,0.2-0.2,0.3-0.3s0.1-0.2,0.2-0.3s0.1-0.2,0.2-0.3s0.1-0.2,0.2-0.3\n                c0,0,0.1-0.1,0.1-0.1s0.1-0.1,0.1-0.1c0-0.4-0.1-0.7-0.2-1c-0.2-0.4-0.5-0.7-0.8-0.9c-0.3-0.2-0.7-0.4-1.1-0.6\n                c-0.2-0.1-0.5-0.2-0.7-0.3s-0.5-0.1-0.7-0.2c-0.1,0-0.2-0.1-0.3-0.1c-0.1,0-0.1,0.1-0.2,0.1c-0.1,0.2-0.1,0.3-0.1,0.4\n                c0,0,0,0.1-0.1,0.2c0,0.1,0,0.2,0,0.3c0,0.1,0,0.1,0,0.2c0,0.9-0.1,1.8-0.3,2.7c-0.1,0.9-0.3,1.8-0.4,2.7c0,0.1,0,0.3,0,0.4\n                s0,0.3,0,0.4c0.1,0,0.2,0,0.2,0c0.1,0,0.3-0.1,0.3-0.1C636.8,122.6,636.9,122.6,637,122.6z\"></path>\n        </svg>",
            styles: ["\n    .command-bar__logo {\n        height: 30px;\n        width: 140px;\n    }\n\n    .command-bar__logo__box {\n        fill: rgba(255, 255, 255, .1);\n        transition: fill 0.2s ease-out;\n    }\n\n    .command-bar__logo__letter {\n        fill: rgba(255, 255, 255, .5);\n        transition: fill 0.2s ease-out;\n    }\n    "]
        })
    ], LogoComponent);
    return LogoComponent;
}());

var PinComponent = /** @class */ (function () {
    function PinComponent() {
    }
    __decorate([
        Input()
    ], PinComponent.prototype, "selected", void 0);
    PinComponent = __decorate([
        Component({
            selector: 'ap-pin',
            template: "<svg class=\"command-bar__scenario-icon\"\n    [class.command-bar__scenario-icon--selected]=\"selected\"\n    viewBox=\"25 25 50 50\">\n<path\n    d=\"M70.32,34.393l-7.628-8.203c-0.854-0.916-2.256-1.066-3.281-0.342l-13.699,9.639c-0.479-0.055-0.956-0.082-1.425-0.082 c-5.935,0-9.126,4.326-9.259,4.51c-0.718,0.994-0.612,2.359,0.249,3.232l7.88,7.98L30.436,63.848c-0.98,0.98-0.98,2.568,0,3.549 c0.49,0.49,1.132,0.734,1.774,0.734c0.642,0,1.284-0.244,1.773-0.734l12.7-12.699l7.34,7.432c0.484,0.49,1.131,0.746,1.786,0.746 c0.436,0,0.874-0.113,1.27-0.346c4.014-2.357,3.876-9.373,3.557-12.727l9.799-12.125C71.22,36.707,71.171,35.307,70.32,34.393z M56.073,47.465c-0.432,0.535-0.626,1.225-0.536,1.906c0.332,2.51,0.239,5.236-0.146,7.002L40.678,41.475 c0.868-0.551,2.079-1.051,3.61-1.051c0.5,0,1.02,0.053,1.546,0.158c0.674,0.137,1.375-0.01,1.938-0.408l12.737-8.963l4.655,5.006 L56.073,47.465z\"></path>\n</svg>",
            styles: ["\n   .command-bar__scenario-icon {\n        display: inline-block;\n        fill: white;\n        height: 20px;\n        margin: 2px 6px 0 0;\n        opacity: .2;\n        width: 20px;\n    }\n\n    .command-bar__scenario-icon--selected {\n        fill: #0097fb;\n        opacity: 1;\n    }\n    "]
        })
    ], PinComponent);
    return PinComponent;
}());

var DrawerComponent = /** @class */ (function () {
    function DrawerComponent() {
        this.openCommandBarClick = new EventEmitter();
    }
    DrawerComponent.prototype.openCommandBar = function () {
        this.openCommandBarClick.emit();
    };
    __decorate([
        Output()
    ], DrawerComponent.prototype, "openCommandBarClick", void 0);
    DrawerComponent = __decorate([
        Component({
            selector: 'ap-drawer',
            template: "<a\n    (click)=\"openCommandBar()\"\n    class=\"link\">\n    Menu\n</a>\n",
            styles: [".link{-webkit-box-align:center;align-items:center;background:#252526;border-radius:50%;bottom:20px;box-shadow:0 0 8px 2px rgba(0,0,0,.5);color:rgba(255,255,255,.5);cursor:pointer;display:-webkit-box;display:flex;-webkit-box-orient:vertical;-webkit-box-direction:normal;flex-direction:column;font-family:Arial,sans-serif;font-size:8px;font-weight:700;height:45px;-webkit-box-pack:center;justify-content:center;position:fixed;right:20px;text-align:center;text-transform:uppercase;-webkit-transition:opacity .5s,-webkit-transform .1s;transition:transform .1s,opacity .5s,-webkit-transform .1s;width:45px;z-index:10000}.link::before{background:-webkit-gradient(linear,left top,left bottom,from(rgba(255,255,255,.5)),color-stop(20%,rgba(255,255,255,.5)),color-stop(20%,transparent),color-stop(40%,transparent),color-stop(40%,rgba(255,255,255,.5)),color-stop(60%,rgba(255,255,255,.5)),color-stop(60%,transparent),color-stop(80%,transparent),color-stop(80%,rgba(255,255,255,.5)),to(rgba(255,255,255,.5)));background:linear-gradient(to bottom,rgba(255,255,255,.5),rgba(255,255,255,.5) 20%,transparent 20%,transparent 40%,rgba(255,255,255,.5) 40%,rgba(255,255,255,.5) 60%,transparent 60%,transparent 80%,rgba(255,255,255,.5) 80%,rgba(255,255,255,.5) 100%);content:'';height:10px;margin-bottom:4px;width:16px}.link:hover{-webkit-transform:scale(1.15);transform:scale(1.15)}:host(.content__menu--hide) .link{opacity:0}"]
        })
    ], DrawerComponent);
    return DrawerComponent;
}());

var PlaygroundCommonModule = /** @class */ (function () {
    function PlaygroundCommonModule() {
    }
    PlaygroundCommonModule = __decorate([
        NgModule({
            imports: [
                CommonModule,
                ReactiveFormsModule,
            ],
            providers: [
                Location,
                { provide: LocationStrategy, useClass: PathLocationStrategy },
                StateService,
                UrlService,
                LevenshteinDistance,
            ],
            declarations: [
                AppComponent,
                ScenarioComponent,
                FocusDirective,
                HighlightSearchMatchPipe,
                LogoComponent,
                PinComponent,
                DrawerComponent,
            ],
            exports: [AppComponent],
        })
    ], PlaygroundCommonModule);
    return PlaygroundCommonModule;
}());

var initializePlayground = function (elementNameToReplace) {
    document.getElementsByTagName('title')[0].innerHTML = 'Playground';
    if (elementNameToReplace && elementNameToReplace.length > 0) {
        var appNode = document.getElementsByTagName(elementNameToReplace)[0];
        if (!appNode) {
            throw new Error("Your configured selector (" + elementNameToReplace + ") does not match your app root selector.");
        }
        appNode.parentNode.replaceChild(document.createElement('playground-app'), appNode);
    }
    var resetStyles = "\n    // Playground reset styles\n    html {\n      -ms-text-size-adjust: 100%;\n      -webkit-text-size-adjust: 100%;\n    }\n    body {\n      margin: 0;\n    }\n  ";
    var style = document.createElement('style');
    style.type = 'text/css';
    style.appendChild(document.createTextNode(resetStyles));
    document.head.insertBefore(style, document.head.firstChild);
};

var _middleware = new BehaviorSubject({
    selector: null,
    overlay: false,
    modules: [],
});
var middleware = _middleware.asObservable();
var ɵ0 = middleware;
var PlaygroundModule = /** @class */ (function () {
    function PlaygroundModule() {
    }
    PlaygroundModule.configure = function (configuration) {
        initializePlayground(configuration.selector);
        _middleware.next(__assign(__assign({}, _middleware.value), configuration));
        return this;
    };
    PlaygroundModule = __decorate([
        NgModule({
            imports: [
                BrowserModule,
                PlaygroundCommonModule,
            ],
            providers: [
                { provide: MIDDLEWARE, useValue: ɵ0 },
            ],
            exports: [
                PlaygroundCommonModule
            ]
        })
    ], PlaygroundModule);
    return PlaygroundModule;
}());

var SandboxBuilder = /** @class */ (function () {
    function SandboxBuilder(_type, _config) {
        if (_config === void 0) { _config = {}; }
        this._type = _type;
        this._config = _config;
        this._scenarios = [];
        this._scenarioCounter = 0;
    }
    SandboxBuilder.prototype.add = function (description, config) {
        var key = ++this._scenarioCounter;
        this._scenarios.push(Object.assign({}, config, { key: key }));
        return this;
    };
    SandboxBuilder.prototype.serialize = function (sandboxPath) {
        return {
            key: sandboxPath,
            type: this._type,
            scenarios: this._scenarios,
            imports: this._config.imports || [],
            declarations: this._config.declarations || [],
            entryComponents: this._config.entryComponents || [],
            providers: this._config.providers || [],
            schemas: this._config.schemas || [],
            declareComponent: this._config.declareComponent !== undefined ? this._config.declareComponent : true,
        };
    };
    return SandboxBuilder;
}());
function sandboxOf(type, config) {
    return new SandboxBuilder(type, config);
}

/*
 * Public API Surface of playground
 */

/**
 * Generated bundle index. Do not edit.
 */

export { PlaygroundModule, SandboxBuilder, sandboxOf, ɵ0, PlaygroundCommonModule as ɵa, StateService as ɵb, UrlService as ɵc, LevenshteinDistance as ɵd, AppComponent as ɵe, MIDDLEWARE as ɵg, ScenarioComponent as ɵh, FocusDirective as ɵi, HighlightSearchMatchPipe as ɵj, LogoComponent as ɵk, PinComponent as ɵl, DrawerComponent as ɵm };
//# sourceMappingURL=angular-playground.js.map
